@@ -29,7 +29,7 @@
 
 #if 0
 static const char *version =
-"cardinfo.c 1.20 1997/05/12 16:19:23 (David Hinds)";
+"cardinfo.c 1.24 1998/01/09 04:18:49 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -95,7 +95,9 @@ static void print_status(status_t *status)
 {
     printf("  Function %d:\n", status->Function);
     printf("  ");
-    if (status->CardState & CS_EVENT_CARD_DETECT)
+    if (status->CardState & CS_EVENT_CB_DETECT)
+	printf("cardbus card present");
+    else if (status->CardState & CS_EVENT_CARD_DETECT)
 	printf("card present");
     else {
 	printf("no card\n");
@@ -127,10 +129,17 @@ static void print_config(config_info_t *config)
 	       config->Vcc/10.0, config->Vpp1/10.0, config->Vpp2/10.0);
 	if (!(config->Attributes & CONF_VALID_CLIENT))
 	    return;
-	printf("  Card type is %s\n", (config->IntType == INT_MEMORY)
-	       ? "memory-only" : "memory and I/O");
+	printf("  Card type is ");
+	switch (config->IntType) {
+	case INT_MEMORY:
+	    printf("memory-only\n"); break;
+	case INT_MEMORY_AND_IO:
+	    printf("memory and and I/O\n"); break;
+	case INT_CARDBUS:
+	    printf("cardbus\n"); break;
+	}
 	if (config->AssignedIRQ != 0) {
-	    printf("  IRQ %ld is ", config->AssignedIRQ);
+	    printf("  IRQ %d is ", config->AssignedIRQ);
 	    switch (config->IRQAttributes & IRQ_TYPE) {
 	    case IRQ_TYPE_EXCLUSIVE:
 		printf("exclusive"); break;
@@ -157,10 +166,10 @@ static void print_config(config_info_t *config)
 	return;
     
     printf("  Function %d:\n", config->Function);
-    
-    printf("    Config register base = %#06lx\n", config->ConfigBase);
-    
+
     if (config->CardValues) {
+	printf("    Config register base = %#06x\n",
+	       config->ConfigBase);
 	printf("    ");
 	if (config->CardValues & CV_OPTION_VALUE)
 	    printf("  Option = %#04x", config->Option);
@@ -450,7 +459,7 @@ void usage(char *name)
 
 #define MAX_SOCKS 8
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int major, cmd, fd[MAX_SOCKS], ns, ret, i;
     int optch, errflg;
@@ -543,4 +552,5 @@ void main(int argc, char *argv[])
 	exit(EXIT_FAILURE);
     else
 	exit(EXIT_SUCCESS);
+    return 0;
 }
