@@ -2,7 +2,7 @@
 
     PCMCIA Card Manager
 
-    Written by David Hinds, dhinds@allegro.stanford.edu
+    Written by David Hinds, dhinds@hyper.stanford.edu
     
 ======================================================================*/
 
@@ -47,11 +47,11 @@
 /*====================================================================*/
 
 typedef struct socket_info_t {
-    int fd;
-    int state;
-    card_info_t *card;
-    bind_info_t *bind[MAX_FUNCTIONS];
-    char *mtd[2*CISTPL_MAX_DEVICES];
+    int			fd;
+    int			state;
+    card_info_t		*card;
+    bind_info_t		*bind[MAX_FUNCTIONS];
+    char		*mtd[2*CISTPL_MAX_DEVICES];
 } socket_info_t;
 
 #define SOCKET_PRESENT	0x01
@@ -131,7 +131,7 @@ static int lookup_dev(char *name)
 	return n;
     else
 	return -ENODEV;
-} /* lookup_dev */
+}
 
 /*====================================================================*/
 
@@ -147,7 +147,7 @@ int open_dev(dev_t dev, int mode)
     fd = open(fn, (mode & S_IWRITE) ? O_RDWR: O_RDONLY);
     unlink(fn);
     return fd;
-} /* open_dev */
+}
 
 /*======================================================================
 
@@ -196,7 +196,7 @@ static int xlate_scsi_name(bind_info_t *bind)
 	close(fd);
     }
     return -1;
-} /* xlate_scsi_name */
+}
 #endif
 
 /*====================================================================*/
@@ -206,7 +206,7 @@ static int xlate_scsi_name(bind_info_t *bind)
 #define BEEP_WARN 2000
 #define BEEP_ERR  4000
 
-void beep(unsigned int ms, unsigned int freq)
+static void beep(unsigned int ms, unsigned int freq)
 {
     int fd, arg;
 
@@ -223,7 +223,19 @@ void beep(unsigned int ms, unsigned int freq)
 
 /*====================================================================*/
 
-void write_stab(void)
+static void write_pid(void)
+{
+    FILE *f;
+    f = fopen(pidfile, "w");
+    if (f == NULL)
+	syslog(LOG_INFO, "could not open %s: %m", pidfile);
+    else {
+	fprintf(f, "%d\n", getpid());
+	fclose(f);
+    }
+}
+
+static void write_stab(void)
 {
     int i, j, k;
     FILE *f;
@@ -265,7 +277,7 @@ void write_stab(void)
     fflush(f);
     flock(fileno(f), LOCK_UN);
     fclose(f);
-} /* write_stab */
+}
 
 /*====================================================================*/
 
@@ -320,36 +332,36 @@ static card_info_t *lookup_card(int ns)
 	    case VERS_1_IDENT:
 		if (vers == NULL)
 		    break;
-		for (i = 0; i < card->vers.ns; i++) {
-		    if (strcmp(card->vers.pi[i], "*") == 0)
+		for (i = 0; i < card->id.vers.ns; i++) {
+		    if (strcmp(card->id.vers.pi[i], "*") == 0)
 			continue;
 		    if (i >= vers->ns)
 			break;
-		    if (strcmp(card->vers.pi[i],
+		    if (strcmp(card->id.vers.pi[i],
 			       vers->str+vers->ofs[i]) != 0)
 			break;
 		}
-		if (i < card->vers.ns)
+		if (i < card->id.vers.ns)
 		    break;
 		match = 1;
 		break;
 
 	    case MANFID_IDENT:
-		if ((manfid.manf == card->manfid.manf) &&
-		    (manfid.card == card->manfid.card))
+		if ((manfid.manf == card->id.manfid.manf) &&
+		    (manfid.card == card->id.manfid.card))
 		    match = 1;
 		break;
 		
 	    case TUPLE_IDENT:
-		arg.tuple.DesiredTuple = card->tuple.code;
+		arg.tuple.DesiredTuple = card->id.tuple.code;
 		arg.tuple.Attributes = 0;
 		ret = ioctl(s->fd, DS_GET_FIRST_TUPLE, &arg);
 		if (ret != 0) break;
-		arg.tuple.TupleOffset = card->tuple.ofs;
+		arg.tuple.TupleOffset = card->id.tuple.ofs;
 		ret = ioctl(s->fd, DS_GET_TUPLE_DATA, &arg);
 		if (ret != 0) break;
-		if (strncmp(arg.tuple_parse.data, card->tuple.info,
-			    strlen(card->tuple.info)) != 0)
+		if (strncmp(arg.tuple_parse.data, card->id.tuple.info,
+			    strlen(card->id.tuple.info)) != 0)
 		    break;
 		match = 1;
 		break;
@@ -370,7 +382,7 @@ static card_info_t *lookup_card(int ns)
     /* Try for a FUNCID match */
     if (get_tuple(ns, CISTPL_FUNCID, &arg) == 0) {
 	for (card = root_func; card; card = card->next)
-	    if (card->func.funcid == arg.tuple_parse.parse.funcid.func)
+	    if (card->id.func.funcid == arg.tuple_parse.parse.funcid.func)
 		break;
 	if (card) {
 	    syslog(LOG_INFO, "socket %d: %s", ns, card->name);
@@ -402,7 +414,7 @@ static card_info_t *lookup_card(int ns)
 	beep(BEEP_TIME, BEEP_WARN);
 	return card;
     }
-} /* lookup_card */
+}
 
 /*====================================================================*/
 
@@ -416,7 +428,7 @@ static void load_config(void)
 	syslog(LOG_INFO, "no device drivers defined");
     if (root_card == NULL)
 	syslog(LOG_INFO, "no cards defined");
-} /* load_config */
+}
 
 /*====================================================================*/
 
@@ -426,11 +438,11 @@ static void free_card(card_info_t *card)
     free(card->name);
     switch(card->ident_type) {
     case VERS_1_IDENT:
-	for (i = 0; i < card->vers.ns; i++)
-	    free(card->vers.pi[i]);
+	for (i = 0; i < card->id.vers.ns; i++)
+	    free(card->id.vers.pi[i]);
 	break;
     case TUPLE_IDENT:
-	free(card->tuple.info);
+	free(card->id.tuple.info);
 	break;
     default:
 	break;
@@ -479,7 +491,7 @@ static void free_config(void)
 	free(mtd);
     }
     default_mtd = NULL;
-} /* free_config */
+}
 
 /*====================================================================*/
 
@@ -507,7 +519,7 @@ static int execute(char *msg, char *cmd)
 	syslog(LOG_INFO, "%s exited on signal %d",
 	       msg, WTERMSIG(ret));
     return -1;
-} /* execute */
+}
 
 /*====================================================================*/
 
@@ -543,9 +555,9 @@ static module_list_t *module_list = NULL;
 
 static int install_module(char *mod, char *opts)
 {
-    char path[128];
-    char cmd[128];
+    char path[128], cmd[128];
     module_list_t *ml;
+    int ret;
     
     for (ml = module_list; ml != NULL; ml = ml->next)
 	if (strcmp(mod, ml->mod) == 0) break;
@@ -557,28 +569,31 @@ static int install_module(char *mod, char *opts)
 	module_list = ml;
     }
     ml->usage++;
-    if (ml->usage == 1) {
-	if (do_modprobe) {
-	    sprintf(cmd, "modprobe %s", mod);
-	} else {
-	    if (strchr(mod, '/') != NULL)
-		sprintf(path, "%s/%s.o", modpath, mod);
-	    else
-		sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
-	    if (access(path, R_OK) != 0) {
-		syslog(LOG_INFO, "module %s not available", path);
-		return -1;
-	    }
-	    sprintf(cmd, "insmod %s", path);
+    if (ml->usage != 1)
+	return 0;
+    if (!do_modprobe) {
+	if (strchr(mod, '/') != NULL)
+	    sprintf(path, "%s/%s.o", modpath, mod);
+	else
+	    sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
+	if (access(path, R_OK) != 0) {
+	    syslog(LOG_INFO, "module %s not available", path);
+	    return -1;
 	}
+	sprintf(cmd, "insmod %s", path);
 	if (opts) {
 	    strcat(cmd, " ");
 	    strcat(cmd, opts);
 	}
-	return execute(do_modprobe ? "modprobe" : "insmod", cmd);
+	ret = execute("insmod", cmd);
+	if (ret == 0) return 0;
     }
-    else
-	return 0;
+    sprintf(cmd, "modprobe %s", mod);
+    if (opts) {
+	strcat(cmd, " ");
+	strcat(cmd, opts);
+    }
+    return execute("modprobe", cmd);
 }
 
 static void remove_module(char *mod)
@@ -629,7 +644,7 @@ static mtd_ident_t *lookup_mtd(region_info_t *region)
 	return mtd;
     else
 	return default_mtd;
-} /* lookup_mtd */
+}
 
 /*====================================================================*/
 
@@ -683,7 +698,7 @@ static void bind_mtd(int sn)
 	    syslog(LOG_INFO, "bind MTD '%s' to socket %d failed: %m",
 		   (char *)bind.dev_info, sn);
     }
-} /* bind_mtd */
+}
 
 /*====================================================================*/
 
@@ -725,7 +740,10 @@ static void do_insert(int sn)
     for (i = 0; i < card->functions; i++) {
 	bind = calloc(1, sizeof(bind_info_t));
 	strcpy((char *)bind->dev_info, (char *)dev[i]->dev_info);
-	bind->function = card->dev_fn[i];
+	if (strcmp(bind->dev_info, "cb_enabler") == 0)
+	    bind->function = BIND_FN_ALL;
+	else
+	    bind->function = card->dev_fn[i];
 	if (ioctl(s->fd, DS_BIND_REQUEST, bind) != 0) {
 	    if (errno == EBUSY) {
 		syslog(LOG_INFO, "'%s' already bound to socket %d",
@@ -778,7 +796,7 @@ static void do_insert(int sn)
     else
 	beep(BEEP_TIME, BEEP_OK);
     
-} /* do_insert */
+}
 
 /*====================================================================*/
 
@@ -904,6 +922,23 @@ static void do_resume(int sn)
 
 /*====================================================================*/
 
+static void wait_for_pending(void)
+{
+    status_t status;
+    int i;
+    status.Function = 0;
+    for (;;) {
+	usleep(100000);
+	for (i = 0; i < sockets; i++)
+	    if ((ioctl(socket[i].fd, DS_GET_STATUS, &status) == 0) &&
+		(status.CardState & CS_EVENT_CARD_INSERTION))
+		break;
+	if (i == sockets) break;
+    }
+}
+
+/*====================================================================*/
+
 static void free_resources(void)
 {
     adjust_list_t *al;
@@ -921,7 +956,7 @@ static void free_resources(void)
 	}
     }
     
-} /* free_resources */
+}
 
 /*====================================================================*/
 
@@ -949,13 +984,13 @@ static void adjust_resources(void)
 			al->adj.resource.io.NumPorts - 1);
 		break;
 	    case RES_IRQ:
-		sprintf(tmp, "irq %lu", al->adj.resource.irq.IRQ);
+		sprintf(tmp, "irq %u", al->adj.resource.irq.IRQ);
 		break;
 	    }
 	    syslog(LOG_INFO, "could not adjust resource: %s: %m", tmp);
 	}
     }
-} /* adjust_resources */
+}
     
 /*====================================================================*/
 
@@ -964,12 +999,16 @@ static int cleanup_files = 0;
 static void fork_now(void)
 {
     int ret;
-    if ((ret = fork()) > 0) exit(0);
+    if ((ret = fork()) > 0) {
+	cleanup_files = 0;
+	exit(0);
+    }
     if (ret == -1)
 	syslog(LOG_INFO, "forking: %m");
     if (setsid() < 0)
 	syslog(LOG_INFO, "detaching from tty: %m");
-}
+    write_pid();
+}    
 
 static void done(void)
 {
@@ -1046,17 +1085,13 @@ static int init_sockets(void)
     if (sockets == 0) {
 	syslog(LOG_INFO, "no sockets found!");
 	return -1;
-    }
-    else
+    } else
 	syslog(LOG_INFO, "watching %d sockets", sockets);
 
     if (ioctl(socket[0].fd, DS_GET_CARD_SERVICES_INFO, &serv) == 0) {
-	if (serv.Revision != CS_RELEASE_CODE) {
+	if (serv.Revision != CS_RELEASE_CODE)
 	    syslog(LOG_INFO, "Card Services release does not match!");
-	    return -1;
-	}
-    }
-    else {
+    } else {
 	syslog(LOG_INFO, "could not get CS revision info!");
 	return -1;
     }
@@ -1066,7 +1101,7 @@ static int init_sockets(void)
 
 /*====================================================================*/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int optch, errflg;
     int i, max_fd, ret, event, pass;
@@ -1074,7 +1109,6 @@ void main(int argc, char *argv[])
     struct timeval tv;
     extern char *optarg;
     fd_set fds;
-    FILE *f;
 
     errflg = 0;
     while ((optch = getopt(argc, argv, "qdvofc:m:p:s:")) != -1) {
@@ -1145,16 +1179,10 @@ void main(int argc, char *argv[])
     if (init_sockets() != 0)
 	exit(EXIT_FAILURE);
 
-    f = fopen(pidfile, "w");
-    if (f == NULL)
-	syslog(LOG_INFO, "could not open %s: %m", pidfile);
-    else {
-	fprintf(f, "%d\n", getpid());
-	fclose(f);
-    }
-    /* If we've gotten this far, then clean up pid and stab on exit */
+    /* If we've gotten this far, then clean up pid and stab at exit */
+    write_stab();
     cleanup_files = 1;
-
+    
     if (signal(SIGHUP, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(SIGHUP): %m");
     if (signal(SIGTERM, catch_signal) == SIG_ERR)
@@ -1169,6 +1197,10 @@ void main(int argc, char *argv[])
 
     /* First select() call: poll, don't wait */
     tv.tv_sec = tv.tv_usec = 0;
+
+    /* Wait for sockets in setup-pending state to settle */
+    if (one_pass || delay_fork)
+	wait_for_pending();
     
     for (pass = 0; ; pass++) {
 	FD_ZERO(&fds);
@@ -1233,4 +1265,5 @@ void main(int argc, char *argv[])
 	    fork_now();
 	
     } /* repeat */
+    return 0;
 }
