@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    ide_info.c 1.2 1996/08/30 15:18:56
+    ide_info.c 1.3 1997/12/18 18:24:03
     
 ======================================================================*/
 
@@ -22,7 +22,7 @@
 
 /*====================================================================*/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int fd;
     struct hd_driveid id;
@@ -46,4 +46,5 @@ void main(int argc, char *argv[])
     printf("FW_REV=\"%s\"\n", id.fw_rev);
     printf("SERIAL_NO=\"%s\"\n", id.serial_no);
     exit(0);
+    return 0;
 }
