@@ -14,7 +14,11 @@
 #include <errno.h>
 #include <fcntl.h>
 
+#ifdef __GLIBC__
+#include <sys/io.h>
+#else
 #include <asm/io.h>
+#endif
 
 #include <pcmcia/config.h>
 #include "i82365.h"
@@ -23,10 +27,12 @@
 #include "tcic.h"
 
 static const char *version =
-"probe.c 1.21 1997/03/21 07:25:23 (David Hinds)\n";
+"probe.c 1.26 1998/02/15 22:08:42 (David Hinds)\n";
 
 static int i365_base = 0x03e0;
 
+typedef u_short ioaddr_t;
+
 /*====================================================================*/
 
 #ifdef CONFIG_PCI
@@ -39,10 +45,13 @@ typedef struct {
 pci_id_t pci_id[] = {
     { "Cirrus Logic CL 6729", "Cirrus PD6729" },
     { "Vendor id=119b. Device id=1221.", "Omega Micro 82C092G" },
-    { "Vendor id=1180. Device id=466.", "Ricoh RL5C466" },
+    { "Vendor id=8086. Device id=1221.", "Intel 82092AA" },
+    { "Vendor id=1180. Device id=466.",  "Ricoh RL5C466" },
     { "Vendor id=10b3. Device id=b106.", "SMC 34C90" },
     { "Vendor id=1013. Device id=1110.", "Cirrus PD6832" },
-    { "Vendor id=104c. Device id=ac12.", "TI 1131" }
+    { "Vendor id=104c. Device id=ac12.", "TI 1130" },
+    { "Vendor id=104c. Device id=ac15.", "TI 1131" },
+    { "Vendor id=1217. Device id=6832.", "O2Micro OZ6832" }
 };
 #define PCI_COUNT (sizeof(pci_id)/sizeof(pci_id_t))
 
@@ -64,6 +73,14 @@ static int pci_probe(int verbose, int module)
 	if (i != PCI_COUNT) {
 	    name = pci_id[i].name;
 	    sock = 2;
+	} else {
+	    char *s = strstr(buf, "CardBus bridge");
+	    if (s != NULL) {
+		name = s + 16;
+		s = strchr(s, '(');
+		s[-1] = '\0';
+		sock = 2;
+	    }
 	}
 	close(fd);
     }
@@ -198,43 +215,43 @@ int i365_probe(int verbose, int module)
   
 /*====================================================================*/
 
-static u_char tcic_getb(u_long base, u_char reg)
+static u_char tcic_getb(ioaddr_t base, u_char reg)
 {
     u_char val = inb(base+reg);
     return val;
 }
 
-static void tcic_setb(u_long base, u_char reg, u_char data)
+static void tcic_setb(ioaddr_t base, u_char reg, u_char data)
 {
     outb(data, base+reg);
 }
 
-static u_short tcic_getw(u_long base, u_char reg)
+static u_short tcic_getw(ioaddr_t base, u_char reg)
 {
     u_short val = inw(base+reg);
     return val;
 }
 
-static void tcic_setw(u_long base, u_char reg, u_short data)
+static void tcic_setw(ioaddr_t base, u_char reg, u_short data)
 {
     outw(data, base+reg);
 }
 
-static u_short tcic_aux_getw(u_long base, u_short reg)
+static u_short tcic_aux_getw(ioaddr_t base, u_short reg)
 {
     u_char mode = (tcic_getb(base, TCIC_MODE) & TCIC_MODE_PGMMASK) | reg;
     tcic_setb(base, TCIC_MODE, mode);
     return tcic_getw(base, TCIC_AUX);
 }
 
-static void tcic_aux_setw(u_long base, u_short reg, u_short data)
+static void tcic_aux_setw(ioaddr_t base, u_short reg, u_short data)
 {
     u_char mode = (tcic_getb(base, TCIC_MODE) & TCIC_MODE_PGMMASK) | reg;
     tcic_setb(base, TCIC_MODE, mode);
     tcic_setw(base, TCIC_AUX, data);
 }
 
-static int get_tcic_id(u_long base)
+static int get_tcic_id(ioaddr_t base)
 {
     u_short id;
     tcic_aux_setw(base, TCIC_AUX_TEST, TCIC_TEST_DIAG);
@@ -244,7 +261,7 @@ static int get_tcic_id(u_long base)
     return id;
 }
 
-int tcic_probe_at(u_long base, int module)
+int tcic_probe_at(ioaddr_t base, int module)
 {
     int i;
     u_short old;
@@ -255,7 +272,7 @@ int tcic_probe_at(u_long base, int module)
 	    return -1;
 
     if (!module)
-	printf("  at %#3.3lx: ", base); fflush(stdout);
+	printf("  at %#3.3x: ", base); fflush(stdout);
 
     /* Try to reset the chip */
     tcic_setw(base, TCIC_SCTRL, TCIC_SCTRL_RESET);
@@ -276,7 +293,7 @@ int tcic_probe_at(u_long base, int module)
     return 2;
 }
 
-int tcic_probe(int verbose, int module, u_long base)
+int tcic_probe(int verbose, int module, ioaddr_t base)
 {
     int sock, id;
 
@@ -315,7 +332,7 @@ int tcic_probe(int verbose, int module, u_long base)
 	default:
 	    printf("Unknown TCIC-2 ID 0x%02x", id);
 	}
-	printf(" found at %#6lx, %d sockets.\n", base, sock);
+	printf(" found at %#6x, %d sockets.\n", base, sock);
     }
     return 0;
     
@@ -323,13 +340,12 @@ int tcic_probe(int verbose, int module, u_long base)
 
 /*====================================================================*/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int optch, errflg;
     extern char *optarg;
     int verbose = 0, module = 0;
-    u_long tcic_base = TCIC_BASE;
-
+    ioaddr_t tcic_base = TCIC_BASE;
     
     errflg = 0;
     while ((optch = getopt(argc, argv, "t:vxm")) != -1) {
@@ -362,4 +378,5 @@ void main(int argc, char *argv[])
 	exit(EXIT_SUCCESS);
     else
 	exit(EXIT_FAILURE);
+    return 0;
 }
