@@ -10,7 +10,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define YYPREFIX "yy"
 #line 2 "yacc_config.y"
 /*
- * yacc_config.y 1.30 1997/04/15 16:00:00 (David Hinds)\n"
+ * yacc_config.y 1.35 1998/01/02 16:49:56 (David Hinds)\n"
  */
     
 #include <stdlib.h>
@@ -37,7 +37,7 @@ static int add_module(device_info_t *card, char *name);
 #line 35 "yacc_config.y"
 typedef union {
     char *str;
-    long num;
+    u_long num;
     struct device_info_t *device;
     struct card_info_t *card;
     struct mtd_ident_t *mtd;
@@ -309,7 +309,7 @@ YYSTYPE yylval;
 short yyss[YYSTACKSIZE];
 YYSTYPE yyvs[YYSTACKSIZE];
 #define yystacksize YYSTACKSIZE
-#line 391 "yacc_config.y"
+#line 390 "yacc_config.y"
 void yyerror(char *msg, ...)
 {
      va_list ap;
@@ -556,29 +556,28 @@ case 5:
 		    if (yyvsp[0].card->ident_type == FUNC_IDENT) {
 			yyvsp[0].card->next = root_func;
 			root_func = yyvsp[0].card;
-		    }
-		    else {
+		    } else {
 			yyvsp[0].card->next = root_card;
 			root_card = yyvsp[0].card;
 		    }
 		}
 break;
 case 8:
-#line 101 "yacc_config.y"
+#line 100 "yacc_config.y"
 {
 		    yyvsp[0].adjust->adj.Action = ADD_MANAGED_RESOURCE;
 		    yyval.adjust = yyvsp[0].adjust;
 		}
 break;
 case 9:
-#line 106 "yacc_config.y"
+#line 105 "yacc_config.y"
 {
 		    yyvsp[0].adjust->adj.Action = REMOVE_MANAGED_RESOURCE;
 		    yyval.adjust = yyvsp[0].adjust;
 		}
 break;
 case 10:
-#line 111 "yacc_config.y"
+#line 110 "yacc_config.y"
 {
 		    yyvsp[0].adjust->adj.Action = ADD_MANAGED_RESOURCE;
 		    yyvsp[0].adjust->adj.Attributes |= RES_RESERVED;
@@ -586,7 +585,7 @@ case 10:
 		}
 break;
 case 11:
-#line 117 "yacc_config.y"
+#line 116 "yacc_config.y"
 {
 		    yyvsp[0].adjust->adj.Action = yyvsp[-2].adjust->adj.Action;
 		    yyvsp[0].adjust->adj.Attributes = yyvsp[-2].adjust->adj.Attributes;
@@ -595,7 +594,7 @@ case 11:
 		}
 break;
 case 12:
-#line 126 "yacc_config.y"
+#line 125 "yacc_config.y"
 {
 		    yyval.adjust = calloc(sizeof(adjust_list_t), 1);
 		    yyval.adjust->adj.Resource = RES_IRQ;
@@ -603,7 +602,7 @@ case 12:
 		}
 break;
 case 13:
-#line 132 "yacc_config.y"
+#line 131 "yacc_config.y"
 {
 		    if ((yyvsp[0].num < yyvsp[-2].num) || (yyvsp[0].num > 0xffff)) {
 			yyerror("invalid port range");
@@ -616,7 +615,7 @@ case 13:
 		}
 break;
 case 14:
-#line 143 "yacc_config.y"
+#line 142 "yacc_config.y"
 {
 		    if (yyvsp[0].num < yyvsp[-2].num) {
 			yyerror("invalid address range");
@@ -629,7 +628,7 @@ case 14:
 		}
 break;
 case 15:
-#line 156 "yacc_config.y"
+#line 155 "yacc_config.y"
 {
 		    yyval.device = calloc(sizeof(device_info_t), 1);
 		    strcpy(yyval.device->dev_info, yyvsp[0].str);
@@ -637,14 +636,14 @@ case 15:
 		}
 break;
 case 19:
-#line 167 "yacc_config.y"
+#line 166 "yacc_config.y"
 {
 		    yyval.card = calloc(sizeof(card_info_t), 1);
 		    yyval.card->name = yyvsp[0].str;
 		}
 break;
 case 26:
-#line 180 "yacc_config.y"
+#line 179 "yacc_config.y"
 {
 		    if (yyvsp[-1].card->ident_type != 0) {
 			yyerror("ID method already defined");
@@ -659,100 +658,100 @@ case 26:
 		}
 break;
 case 27:
-#line 195 "yacc_config.y"
+#line 194 "yacc_config.y"
 {
 		    if (yyvsp[-6].card->ident_type != 0) {
 			yyerror("ID method already defined");
 			YYERROR;
 		    }
 		    yyvsp[-6].card->ident_type = TUPLE_IDENT;
-		    yyvsp[-6].card->tuple.code = yyvsp[-4].num;
-		    yyvsp[-6].card->tuple.ofs = yyvsp[-2].num;
-		    yyvsp[-6].card->tuple.info = yyvsp[0].str;
+		    yyvsp[-6].card->id.tuple.code = yyvsp[-4].num;
+		    yyvsp[-6].card->id.tuple.ofs = yyvsp[-2].num;
+		    yyvsp[-6].card->id.tuple.info = yyvsp[0].str;
 		}
 break;
 case 28:
-#line 208 "yacc_config.y"
+#line 207 "yacc_config.y"
 {
 		    if (yyvsp[-4].card->ident_type != 0) {
 			yyerror("ID method already defined");
 			YYERROR;
 		    }
 		    yyvsp[-4].card->ident_type = MANFID_IDENT;
-		    yyvsp[-4].card->manfid.manf = yyvsp[-2].num;
-		    yyvsp[-4].card->manfid.card = yyvsp[0].num;
+		    yyvsp[-4].card->id.manfid.manf = yyvsp[-2].num;
+		    yyvsp[-4].card->id.manfid.card = yyvsp[0].num;
 		}
 break;
 case 29:
-#line 219 "yacc_config.y"
+#line 218 "yacc_config.y"
 {
 		    if (yyvsp[-2].card->ident_type != 0) {
 			yyerror("ID method already defined\n");
 			YYERROR;
 		    }
 		    yyvsp[-2].card->ident_type = VERS_1_IDENT;
-		    yyvsp[-2].card->vers.ns = 1;
-		    yyvsp[-2].card->vers.pi[0] = yyvsp[0].str;
+		    yyvsp[-2].card->id.vers.ns = 1;
+		    yyvsp[-2].card->id.vers.pi[0] = yyvsp[0].str;
 		}
 break;
 case 30:
-#line 229 "yacc_config.y"
+#line 228 "yacc_config.y"
 {
-		    if (yyvsp[-2].card->vers.ns == 4) {
+		    if (yyvsp[-2].card->id.vers.ns == 4) {
 			yyerror("too many version strings");
 			YYERROR;
 		    }
-		    yyvsp[-2].card->vers.pi[yyvsp[-2].card->vers.ns] = yyvsp[0].str;
-		    yyvsp[-2].card->vers.ns++;
+		    yyvsp[-2].card->id.vers.pi[yyvsp[-2].card->id.vers.ns] = yyvsp[0].str;
+		    yyvsp[-2].card->id.vers.ns++;
 		}
 break;
 case 31:
-#line 240 "yacc_config.y"
+#line 239 "yacc_config.y"
 {
 		    if (yyvsp[-2].card->ident_type != 0) {
 			yyerror("ID method already defined\n");
 			YYERROR;
 		    }
 		    yyvsp[-2].card->ident_type = FUNC_IDENT;
-		    yyvsp[-2].card->func.funcid = yyvsp[0].num;
+		    yyvsp[-2].card->id.func.funcid = yyvsp[0].num;
 		}
 break;
 case 32:
-#line 251 "yacc_config.y"
+#line 250 "yacc_config.y"
 {
 		    if (add_binding(yyvsp[-2].card, yyvsp[0].str, 0) != 0)
 			YYERROR;
 		}
 break;
 case 33:
-#line 256 "yacc_config.y"
+#line 255 "yacc_config.y"
 {
 		    if (add_binding(yyvsp[-4].card, yyvsp[-2].str, yyvsp[0].num) != 0)
 			YYERROR;
 		}
 break;
 case 34:
-#line 261 "yacc_config.y"
+#line 260 "yacc_config.y"
 {
 		    if (add_binding(yyvsp[-2].card, yyvsp[0].str, 0) != 0)
 			YYERROR;
 		}
 break;
 case 35:
-#line 266 "yacc_config.y"
+#line 265 "yacc_config.y"
 {
 		    if (add_binding(yyvsp[-4].card, yyvsp[-2].str, yyvsp[0].num) != 0)
 			YYERROR;
 		}
 break;
 case 36:
-#line 273 "yacc_config.y"
+#line 272 "yacc_config.y"
 {
 		    yyvsp[-1].device->needs_mtd = 1;
 		}
 break;
 case 37:
-#line 279 "yacc_config.y"
+#line 278 "yacc_config.y"
 {
 		    device_info_t *d;
 		    int i, found = 0;
@@ -774,14 +773,14 @@ case 37:
 		}
 break;
 case 38:
-#line 301 "yacc_config.y"
+#line 300 "yacc_config.y"
 {
 		    if (add_module(yyvsp[-2].device, yyvsp[0].str) != 0)
 			YYERROR;
 		}
 break;
 case 39:
-#line 306 "yacc_config.y"
+#line 305 "yacc_config.y"
 {
 		    if (yyvsp[-2].device->opts[yyvsp[-2].device->modules-1] == NULL)
 			yyvsp[-2].device->opts[yyvsp[-2].device->modules-1] = yyvsp[0].str;
@@ -792,14 +791,14 @@ case 39:
 		}
 break;
 case 40:
-#line 315 "yacc_config.y"
+#line 314 "yacc_config.y"
 {
 		    if (add_module(yyvsp[-2].device, yyvsp[0].str) != 0)
 			YYERROR;
 		}
 break;
 case 41:
-#line 322 "yacc_config.y"
+#line 321 "yacc_config.y"
 {
 		    if (yyvsp[-2].device->class != NULL) {
 			yyerror("extra class string");
@@ -809,14 +808,14 @@ case 41:
 		}
 break;
 case 42:
-#line 332 "yacc_config.y"
+#line 331 "yacc_config.y"
 {
 		    yyval.mtd = calloc(sizeof(mtd_ident_t), 1);
 		    yyval.mtd->name = yyvsp[0].str;
 		}
 break;
 case 47:
-#line 343 "yacc_config.y"
+#line 342 "yacc_config.y"
 {
 		    if (yyvsp[-2].mtd->mtd_type != 0) {
 			yyerror("ID method already defined");
@@ -827,7 +826,7 @@ case 47:
 		}
 break;
 case 48:
-#line 354 "yacc_config.y"
+#line 353 "yacc_config.y"
 {
 		    if (yyvsp[-3].mtd->mtd_type != 0) {
 			yyerror("ID method already defined");
@@ -839,7 +838,7 @@ case 48:
 		}
 break;
 case 49:
-#line 366 "yacc_config.y"
+#line 365 "yacc_config.y"
 {
 		    if (yyvsp[-1].mtd->mtd_type != 0) {
 			yyerror("ID method already defined");
@@ -854,7 +853,7 @@ case 49:
 		}
 break;
 case 50:
-#line 381 "yacc_config.y"
+#line 380 "yacc_config.y"
 {
 		    if (yyvsp[-2].mtd->module != NULL) {
 			yyerror("extra MTD entry");
@@ -863,7 +862,7 @@ case 50:
 		    yyvsp[-2].mtd->module = yyvsp[0].str;
 		}
 break;
-#line 867 "y.tab.c"
+#line 866 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
