@@ -21,7 +21,11 @@ typedef unsigned short u_short;
 #else /* __MSDOS__ */
 
 #include <sys/types.h>
+#ifdef __GLIBC__
+#include <sys/io.h>
+#else
 #include <asm/io.h>
+#endif
 #include <unistd.h>
 #define INB(a) inb_p(a)
 #define OUTB(d, a) outb_p(d, a)
@@ -33,7 +37,7 @@ typedef unsigned short u_short;
 #include "vg468.h"
 
 static const char *version =
-"dump_i365.c 1.15 1997/03/21 07:22:47 (David Hinds)\n";
+"dump_i365.c 1.16 1998/01/17 05:13:26 (David Hinds)\n";
 
 static int i365_base = 0x03e0;
 static int type = 0;
