@@ -23,11 +23,11 @@
 #include <pcmcia/ds.h>
 
 static const char *version =
-    "dump_tuples.c v1.23: 1996/11/26 07:32:27 (David Hinds)\n";
+"dump_tuples.c 1.25 1998/01/02 02:06:36 (David Hinds)\n";
 
 /*====================================================================*/
 
-int lookup_dev(char *name)
+static int lookup_dev(char *name)
 {
     FILE *f;
     int n;
@@ -46,11 +46,11 @@ int lookup_dev(char *name)
 	return n;
     else
 	return -1;
-} /* lookup_dev */
+}
 
 /*====================================================================*/
 
-int open_dev(dev_t dev)
+static int open_dev(dev_t dev)
 {
     char *fn;
     int fd;
@@ -62,14 +62,14 @@ int open_dev(dev_t dev)
     fd = open(fn, O_RDONLY);
     unlink(fn);
     return fd;
-} /* open_dev */
+}
 
 /*====================================================================*/
 
-void print_tuple(tuple_parse_t *tup)
+static void print_tuple(tuple_parse_t *tup)
 {
     int i;
-    printf("  Offset: 0x%2.2lx, tuple: %#2.2x, link: %#2.2x\n",
+    printf("  Offset: 0x%2.2x, tuple: %#2.2x, link: %#2.2x\n",
 	   tup->tuple.CISOffset, tup->tuple.TupleCode,
 	   tup->tuple.TupleLink);
     for (i = 0; i < tup->tuple.TupleDataLen; i++) {
@@ -78,11 +78,11 @@ void print_tuple(tuple_parse_t *tup)
 	if ((i % 16) == 15) printf("\n");
     }
     if ((i % 16) != 0) printf("\n");
-} /* print_tuple */
+}
 
 /*====================================================================*/
 
-void print_funcid(cistpl_funcid_t *fn)
+static void print_funcid(cistpl_funcid_t *fn)
 {
     printf("  Function: ");
     switch (fn->func) {
@@ -108,25 +108,25 @@ void print_funcid(cistpl_funcid_t *fn)
 	printf("unknown"); break;
     }
     if (fn->sysinit & CISTPL_SYSINIT_POST)
-	printf(" [POST]");
+	printf(" [post]");
     if (fn->sysinit & CISTPL_SYSINIT_ROM)
-	printf(" [ROM]");
+	printf(" [rom]");
     printf("\n");
-} /* print_funcid */
+}
 
 /*====================================================================*/
 
-void print_size(u_long size)
+static void print_size(u_int size)
 {
     if (size < 1024)
-	printf("%lu bytes", size);
+	printf("%u bytes", size);
     else if (size < 1024*1024)
-	printf("%lu kb", size/1024);
+	printf("%u kb", size/1024);
     else
-	printf("%lu mb", size/(1024*1024));
+	printf("%u mb", size/(1024*1024));
 }
 
-void print_time(u_long tm, u_long scale)
+static void print_time(u_int tm, u_long scale)
 {
     float t = (float)(tm) * scale * 0.000000001;
     if (t > 1.0)
@@ -139,7 +139,7 @@ void print_time(u_long tm, u_long scale)
 	printf("%.0f ns", t*1000000000.0);
 }
 
-void print_volt(u_long vi)
+static void print_volt(u_long vi)
 {
     float v = vi * 0.00001;
     if (v > 1.0)
@@ -150,7 +150,7 @@ void print_volt(u_long vi)
 	printf("%.0f uV", v*1000000.0);
 }
     
-void print_current(u_long ii)
+static void print_current(u_long ii)
 {
     float i = ii * 0.0000001;
     if (i > 1.0)
@@ -161,14 +161,14 @@ void print_current(u_long ii)
 	printf("%.0f uA", i*1000000.0);
 }
 
-void print_speed(u_long b)
+static void print_speed(u_int b)
 {
     if (b < 1024)
-	printf("%ld bits/sec", b);
+	printf("%u bits/sec", b);
     else if (b < 1024*1024)
-	printf("%ld kbits/sec", b/1024);
+	printf("%u kbits/sec", b/1024);
     else
-	printf("%ld mbits/sec", b/(1024*1024));
+	printf("%u mbits/sec", b/(1024*1024));
 }
 
 /*====================================================================*/
@@ -179,24 +179,25 @@ static const char *dtype[] = {
     "Extended", "rsvd"
 };
 
-void print_device(cistpl_device_t *dev)
+static void print_device(cistpl_device_t *dev)
 {
     int i;
     for (i = 0; i < dev->ndev; i++) {
 	printf("    %s: ", dtype[dev->dev[i].type]);
-	printf("%ld ns, ", dev->dev[i].speed);
+	printf("%u ns, ", dev->dev[i].speed);
 	print_size(dev->dev[i].size);
 	printf("\n");
     }
     if (dev->ndev == 0)
 	printf("    No device info.\n");
-} /* print_device */
+}
 
 /*====================================================================*/
 
-void print_power(cistpl_power_t *power)
+static void print_power(char *tag, cistpl_power_t *power)
 {
     int first = 1;
+    printf("  %s: ", tag);
     if (power->present & (1<<CISTPL_POWER_VNOM)) {
 	first = 0;
 	printf(" Vnom ");
@@ -242,11 +243,11 @@ void print_power(cistpl_power_t *power)
 	printf(" highz");
     }
     printf("\n");
-} /* print_power */
+}
 
 /*====================================================================*/
 
-void print_cftable_entry(cistpl_cftable_entry_t *entry)
+static void print_cftable_entry(cistpl_cftable_entry_t *entry)
 {
     int i;
     
@@ -256,34 +257,28 @@ void print_cftable_entry(cistpl_cftable_entry_t *entry)
     if (entry->flags & ~CISTPL_CFTABLE_DEFAULT) {
 	printf(" ");
 	if (entry->flags & CISTPL_CFTABLE_BVDS)
-	    printf(" [BVD]");
+	    printf(" [bvd]");
 	if (entry->flags & CISTPL_CFTABLE_WP)
-	    printf(" [WP]");
+	    printf(" [wp]");
 	if (entry->flags & CISTPL_CFTABLE_RDYBSY)
-	    printf(" [RDYBSY]");
+	    printf(" [rdybsy]");
 	if (entry->flags & CISTPL_CFTABLE_MWAIT)
-	    printf(" [MWAIT]");
+	    printf(" [mwait]");
 	if (entry->flags & CISTPL_CFTABLE_AUDIO)
-	    printf(" [AUDIO]");
+	    printf(" [audio]");
 	if (entry->flags & CISTPL_CFTABLE_READONLY)
-	    printf(" [READONLY]");
+	    printf(" [readonly]");
 	if (entry->flags & CISTPL_CFTABLE_PWRDOWN)
-	    printf(" [PWRDOWN]");
+	    printf(" [pwrdown]");
 	printf("\n");
     }
     
-    if (entry->vcc.present) {
-	printf("  Vcc: ");
-	print_power(&entry->vcc);
-    }
-    if (entry->vpp1.present) {
-	printf("  Vpp1:");
-	print_power(&entry->vpp1);
-    }
-    if (entry->vpp2.present) {
-	printf("  Vpp2:");
-	print_power(&entry->vpp2);
-    }
+    if (entry->vcc.present)
+	print_power("Vcc", &entry->vcc);
+    if (entry->vpp1.present)
+	print_power("Vpp1", &entry->vpp1);
+    if (entry->vpp2.present)
+	print_power("Vpp2", &entry->vpp2);
 
     if ((entry->timing.wait != 0) || (entry->timing.ready != 0) ||
 	(entry->timing.reserved != 0)) {
@@ -312,16 +307,16 @@ void print_cftable_entry(cistpl_cftable_entry_t *entry)
 	if (io->flags & CISTPL_IO_RANGE) printf(" [range]");
 	printf("\n");
 	for (i = 0; i < io->nwin; i++)
-	    printf("    base = %#4.4lx, length = %#4.4lx\n",
+	    printf("    base = %#4.4x, length = %#4.4x\n",
 		   io->win[i].base, io->win[i].len);
     }
 
     if (entry->irq.IRQInfo1) {
 	printf("  Interrupt ");
 	if (entry->irq.IRQInfo1 & IRQ_INFO2_VALID)
-	    printf("mask = %#4.4lx", entry->irq.IRQInfo2);
+	    printf("mask = 0x%#4.4x", entry->irq.IRQInfo2);
 	else
-	    printf("%ld", entry->irq.IRQInfo1 & IRQ_MASK);
+	    printf("%u", entry->irq.IRQInfo1 & IRQ_MASK);
 	if (entry->irq.IRQInfo1 & IRQ_LEVEL_ID) printf(" [level]");
 	if (entry->irq.IRQInfo1 & IRQ_PULSE_ID) printf(" [pulse]");
 	if (entry->irq.IRQInfo1 & IRQ_SHARE_ID) printf(" [shared]");
@@ -332,44 +327,114 @@ void print_cftable_entry(cistpl_cftable_entry_t *entry)
 	cistpl_mem_t *mem = &entry->mem;
 	printf("  Memory windows:\n");
 	for (i = 0; i < mem->nwin; i++)
-	    printf("    card = 0x%4.4lx, host = 0x%4.4lx, length = "
-		   "0x%4.4lx\n", mem->win[i].card_addr,
+	    printf("    card = 0x%4.4x, host = 0x%4.4lx, length = "
+		   "0x%4.4x\n", mem->win[i].card_addr,
 		   (u_long)mem->win[i].host_addr, mem->win[i].len);
     }
 
     if (entry->subtuples)
 	printf("  %d bytes in subtuples\n", entry->subtuples);
     
-} /* print_cftable_entry */
+}
 
 /*====================================================================*/
 
-void print_jedec(cistpl_jedec_t *jedec)
+static void print_cftable_entry_cb(cistpl_cftable_entry_cb_t *entry)
+{
+    int i;
+    
+    printf("  Config entry %#2.2x%s:\n", entry->index,
+	   (entry->flags & CISTPL_CFTABLE_DEFAULT) ? " (default)" : "");
+
+    if (entry->flags & ~CISTPL_CFTABLE_DEFAULT) {
+	printf(" ");
+	if (entry->flags & CISTPL_CFTABLE_MASTER)
+	    printf(" [master]");
+	if (entry->flags & CISTPL_CFTABLE_INVALIDATE)
+	    printf(" [invalidate]");
+	if (entry->flags & CISTPL_CFTABLE_VGA_PALETTE)
+	    printf(" [vga palette]");
+	if (entry->flags & CISTPL_CFTABLE_PARITY)
+	    printf(" [parity]");
+	if (entry->flags & CISTPL_CFTABLE_WAIT)
+	    printf(" [wait]");
+	if (entry->flags & CISTPL_CFTABLE_SERR)
+	    printf(" [serr]");
+	if (entry->flags & CISTPL_CFTABLE_FAST_BACK)
+	    printf(" [fast back]");
+	if (entry->flags & CISTPL_CFTABLE_BINARY_AUDIO)
+	    printf(" [binary audio]");
+	if (entry->flags & CISTPL_CFTABLE_PWM_AUDIO)
+	    printf(" [pwm audio]");
+	printf("\n");
+    }
+    
+    if (entry->vcc.present)
+	print_power("Vcc", &entry->vcc);
+    if (entry->vpp1.present)
+	print_power("Vpp1", &entry->vpp1);
+    if (entry->vpp2.present)
+	print_power("Vpp2", &entry->vpp2);
+
+    if (entry->io) {
+	printf("  IO base registers: ");
+	for (i = 0; i < 8; i++)
+	    if (entry->io & (1<<i)) printf(" %d", i);
+	printf("\n");
+    }
+
+    if (entry->irq.IRQInfo1) {
+	printf("  Interrupt ");
+	if (entry->irq.IRQInfo1 & IRQ_INFO2_VALID)
+	    printf("mask = 0x%#4.4x", entry->irq.IRQInfo2);
+	else
+	    printf("%u", entry->irq.IRQInfo1 & IRQ_MASK);
+	if (entry->irq.IRQInfo1 & IRQ_LEVEL_ID) printf(" [level]");
+	if (entry->irq.IRQInfo1 & IRQ_PULSE_ID) printf(" [pulse]");
+	if (entry->irq.IRQInfo1 & IRQ_SHARE_ID) printf(" [shared]");
+	printf("\n");
+    }
+
+    if (entry->mem) {
+	printf("  Memory base registers: ");
+	for (i = 0; i < 8; i++)
+	    if (entry->mem & (1<<i)) printf(" %d", i);
+	printf("\n");
+    }
+
+    if (entry->subtuples)
+	printf("  %d bytes in subtuples\n", entry->subtuples);
+    
+}
+
+/*====================================================================*/
+
+static void print_jedec(cistpl_jedec_t *jedec)
 {
     int i;
     for (i = 0; i < jedec->nid; i++) {
 	printf("    mfr 0x%02x, info 0x%02x\n",
 	       jedec->id[i].mfr, jedec->id[i].info);
     }
-} /* print_jedec */
+}
 
 /*====================================================================*/
 
-void print_device_geo(cistpl_device_geo_t *geo)
+static void print_device_geo(cistpl_device_geo_t *geo)
 {
     int i;
     for (i = 0; i < geo->ngeo; i++) {
-	printf("    bus %d, erase %#lx, read %#lx, write %#lx, "
-	       "partition %#lx, interleave %#lx\n",
+	printf("    bus %d, erase %#x, read %#x, write %#x, "
+	       "partition %#x, interleave %#x\n",
 	       geo->geo[i].buswidth, geo->geo[i].erase_block,
 	       geo->geo[i].read_block, geo->geo[i].write_block,
 	       geo->geo[i].partition, geo->geo[i].interleave);
     }
-} /* print_device_geo */
+}
 
 /*====================================================================*/
 
-void print_org(cistpl_org_t *org)
+static void print_org(cistpl_org_t *org)
 {
     printf("  Data organization: ");
     switch (org->data_org) {
@@ -386,17 +451,17 @@ void print_org(cistpl_org_t *org)
 	    printf("vendor-specific");
     }
     printf(", \"%s\"\n", org->desc);
-} /* print_org */
+}
 
 /*====================================================================*/
 
-void print_serial(cistpl_funce_t *funce)
+static void print_serial(cistpl_funce_t *funce)
 {
 }
 
 /*====================================================================*/
 
-void print_fixed(cistpl_funce_t *funce)
+static void print_fixed(cistpl_funce_t *funce)
 {
     cistpl_ide_interface_t *i;
     cistpl_ide_feature_t *f;
@@ -458,7 +523,7 @@ static const char *media[] = {
     "5.4 GHz", "Diffuse Infrared", "Point-to-Point Infrared"
 };
 
-void print_network(cistpl_funce_t *funce)
+static void print_network(cistpl_funce_t *funce)
 {
     cistpl_lan_tech_t *t;
     cistpl_lan_speed_t *s;
@@ -510,7 +575,7 @@ void print_vers_2(cistpl_vers_2_t *v2)
 	   v2->vspec8, v2->vspec9, v2->nhdr);
     printf("  Vendor \"%s\"\n", v2->str+v2->vendor);
     printf("  Info: %s\n", v2->str+v2->info);
-} /* print_vers_2 */
+}
 
 /*====================================================================*/
 
@@ -534,15 +599,15 @@ void print_parse(tuple_parse_t *tup)
 	       tup->parse.checksum.sum);
 	break;
     case CISTPL_LONGLINK_A:
-	printf("  Long link attr 0x%04lx\n", tup->parse.longlink.addr);
+	printf("  Long link attr 0x%04x\n", tup->parse.longlink.addr);
 	break;
     case CISTPL_LONGLINK_C:
-	printf("  Long link common 0x%04lx\n", tup->parse.longlink.addr);
+	printf("  Long link common 0x%04x\n", tup->parse.longlink.addr);
 	break;
     case CISTPL_LONGLINK_MFC:
 	printf("  Multifunction long links:\n");
 	for (i = 0; i < tup->parse.longlink_mfc.nfn; i++)
-	    printf("   function %d: %s 0x%04lx\n", i,
+	    printf("   function %d: %s 0x%04x\n", i,
 		   tup->parse.longlink_mfc.fn[i].space ? "common" : "attr",
 		   tup->parse.longlink_mfc.fn[i].addr);
 	break;
@@ -601,9 +666,34 @@ void print_parse(tuple_parse_t *tup)
 	    break;
 	}
 	break;
+    case CISTPL_BAR:
+	printf("  Base address register %d: size = ",
+	       tup->parse.bar.attr & CISTPL_BAR_SPACE);
+	print_size(tup->parse.bar.size);
+	if (tup->parse.bar.attr & CISTPL_BAR_SPACE_IO)
+	    printf(" [io]");
+	else
+	    printf(" [mem]");
+	if (tup->parse.bar.attr & CISTPL_BAR_PREFETCH)
+	    printf(" [prefetch]");
+	if (tup->parse.bar.attr & CISTPL_BAR_CACHEABLE)
+	    printf(" [cacheable]");
+	if (tup->parse.bar.attr & CISTPL_BAR_1MEG_MAP)
+	    printf(" [below 1mb]");
+	printf("\n");
+	break;
     case CISTPL_CONFIG:
-	printf("  Config base = %#4.4lx, mask = %#4.4lx, ",
-	       tup->parse.config.base, tup->parse.config.rmask[0]);
+    case CISTPL_CONFIG_CB:
+	printf("  Config base = %#4.4x, ", tup->parse.config.base);
+	if (tup->tuple.TupleCode == CISTPL_CONFIG)
+	    printf("mask = %#4.4x, ", tup->parse.config.rmask[0]);
+	printf("last config index = %#2.2x\n",
+	       tup->parse.config.last_idx);
+	if (tup->parse.config.subtuples)
+	    printf("  %d bytes in subtuples\n",
+		   tup->parse.config.subtuples);
+	break;
+	printf("  Config base = %#4.4x, ", tup->parse.config.base);
 	printf("last config index = %#2.2x\n",
 	       tup->parse.config.last_idx);
 	if (tup->parse.config.subtuples)
@@ -613,6 +703,9 @@ void print_parse(tuple_parse_t *tup)
     case CISTPL_CFTABLE_ENTRY:
 	print_cftable_entry(&tup->parse.cftable_entry);
 	break;
+    case CISTPL_CFTABLE_ENTRY_CB:
+	print_cftable_entry_cb(&tup->parse.cftable_entry_cb);
+	break;
     case CISTPL_VERS_2:
 	print_vers_2(&tup->parse.vers_2);
 	break;
@@ -620,13 +713,13 @@ void print_parse(tuple_parse_t *tup)
 	print_org(&tup->parse.org);
 	break;
     }
-} /* print_parse */
+}
 
 /*====================================================================*/
 
 #define MAX_SOCKS 8
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int i, major, fd, ret;
     ds_ioctl_arg_t arg;
@@ -668,11 +761,9 @@ void main(int argc, char *argv[])
 		if (ret != 0) {
 		    if (errno != ENOSYS)
 			printf("  parse error: %s\n", strerror(errno));
-		}
-		else
+		} else
 		    print_parse(&arg.tuple_parse);
-	    }
-	    else {
+	    } else {
 		printf("  get tuple data: %s\n", strerror(errno));
 		break;
 	    }
@@ -685,4 +776,5 @@ void main(int argc, char *argv[])
 	    }
 	}
     }
+    return 0;
 }
