@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    ftl_format.c 1.3 1996/11/26 07:24:56
+    ftl_format.c 1.4 1997/12/29 17:49:52
 
 ======================================================================*/
 
@@ -27,14 +27,14 @@
 
 /*====================================================================*/
 
-static void print_size(u_long s)
+static void print_size(u_int s)
 {
     if ((s > 0x100000) && ((s % 0x100000) == 0))
-	printf("%ld mb", s / 0x100000);
+	printf("%d mb", s / 0x100000);
     else if ((s > 0x400) && ((s % 0x400) == 0))
-	printf("%ld kb", s / 0x400);
+	printf("%d kb", s / 0x400);
     else
-	printf("%ld bytes", s);
+	printf("%d bytes", s);
 }
 
 /*====================================================================*/
@@ -43,7 +43,7 @@ static void check_partition(int fd, int verbose)
 {
     region_info_t region;
     erase_unit_header_t hdr, hdr2;
-    u_long i, j, nbam, *bam;
+    u_int i, j, nbam, *bam;
     int control, data, free, deleted;
     
     /* Get partition size, block size */
@@ -53,9 +53,9 @@ static void check_partition(int fd, int verbose)
     }
 
     printf("Memory region info:\n");
-    printf("  Card offset = 0x%lx, region size = ", region.CardOffset);
+    printf("  Card offset = 0x%x, region size = ", region.CardOffset);
     print_size(region.RegionSize);
-    printf(", access speed = %ld ns\n", region.AccessSpeed);
+    printf(", access speed = %u ns\n", region.AccessSpeed);
     printf("  Erase block size = ");
     print_size(region.BlockSize);
     printf(", partition multiple = ");
@@ -92,7 +92,7 @@ static void check_partition(int fd, int verbose)
     
     /* Create basic block allocation table for control blocks */
     nbam = (region.BlockSize >> hdr.BlockSize);
-    bam = malloc(nbam * sizeof(u_long));
+    bam = malloc(nbam * sizeof(u_int));
 
     for (i = 0; i < hdr.NumEraseUnits; i++) {
 	if (lseek(fd, (i << hdr.EraseUnitSize), SEEK_SET) == -1) {
@@ -103,22 +103,22 @@ static void check_partition(int fd, int verbose)
 	    perror("read failed");
 	    break;
 	}
-	printf("\nErase unit %ld:\n", i);
+	printf("\nErase unit %d:\n", i);
 	if ((hdr2.FormattedSize != hdr.FormattedSize) ||
 	    (hdr2.NumEraseUnits != hdr.NumEraseUnits) ||
 	    (hdr2.SerialNumber != hdr.SerialNumber))
 	    printf("  Erase unit header is corrupt.\n");
 	else if (hdr2.LogicalEUN == 0xffff)
-	    printf("  Transfer unit, erase count = %ld\n", hdr2.EraseCount);
+	    printf("  Transfer unit, erase count = %d\n", hdr2.EraseCount);
 	else {
-	    printf("  Logical unit %d, erase count = %ld\n",
+	    printf("  Logical unit %d, erase count = %d\n",
 		   hdr2.LogicalEUN, hdr2.EraseCount);
 	    if (lseek(fd, (i << hdr.EraseUnitSize)+hdr.BAMOffset,
 		      SEEK_SET) == -1) {
 		perror("seek failed");
 		break;
 	    }
-	    if (read(fd, bam, nbam * sizeof(u_long)) == -1) {
+	    if (read(fd, bam, nbam * sizeof(u_int)) == -1) {
 		perror("read failed");
 		break;
 	    }
@@ -142,7 +142,7 @@ static void check_partition(int fd, int verbose)
 
 /*====================================================================*/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int verbose;
     int optch, errflg, fd;
@@ -182,4 +182,5 @@ void main(int argc, char *argv[])
     close(fd);
     
     exit(EXIT_SUCCESS);
+    return 0;
 }
