@@ -1,5 +1,5 @@
 /*
- * ciscode.h 1.14 1997/06/04 14:58:43 (David Hinds)
+ * ciscode.h 1.21 1997/10/22 17:50:59 (David Hinds)
  */
 
 #ifndef _LINUX_CISCODE_H
@@ -19,6 +19,9 @@
 #define MANFID_CONTEC			0xc001
 
 #define MANFID_FUJITSU			0x0004
+#define PRODID_FUJITSU_MBH10302		0x0004
+#define PRODID_FUJITSU_MBH10304		0x1003
+#define PRODID_FUJITSU_LA501		0x2000
 
 #define MANFID_IBM			0x00a4
 #define PRODID_IBM_HOME_AND_AWAY	0x002e
@@ -36,6 +39,9 @@
 
 #define MANFID_MACNICA			0xc00b
 
+#define MANFID_MOTOROLA			0x0109
+#define PRODID_MOTOROLA_MARINER		0x0501
+
 #define MANFID_NEW_MEDIA		0x0057
 
 #define MANFID_OSITECH			0x0140
@@ -45,6 +51,7 @@
 
 #define MANFID_QUATECH			0x0137
 #define PRODID_QUATECH_DUAL_RS232	0x0012
+#define PRODID_QUATECH_DUAL_RS232_D1	0x0007
 #define PRODID_QUATECH_QUAD_RS232	0x001b
 
 #define MANFID_SMC			0x0108
@@ -57,4 +64,6 @@
 
 #define MANFID_TDK			0x0105
 
+#define MANFID_XIRCOM			0x0105
+
 #endif /* _LINUX_CISCODE_H */
