@@ -1,10 +1,12 @@
 /*
- * ds.h 1.33 1997/01/02 06:33:13 (David Hinds)
+ * ds.h 1.36 1998/02/03 06:17:19 (David Hinds)
  */
 
 #ifndef _LINUX_DS_H
 #define _LINUX_DS_H
 
+#include <pcmcia/driver_ops.h>
+
 typedef struct tuple_parse_t {
     tuple_t		tuple;
     char		data[255];
@@ -16,14 +18,14 @@ typedef struct bind_info_t {
     u_char		function;
     struct dev_link_t	*instance;
     char		name[8];
-    u_char		major, minor;
+    u_short		major, minor;
     void		*next;
 } bind_info_t;
 
 typedef struct mtd_info_t {
     dev_info_t		dev_info;
-    u_long		Attributes;
-    u_long		CardOffset;
+    u_int		Attributes;
+    u_int		CardOffset;
 } mtd_info_t;
 
 typedef union ds_ioctl_arg_t {
@@ -67,15 +69,9 @@ typedef union ds_ioctl_arg_t {
 
 #ifdef __KERNEL__
 
-typedef struct dev_node_t {
-    char		dev_name[8];
-    u_char		major, minor;
-    struct dev_node_t	*next;
-} dev_node_t;
-
 typedef struct dev_link_t {
     dev_node_t		*dev;
-    u_long		state, open;
+    u_int		state, open;
     struct wait_queue	*pending;
     struct timer_list	release;
     client_handle_t	handle;
