@@ -104,7 +104,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"3c589_cs.c 1.96 1997/06/01 05:03:16 (David Hinds)";
+"3c589_cs.c 1.102 1998/02/12 00:05:48 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -117,10 +117,12 @@ static char *version =
 static int if_port = 0;
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(if_port, "i");
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /*====================================================================*/
 
@@ -156,9 +158,10 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -186,7 +189,7 @@ static dev_link_t *tc589_attach(void)
     client_reg_t client_reg;
     dev_link_t *link;
     struct device *dev;
-    int ret;
+    int i, ret;
 
     DEBUG(0, "3c589_attach()\n");
 
@@ -200,7 +203,11 @@ static dev_link_t *tc589_attach(void)
     link->io.IOAddrLines = 4;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->irq.Handler = &el3_interrupt;
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
@@ -226,7 +233,7 @@ static dev_link_t *tc589_attach(void)
     dev->open = &el3_open;
     dev->stop = &el3_close;
     dev->tbusy = 1;
-    link->priv = dev;
+    link->priv = link->irq.Instance = dev;
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -242,7 +249,7 @@ static dev_link_t *tc589_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	tc589_detach(link);
 	return NULL;
     }
@@ -262,7 +269,8 @@ static dev_link_t *tc589_attach(void)
 static void tc589_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
-
+    long flags;
+    
     DEBUG(0, "3c589_detach(0x%p)\n", link);
     
     /* Locate device structure */
@@ -271,6 +279,14 @@ static void tc589_detach(dev_link_t *link)
     if (*linkp == NULL)
 	return;
 
+    save_flags(flags);
+    cli();
+    if (link->state & DEV_RELEASE_PENDING) {
+	del_timer(&link->release);
+	link->state &= ~DEV_RELEASE_PENDING;
+    }
+    restore_flags(flags);
+    
     if (link->state & DEV_CONFIG) {
 	tc589_release((u_long)link);
 	if (link->state & DEV_STALE_CONFIG) {
@@ -302,6 +318,9 @@ static void tc589_detach(dev_link_t *link)
     
 ======================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void tc589_config(dev_link_t *link)
 {
     client_handle_t handle;
@@ -309,7 +328,7 @@ static void tc589_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     u_short buf[32];
-    int i, j, multi = 0;
+    int last_fn, last_ret, i, j, multi = 0;
     short ioaddr, *phys_addr;
     
     handle = link->handle;
@@ -318,26 +337,16 @@ static void tc589_config(dev_link_t *link)
 
     DEBUG(0, "3c589_config(0x%p)\n", link);
 
-    do {
-	tuple.Attributes = 0;
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	tuple.TupleData = (cisdata_t *)buf;
-	tuple.TupleDataMax = 64;
-	tuple.TupleOffset = 0;
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i != CS_SUCCESS) break;
-	link->conf.ConfigBase = parse.config.base;
-	link->conf.Present = parse.config.rmask[0];
-    } while (0);
-    if (i != CS_SUCCESS) {
-	printk(KERN_NOTICE "3c589_cs: unable to parse CIS\n");
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
+    tuple.Attributes = 0;
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    tuple.TupleData = (cisdata_t *)buf;
+    tuple.TupleDataMax = 64;
+    tuple.TupleOffset = 0;
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
+    link->conf.ConfigBase = parse.config.base;
+    link->conf.Present = parse.config.rmask[0];
     
     /* Is this a 3c562? */
     tuple.DesiredTuple = CISTPL_MANFID;
@@ -353,45 +362,29 @@ static void tc589_config(dev_link_t *link)
     /* Configure card */
     link->state |= DEV_CONFIG;
 
-    do {
-	/* For the 3c562, the base address must be xx00-xx7f */
-	for (j = 0; j < 0x400; j += 0x10) {
-	    if (multi && (j & 0x80)) continue;
-	    link->io.BasePort1 = j ^ 0x300;
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i == CS_SUCCESS) break;
-	}
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIO, i);
-	    break;
-	}
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    break;
-	}
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, i);
-	    break;
-	}
+    /* For the 3c562, the base address must be xx00-xx7f */
+    for (i = j = 0; j < 0x400; j += 0x10) {
+	if (multi && (j & 0x80)) continue;
+	link->io.BasePort1 = j ^ 0x300;
+	i = CardServices(RequestIO, link->handle, &link->io);
+	if (i == CS_SUCCESS) break;
+    }
+    if (i != CS_SUCCESS) {
+	cs_error(link->handle, RequestIO, i);
+	goto failed;
+    }
+    CS_CHECK(RequestIRQ, link->handle, &link->irq);
+    CS_CHECK(RequestConfiguration, link->handle, &link->conf);
 	
-	dev->irq = link->irq.AssignedIRQ;
-	dev->base_addr = link->io.BasePort1;
-	dev->tbusy = 0;
-	i = register_netdev(dev);
-	if (i != 0) {
-	    printk(KERN_NOTICE "3c589_cs: register_netdev() failed\n");
-	    break;
-	}
-    } while (0);
+    dev->irq = link->irq.AssignedIRQ;
+    dev->base_addr = link->io.BasePort1;
+    dev->tbusy = 0;
+    if (register_netdev(dev) != 0) {
+	printk(KERN_NOTICE "3c589_cs: register_netdev() failed\n");
+	goto failed;
+    }
     
     link->state &= ~DEV_CONFIG_PENDING;
-    if (i != 0) {
-	tc589_release((u_long)link);
-	return;
-    }
-
     ioaddr = dev->base_addr;
     EL3WINDOW(0);
 
@@ -408,8 +401,7 @@ static void tc589_config(dev_link_t *link)
 	if (phys_addr[0] == 0x6060) {
 	    printk(KERN_NOTICE "3c589_cs: IO port conflict at 0x%03lx"
 		   "-0x%03lx\n", dev->base_addr, dev->base_addr+15);
-	    tc589_release((u_long)link);
-	    return;
+	    goto failed;
 	}
     }
     
@@ -433,12 +425,18 @@ static void tc589_config(dev_link_t *link)
     case 3: outw(1<<14, ioaddr + 6); break;
     }
     
-    printk(KERN_INFO "%s: 3Com 3c%s, port %#3lx, irq %d, %s port, ",
-	   dev->name, (multi ? "562" : "589"),
+    printk(KERN_INFO "%s: 3Com 3c%s, port %#3lx, irq %d, %s port, "
+	   "hw_addr ", dev->name, (multi ? "562" : "589"),
 	   dev->base_addr, dev->irq, if_names[dev->if_port]);
     for (i = 0; i < 6; i++)
-	printk(" %02X", dev->dev_addr[i]);
-    printk("\n");
+	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
+    return;
+
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    tc589_release((u_long)link);
+    return;
     
 } /* tc589_config */
 
@@ -464,14 +462,12 @@ static void tc589_release(u_long arg)
 	return;
     }
     
-    if (link->dev != '\0')
-	unregister_netdev(dev);
-    link->dev = NULL;
     CardServices(ReleaseConfiguration, link->handle);
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
-    if (dev->irq != 0)
-	irq2dev_map[dev->irq] = NULL;
+    if (link->dev)
+	unregister_netdev(dev);
+    link->dev = NULL;
     
     link->state &= ~DEV_CONFIG;
     if (link->state & DEV_STALE_LINK)
@@ -634,8 +630,6 @@ static int el3_open(struct device *dev)
 
     EL3WINDOW(0);
     
-    irq2dev_map[dev->irq] = dev;
-    
     dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
     if (dev->if_port == 0) {
 	lp->probe_port = 15; lp->flipped = 0;
@@ -725,7 +719,7 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 	    outw(SetTxThreshold + 1536, ioaddr + EL3_CMD);
     }
 
-    dev_kfree_skb (skb, FREE_WRITE);
+    DEV_KFREE_SKB (skb);
     
     /* Clear the Tx status stack. */
     {
@@ -749,12 +743,12 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 /* The EL3 interrupt handler. */
 static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 {
-    struct device *dev = (struct device *)(irq2dev_map[irq]);
+    struct device *dev = (struct device *)DEV_ID;
     struct el3_private *lp;
     int ioaddr, status;
     int i = 0;
     
-    if (dev == NULL)
+    if ((dev == NULL) || !dev->start)
 	return;
     lp = (struct el3_private *)dev->priv;
     ioaddr = dev->base_addr;
@@ -1093,7 +1087,7 @@ int init_module(void)
     DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
-	printk(KERN_NOTICE "serial_cs: Card Services release "
+	printk(KERN_NOTICE "3c589_cs: Card Services release "
 	       "does not match!\n");
 	return -1;
     }
