@@ -2,7 +2,7 @@
 
     PCMCIA Bulk Memory Services
 
-    Written by David Hinds, dhinds@allegro.stanford.edu
+    Written by David Hinds, dhinds@hyper.stanford.edu
     
 ======================================================================*/
 
@@ -72,7 +72,7 @@ static int do_mtd_request(memory_handle_t handle, mtd_request_t *req,
 	    interruptible_sleep_on(&mtd->mtd_req);
 	    break;
 	}
-	if (current->signal & ~current->blocked)
+	if (signal_pending(current))
 	    printk(KERN_NOTICE "cs: do_mtd_request interrupted!\n");
     }
     if (tries == 20)
@@ -106,7 +106,7 @@ static void remove_queue(erase_busy_t *entry)
     entry->prev->next = entry->next;
 }
 
-static void retry_erase(erase_busy_t *busy, u_long cause)
+static void retry_erase(erase_busy_t *busy, u_int cause)
 {
     eraseq_entry_t *erase = busy->erase;
     mtd_request_t req;
@@ -126,7 +126,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     mtd->event_callback_args.mtdrequest = &req;
     ret = EVENT(mtd, CS_EVENT_MTD_REQUEST, CS_EVENT_PRI_LOW);
     if (ret == CS_BUSY) {
-	DEBUG(2, "  Status = %ld, requeueing.\n", req.Status);
+	DEBUG(2, "  Status = %d, requeueing.\n", req.Status);
 	switch (req.Status) {
 	case MTD_WAITREQ:
 	case MTD_WAITPOWER:
@@ -140,8 +140,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
 	    add_timer(&busy->timeout);
 	    break;
 	}
-    }
-    else {
+    } else {
 	/* update erase queue status */
 	DEBUG(2, "  Ret = %d\n", ret);
 	switch (ret) {
@@ -167,7 +166,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     }
 } /* retry_erase */
 
-void retry_erase_list(erase_busy_t *list, u_long cause)
+void retry_erase_list(erase_busy_t *list, u_int cause)
 {
     erase_busy_t tmp = *list;
 
@@ -201,7 +200,7 @@ static void setup_erase_request(client_handle_t handle, eraseq_entry_t *erase)
 	erase->State = ERASE_BAD_SOCKET;
     else {
 	info = &erase->Handle->info;
-	if ((erase->Offset > info->RegionSize) ||
+	if ((erase->Offset >= info->RegionSize) ||
 	    (erase->Offset & (info->BlockSize-1)))
 	    erase->State = ERASE_BAD_OFFSET;
 	else if ((erase->Offset+erase->Size > info->RegionSize) ||
@@ -225,9 +224,6 @@ static void setup_erase_request(client_handle_t handle, eraseq_entry_t *erase)
 
     MTD helper functions
 
-    We don't support MTDRequestWindow or MTDReleaseWindow -- clients
-    should use the normal Card Services calls.
-    
 ======================================================================*/
 
 static int mtd_modify_window(window_handle_t win, mtd_mod_win_t *req)
@@ -277,6 +273,10 @@ static int mtd_rdy_mask(client_handle_t handle, mtd_rdy_req_t *req)
 int MTDHelperEntry(int func, void *a1, void *a2)
 {
     switch (func) {
+    case MTDRequestWindow:
+	return CardServices(RequestWindow, a1, a2, NULL);
+    case MTDReleaseWindow:
+	return CardServices(ReleaseWindow, a1, NULL, NULL);
     case MTDModifyWindow:
 	return mtd_modify_window(a1, a2); break;
     case MTDSetVpp:
@@ -300,7 +300,7 @@ static void setup_regions(client_handle_t handle, int attr,
 			  memory_handle_t *list)
 {
     int i, code, has_jedec, has_geo;
-    u_long offset;
+    u_int offset;
     cistpl_device_t device;
     cistpl_jedec_t jedec;
     cistpl_device_geo_t geo;
@@ -345,16 +345,14 @@ static void setup_regions(client_handle_t handle, int attr,
 	    if (has_jedec) {
 		r->info.JedecMfr = jedec.id[i].mfr;
 		r->info.JedecInfo = jedec.id[i].info;
-	    }
-	    else
+	    } else
 		r->info.JedecMfr = r->info.JedecInfo = 0;
 	    if (has_geo) {
 		r->info.BlockSize = geo.geo[i].buswidth *
 		    geo.geo[i].erase_block * geo.geo[i].interleave;
 		r->info.PartMultiple =
 		    r->info.BlockSize * geo.geo[i].partition;
-	    }
-	    else
+	    } else
 		r->info.BlockSize = r->info.PartMultiple = 1;
 	    r->info.next = *list; *list = r;
 	}
@@ -429,7 +427,7 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 	list = s->a_region;
     else
 	list = s->c_region;
-    DEBUG(1, "cs: register_mtd(0x%p, '%s', 0x%lx)\n",
+    DEBUG(1, "cs: register_mtd(0x%p, '%s', 0x%x)\n",
 	  handle, (char *)handle->dev_info, reg->Offset);
     while (list) {
 	if (list->info.CardOffset == reg->Offset) break;
@@ -442,8 +440,7 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 	list->mtd = handle;
 	handle->mtd_count++;
 	return CS_SUCCESS;
-    }
-    else
+    } else
 	return CS_BAD_OFFSET;
 } /* register_mtd */
 
@@ -518,11 +515,10 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
     }
     if (region && region->mtd) {
 	*handle = (client_handle_t)region;
-	DEBUG(1, "cs: open_memory(0x%p, 0x%lx) = 0x%p\n",
+	DEBUG(1, "cs: open_memory(0x%p, 0x%x) = 0x%p\n",
 	      handle, open->Offset, region);
 	return CS_SUCCESS;
-    }
-    else
+    } else
 	return CS_BAD_OFFSET;
 } /* open_memory */
 
@@ -554,7 +550,7 @@ int read_memory(memory_handle_t handle, mem_op_t *req, caddr_t buf)
     mtd_request_t mtd;
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
-    if (req->Offset > handle->info.RegionSize)
+    if (req->Offset >= handle->info.RegionSize)
 	return CS_BAD_OFFSET;
     if (req->Offset+req->Count > handle->info.RegionSize)
 	return CS_BAD_SIZE;
@@ -580,7 +576,7 @@ int write_memory(memory_handle_t handle, mem_op_t *req, caddr_t buf)
     mtd_request_t mtd;
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
-    if (req->Offset > handle->info.RegionSize)
+    if (req->Offset >= handle->info.RegionSize)
 	return CS_BAD_OFFSET;
     if (req->Offset+req->Count > handle->info.RegionSize)
 	return CS_BAD_SIZE;
