@@ -20,6 +20,10 @@
 #include <linux/sched.h>
 #include <asm/io.h>
 
+#ifdef CONFIG_CARDBUS
+#include <linux/bios32.h>
+#endif
+
 #include <pcmcia/cs_types.h>
 #include <pcmcia/ss.h>
 #include <pcmcia/cs.h>
@@ -35,7 +39,7 @@ static const u_char mantissa[] = {
     40, 45, 50, 55, 60, 70, 80, 90
 };
 
-static const u_long exponent[] = {
+static const u_int exponent[] = {
     1, 10, 100, 1000, 10000, 100000, 1000000, 10000000
 };
 
@@ -57,87 +61,117 @@ static const u_long exponent[] = {
     
 ======================================================================*/
 
-void read_cis_mem(socket_info_t *s, int attr,
-		  long addr, long len, u_char *ptr)
+void read_cis_mem(socket_info_t *s, int attr, u_int addr,
+		  u_int len, void *ptr)
 {
     pcmcia_mem_map *mem = &s->cis_mem;
-    u_char *sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
-    int inc = 1;
+    u_char *sys;
+    u_int inc = 1;
     
-    DEBUG(3, "cs: read_cis_mem(%d, %#lx, %ld)\n", attr, addr, len);
-    mem->card_start = addr & ~0x0fff;
+    DEBUG(3, "cs: read_cis_mem(%d, %#x, %u)\n", attr, addr, len);
     mem->flags &= ~MAP_ATTRIB;
-    if (attr) { mem->flags |= MAP_ATTRIB; inc++; }
+    if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
+    sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
+    mem->card_start = addr & ~0x0fff;
     
     for (; len > 0; mem->card_start += 0x1000, sys -= 0x1000) {
 	s->ss_entry(s->sock, SS_SetMemMap, mem);
-	DEBUG(3, "cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
-	      sys[0], sys[inc], sys[2*inc], sys[3*inc], sys[4*inc]);
+	DEBUG(3,  "cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
+	      readb(sys), readb(sys+inc), readb(sys+2*inc),
+	      readb(sys+3*inc), readb(sys+4*inc));
 	for ( ; len > 0; len--, ptr++, sys += inc) {
 	    if (sys > (u_char *)mem->sys_stop) break;
-	    *ptr = readb(sys);
+	    *(u_char *)ptr = readb(sys);
 	}
     }
-} /* read_cis_mem */
+}
 
-void write_cis_mem(socket_info_t *s, int attr,
-		   long addr, long len, u_char *ptr)
+#ifdef CONFIG_CARDBUS
+void read_cb_mem(socket_info_t *s, u_char fn, int space, u_int addr,
+		 u_int len, void *ptr)
+{
+    DEBUG(3, "cs: read_cb_mem(%d, %#x, %u)\n", space, addr, len);
+    if (space == 0) {
+	for (; len; addr++, ptr++, len--)
+	    pcibios_read_config_byte(s->cap.cardbus, fn, addr, ptr);
+    }
+}
+#endif
+
+void write_cis_mem(socket_info_t *s, int attr, u_int addr,
+		   u_int len, void *ptr)
 {
     pcmcia_mem_map *mem = &s->cis_mem;
-    u_char *sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
+    u_char *sys;
     int inc = 1;
     
-    mem->card_start = addr & ~0x0fff;
     mem->flags &= ~MAP_ATTRIB;
-    if (attr) { mem->flags |= MAP_ATTRIB; inc++; }
+    if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
+    sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
+    mem->card_start = addr & ~0x0fff;
     
     for (; len > 0; mem->card_start += 0x1000, sys -= 0x1000) {
 	s->ss_entry(s->sock, SS_SetMemMap, mem);
 	for ( ; len > 0; len--, ptr++, sys += inc) {
 	    if (sys > (u_char *)mem->sys_stop) break;
-	    writeb(*ptr, sys);
+	    writeb(*(u_char *)ptr, sys);
 	}
     }
-} /* write_cis_mem */
+}
+
+/*======================================================================
+
+    This is tricky... when we set up CIS memory, we try to validate
+    the memory window space below 1 MB.
+    
+======================================================================*/
+
+/* Scratch pointer to the socket we use for validation */
+static socket_info_t *vs = NULL;
+
+/* Validation function for cards with a valid CIS */
+static int cis_readable(u_long base)
+{
+    cisinfo_t info1, info2;
+    int ret;
+    vs->cis_mem.sys_start = base;
+    vs->cis_mem.sys_stop = base+0x0fff;
+    ret = validate_cis(vs->clients, &info1);
+    vs->cis_used = 0; /* invalidate CIS cache */
+    if ((ret != 0) || (info1.Chains == 0))
+	return 0;
+    vs->cis_mem.sys_start = base+0x1000;
+    vs->cis_mem.sys_stop = base+0x1fff;
+    ret = validate_cis(vs->clients, &info2);
+    vs->cis_used = 0;
+    return ((ret == 0) && (info1.Chains == info2.Chains));
+}
+
+/* Validation function for simple memory cards */
+static int checksum(u_long base)
+{
+    int i, a;
+    vs->cis_mem.sys_start = base;
+    vs->cis_mem.sys_stop = base+0xfff;
+    vs->cis_mem.card_start = 0;
+    vs->cis_mem.flags = MAP_ACTIVE;
+    vs->ss_entry(vs->sock, SS_SetMemMap, &vs->cis_mem);
+    /* Don't bother checking every word... */
+    for (i = a = 0; i < 0x1000; i += 56)
+	a += (int)readl(base+i);
+    return a;
+}
+
+static int checksum_match(u_long base)
+{
+    return (checksum(base) == checksum(base+0x1000));
+}
 
 int setup_cis_mem(socket_info_t *s)
 {
-    /* Validation function for cards with a valid CIS */
-    int cis_readable(u_long base)
-    {
-	cisinfo_t info;
-	int ret;
-	s->cis_mem.sys_start = base;
-	s->cis_mem.sys_stop = base+0x0fff;
-	ret = validate_cis(s->clients, &info);
-	s->cis_used = 0; /* invalidate CIS cache */
-	return ((ret == 0) && (info.Chains > 0));
-    }
-
-    /* Validation function for simple memory cards */
-    int checksum(u_long base)
-    {
-	int i, a;
-	s->cis_mem.sys_start = base;
-	s->cis_mem.sys_stop = base+0xfff;
-	s->cis_mem.card_start = 0;
-	s->cis_mem.flags = MAP_ACTIVE;
-	s->ss_entry(s->sock, SS_SetMemMap, &s->cis_mem);
-	/* Don't bother checking every word... */
-	for (i = a = 0; i < 0x1000; i += 53)
-	    a += (int)readl(base+i);
-	return a;
-    }
-
-    int checksum_match(u_long base)
-    {
-	return (checksum(base) == checksum(base+0x1000));
-    }
-    
-    static int first = 1;
     if (s->cis_mem.sys_start == 0) {
-	if (first == 1) {
-	    first = 0;
+	if (vs == NULL) {
+	    vs = s;
 	    validate_mem(cis_readable, checksum_match);
 	    s->cis_mem.sys_start = 0;
 	}
@@ -147,7 +181,7 @@ int setup_cis_mem(socket_info_t *s)
 	s->cis_mem.sys_stop = s->cis_mem.sys_start + 0x0fff;
     }
     return 0;
-} /* setup_cis_mem */
+}
 
 /*======================================================================
 
@@ -157,8 +191,8 @@ int setup_cis_mem(socket_info_t *s)
     
 ======================================================================*/
 
-static void read_cis_cache(socket_info_t *s, int attr,
-			   long addr, long len, u_char *ptr)
+static void read_cis_cache(socket_info_t *s, int attr, u_int addr,
+			   u_int len, void *ptr)
 {
     int i;
     char *caddr;
@@ -174,7 +208,12 @@ static void read_cis_cache(socket_info_t *s, int attr,
 	memcpy(ptr, caddr, len);
 	return;
     }
-    read_cis_mem(s, attr, addr, len, ptr);
+#ifdef CONFIG_CARDBUS
+    if (s->state & SOCKET_CARDBUS)
+	read_cb_mem(s, 0, attr, addr, len, ptr);
+    else
+#endif
+	read_cis_mem(s, attr, addr, len, ptr);
     /* Copy data into the cache, if there is room */
     if ((i < MAX_CIS_TABLE) &&
 	(caddr+len < s->cis_cache+MAX_CIS_DATA)) {
@@ -215,21 +254,46 @@ int verify_cis_cache(socket_info_t *s)
     
 ======================================================================*/
 
-#define LINK_MFC_N	0x00ff
-#define LINK_C		0x0100
-#define LINK_A		0x0200
-#define LINK_MFC	0x0400
-#define IS_ATTR 	0x1000
-#define MFC_IS_ATTR	0x2000
+typedef struct tuple_flags {
+    u_int		link_space:3;
+    u_int		has_link:1;
+    u_int		mfc_fn:3;
+    u_int		space:3;
+} tuple_flags;
+
+#define LINK_SPACE(f)	(((tuple_flags *)(&(f)))->link_space)
+#define HAS_LINK(f)	(((tuple_flags *)(&(f)))->has_link)
+#define MFC_FN(f)	(((tuple_flags *)(&(f)))->mfc_fn)
+#define SPACE(f)	(((tuple_flags *)(&(f)))->space)
 
 int get_next_tuple(client_handle_t handle, tuple_t *tuple);
 
 int get_first_tuple(client_handle_t handle, tuple_t *tuple)
 {
+    socket_info_t *s;
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
-    tuple->CISOffset = -1;
-    if ((SOCKET(handle)->functions > 1) &&
+    s = SOCKET(handle);
+#ifdef CONFIG_CARDBUS
+    if (s->state & SOCKET_CARDBUS) {
+	u_int ptr;
+	pcibios_read_config_dword(s->cap.cardbus, 0, 0x28, &ptr);
+	tuple->CISOffset = ptr & ~7;
+	tuple->TupleLink = 0;
+	SPACE(tuple->Flags) = (ptr & 7);
+    } else {
+	tuple->CISOffset = tuple->TupleLink = 0;
+	/* Assume presence of a LONGLINK_C to address 0 */
+	tuple->LinkOffset = tuple->Flags = 0;
+	SPACE(tuple->Flags) = HAS_LINK(tuple->Flags) = 1;
+    }
+#else
+    tuple->CISOffset = tuple->TupleLink = 0;
+    /* Assume presence of a LONGLINK_C to address 0 */
+    tuple->LinkOffset = tuple->Flags = 0;
+    SPACE(tuple->Flags) = HAS_LINK(tuple->Flags) = 1;
+#endif
+    if (!(s->state & SOCKET_CARDBUS) && (s->functions > 1) &&
 	!(tuple->Attributes & TUPLE_RETURN_COMMON)) {
 	cisdata_t req = tuple->DesiredTuple;
 	tuple->DesiredTuple = CISTPL_LONGLINK_MFC;
@@ -238,68 +302,55 @@ int get_first_tuple(client_handle_t handle, tuple_t *tuple)
 	    if (get_next_tuple(handle, tuple) != CS_SUCCESS)
 		return CS_NO_MORE_ITEMS;
 	} else
-	    tuple->CISOffset = -1;
+	    tuple->CISOffset = tuple->TupleLink = 0;
 	tuple->DesiredTuple = req;
     }
     return get_next_tuple(handle, tuple);
-} /* get_first_tuple */
+}
 
-static long follow_link(socket_info_t *s, tuple_t *tuple)
+static int follow_link(socket_info_t *s, tuple_t *tuple)
 {
     u_char link[5];
-    long ofs;
-    
-    switch (tuple->Flags & (LINK_A|LINK_C|LINK_MFC)) {
-    case LINK_A:
-	ofs = tuple->LinkOffset;
-	tuple->Flags = IS_ATTR;
-	break;
-    case LINK_C:
-	ofs = tuple->LinkOffset;
-	tuple->Flags = 0;
-	break;
-    case LINK_MFC:
+    u_int ofs;
+
+    if (MFC_FN(tuple->Flags)) {
 	/* Get indirect link from the MFC tuple */
-	read_cis_cache(s, tuple->Flags & MFC_IS_ATTR,
+	read_cis_cache(s, LINK_SPACE(tuple->Flags),
 		       tuple->LinkOffset, 5, link);
-	ofs = *(long *)(link+1);
-	if (link[0] == CISTPL_MFC_ATTR)
-	    tuple->Flags |= IS_ATTR;
-	else
-	    tuple->Flags &= ~IS_ATTR;
+	ofs = *(u_int *)(link+1);
+	SPACE(tuple->Flags) = (link[0] == CISTPL_MFC_ATTR);
 	/* Move to the next indirect link */
-	tuple->Flags--;
-	tuple->LinkOffset += (tuple->Flags & MFC_IS_ATTR) ? 10 : 5;
-	if ((tuple->Flags & LINK_MFC_N) == 0)
-	    tuple->Flags &= ~LINK_MFC;
-	break;
-    default:
+	tuple->LinkOffset += 5;
+	MFC_FN(tuple->Flags)--;
+    } else if (HAS_LINK(tuple->Flags)) {
+	ofs = tuple->LinkOffset;
+	SPACE(tuple->Flags) = LINK_SPACE(tuple->Flags);
+	HAS_LINK(tuple->Flags) = 0;
+    } else {
 	return -1;
     }
-    if (tuple->Flags & IS_ATTR) {
-	ofs = ofs << 1;
+    if (!(s->state & SOCKET_CARDBUS) && SPACE(tuple->Flags)) {
 	/* This is ugly, but a common CIS error is to code the long
 	   link offset incorrectly, so we check the right spot... */
-	read_cis_cache(s, tuple->Flags & IS_ATTR, ofs, 5, link);
+	read_cis_cache(s, SPACE(tuple->Flags), ofs, 5, link);
 	if ((link[0] == CISTPL_LINKTARGET) && (link[1] >= 3) &&
 	    (strncmp(link+2, "CIS", 3) == 0))
 	    return ofs;
 	/* Then, we try the wrong spot... */
 	ofs = ofs >> 1;
     }
-    read_cis_cache(s, tuple->Flags & IS_ATTR, ofs, 5, link);
+    read_cis_cache(s, SPACE(tuple->Flags), ofs, 5, link);
     if ((link[0] != CISTPL_LINKTARGET) || (link[1] < 3) ||
 	(strncmp(link+2, "CIS", 3) != 0))
 	return -1;
     return ofs;
-} /* follow_link */
+}
 
 int get_next_tuple(client_handle_t handle, tuple_t *tuple)
 {
     socket_info_t *s;
     u_char link[2], tmp;
-    long ofs;
-    int i, attr;
+    int ofs, i, attr;
     
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
@@ -308,36 +359,25 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
     if (setup_cis_mem(s) != 0)
 	return CS_OUT_OF_RESOURCE;
 
-    if (tuple->CISOffset == -1) {
-	ofs = 0;
-	/* Assume presence of a LONGLINK_C to address 0 */
-	tuple->LinkOffset = 0;
-	tuple->Flags = IS_ATTR | LINK_C;
-    }
-    else {
-	if (tuple->TupleLink == 0xff) {
-	    if ((ofs = follow_link(s, tuple)) < 0)
-		return CS_NO_MORE_ITEMS;
-	}
-	else
-	    ofs = tuple->CISOffset +
-		((tuple->Flags & IS_ATTR) ? 2 : 1) * (tuple->TupleLink+2);
-    }
-    attr = tuple->Flags & IS_ATTR;
-    
-    for (i = 0; i < MAX_TUPLES; i++) {
-	read_cis_cache(s, attr, ofs, 2, link);
+    link[1] = tuple->TupleLink;
+    ofs = tuple->CISOffset + tuple->TupleLink;
+    attr = SPACE(tuple->Flags);
 
-	if (link[0] == CISTPL_NULL) {
-	    ofs += attr ? 2 : 1;
-	    continue;
+    for (i = 0; i < MAX_TUPLES; i++) {
+	if (link[1] == 0xff) {
+	    link[0] = CISTPL_END;
+	} else {
+	    read_cis_cache(s, attr, ofs, 2, link);
+	    if (link[0] == CISTPL_NULL) {
+		ofs++; continue;
+	    }
 	}
 	
 	/* End of chain?  Follow long link if possible */
 	if (link[0] == CISTPL_END) {
 	    if ((ofs = follow_link(s, tuple)) < 0)
 		return CS_NO_MORE_ITEMS;
-	    attr = tuple->Flags & IS_ATTR;
+	    attr = SPACE(tuple->Flags);
 	    read_cis_cache(s, attr, ofs, 2, link);
 	}
 
@@ -349,68 +389,61 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
 	    (link[0] == CISTPL_NO_LINK)) {
 	    switch (link[0]) {
 	    case CISTPL_LONGLINK_A:
-		tuple->Flags = attr | LINK_A;
-		read_cis_cache(s, attr, ofs + (attr ? 4 : 2), 4,
-			       (u_char *)&tuple->LinkOffset);
+		HAS_LINK(tuple->Flags) = 1;
+		LINK_SPACE(tuple->Flags) = 1;
+		read_cis_cache(s, attr, ofs+2, 4, &tuple->LinkOffset);
 		break;
 	    case CISTPL_LONGLINK_C:
-		tuple->Flags = attr | LINK_C;
-		read_cis_cache(s, attr, ofs + (attr ? 4 : 2), 4,
-			       (u_char *)&tuple->LinkOffset);
+		HAS_LINK(tuple->Flags) = 1;
+		LINK_SPACE(tuple->Flags) = 0;
+		read_cis_cache(s, attr, ofs+2, 4, &tuple->LinkOffset);
 		break;
 	    case CISTPL_LONGLINK_MFC:
+		tuple->LinkOffset = ofs + 3;
+		LINK_SPACE(tuple->Flags) = attr;
 		if (handle->Function == BIND_FN_ALL) {
 		    /* Follow all the MFC links */
-		    read_cis_cache(s, attr, ofs + (attr ? 4 : 2), 1,
-				 (u_char *)&tmp);
-		    tuple->Flags = attr | LINK_MFC | tmp;
-		    tuple->LinkOffset = ofs + (attr ? 6 : 3);
-		}
-		else {
+		    read_cis_cache(s, attr, ofs+2, 1, &tmp);
+		    MFC_FN(tuple->Flags) = tmp;
+		} else {
 		    /* Follow exactly one of the links */
-		    tuple->Flags = attr | LINK_MFC | 1;
-		    tuple->LinkOffset = ofs + (attr ? 2 : 1) *
-			(3 + handle->Function * 5);
+		    MFC_FN(tuple->Flags) = 1;
+		    tuple->LinkOffset += handle->Function * 5;
 		}
-		if (attr)
-		    tuple->Flags |= MFC_IS_ATTR;
 		break;
 	    case CISTPL_NO_LINK:
-		tuple->Flags = attr;
+		HAS_LINK(tuple->Flags) = 0;
 		break;
 	    }
 	    if ((tuple->Attributes & TUPLE_RETURN_LINK) &&
 		(tuple->DesiredTuple == RETURN_FIRST_TUPLE))
 		break;
-	}
-	else
+	} else
 	    if (tuple->DesiredTuple == RETURN_FIRST_TUPLE)
 		break;
 	
 	if (link[0] == tuple->DesiredTuple)
 	    break;
-	ofs += (attr ? 2 : 1) * (link[1]+2);
+	ofs += link[1] + 2;
     }
     if (i == MAX_TUPLES) {
-#ifdef PCMCIA_DEBUG
-	printk(KERN_DEBUG "cs: overrun in get_next_tuple for "
-	       "socket %d\n", handle->Socket);
-#endif
+	DEBUG(1, "cs: overrun in get_next_tuple for socket %d\n",
+	      handle->Socket);
 	return CS_NO_MORE_ITEMS;
     }
     
     tuple->TupleCode = link[0];
     tuple->TupleLink = link[1];
-    tuple->CISOffset = ofs;
+    tuple->CISOffset = ofs + 2;
     return CS_SUCCESS;
-} /* get_next_tuple */
+}
 
 /*====================================================================*/
 
 int get_tuple_data(client_handle_t handle, tuple_t *tuple)
 {
     socket_info_t *s;
-    u_long len;
+    u_int len;
     
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
@@ -425,14 +458,11 @@ int get_tuple_data(client_handle_t handle, tuple_t *tuple)
     tuple->TupleDataLen = tuple->TupleLink;
     if (len == 0)
 	return CS_SUCCESS;
-    if (tuple->Flags & IS_ATTR)
-	read_cis_cache(s, 1, tuple->CISOffset+((tuple->TupleOffset+2)<<1),
-		       MIN(len, tuple->TupleDataMax), tuple->TupleData);
-    else
-	read_cis_cache(s, 0, tuple->CISOffset+(tuple->TupleOffset+2),
-		       MIN(len, tuple->TupleDataMax), tuple->TupleData);
+    read_cis_cache(s, SPACE(tuple->Flags),
+		   tuple->CISOffset + tuple->TupleOffset,
+		   MIN(len, tuple->TupleDataMax), tuple->TupleData);
     return CS_SUCCESS;
-} /* get_tuple_data */
+}
 
 /*======================================================================
 
@@ -483,7 +513,7 @@ static int parse_device(tuple_t *tuple, cistpl_device_t *device)
     }
     
     return CS_SUCCESS;
-} /* parse_device */
+}
 
 /*====================================================================*/
 
@@ -497,7 +527,7 @@ static int parse_checksum(tuple_t *tuple, cistpl_checksum_t *csum)
     csum->len = *(u_short *)(p + 2);
     csum->sum = *(p+4);
     return CS_SUCCESS;
-} /* parse_checksum */
+}
 
 /*====================================================================*/
 
@@ -505,9 +535,9 @@ static int parse_longlink(tuple_t *tuple, cistpl_longlink_t *link)
 {
     if (tuple->TupleDataLen < 4)
 	return CS_BAD_TUPLE;
-    link->addr = *(u_long *)tuple->TupleData;
+    link->addr = *(u_int *)tuple->TupleData;
     return CS_SUCCESS;
-} /* parse_longlink */
+}
 
 /*====================================================================*/
 
@@ -524,7 +554,7 @@ static int parse_longlink_mfc(tuple_t *tuple,
 	return CS_BAD_TUPLE;
     for (i = 0; i < link->nfn; i++) {
 	link->fn[i].space = *p; p++;
-	link->fn[i].addr = *(u_long *)p; p += 4;
+	link->fn[i].addr = *(u_int *)p; p += 4;
     }
     return CS_SUCCESS;
 }
@@ -552,14 +582,13 @@ static int parse_strings(u_char *p, u_char *q, int max,
     if (found) {
 	*found = ns;
 	return CS_SUCCESS;
-    }
-    else {
+    } else {
 	if (ns == max)
 	    return CS_SUCCESS;
 	else
 	    return CS_BAD_TUPLE;
     }
-} /* parse_strings */
+}
 
 /*====================================================================*/
 
@@ -576,7 +605,7 @@ static int parse_vers_1(tuple_t *tuple, cistpl_vers_1_t *vers_1)
 
     return parse_strings(p, q, CISTPL_VERS_1_MAX_PROD_STRINGS,
 			 vers_1->str, vers_1->ofs, &vers_1->ns);
-} /* parse_vers_1 */
+}
 
 /*====================================================================*/
 
@@ -589,7 +618,7 @@ static int parse_altstr(tuple_t *tuple, cistpl_altstr_t *altstr)
     
     return parse_strings(p, q, CISTPL_MAX_ALTSTR_STRINGS,
 			 altstr->str, altstr->ofs, &altstr->ns);
-} /* parse_altstr */
+}
 
 /*====================================================================*/
 
@@ -609,7 +638,7 @@ static int parse_jedec(tuple_t *tuple, cistpl_jedec_t *jedec)
     }
     jedec->nid = nid;
     return CS_SUCCESS;
-} /* parse_jedec */
+}
 
 /*====================================================================*/
 
@@ -622,7 +651,7 @@ static int parse_manfid(tuple_t *tuple, cistpl_manfid_t *m)
     m->manf = p[0];
     m->card = p[1];
     return CS_SUCCESS;
-} /* parse_manfid */
+}
 
 /*====================================================================*/
 
@@ -635,7 +664,7 @@ static int parse_funcid(tuple_t *tuple, cistpl_funcid_t *f)
     f->func = p[0];
     f->sysinit = p[1];
     return CS_SUCCESS;
-} /* parse_funcid */
+}
 
 /*====================================================================*/
 
@@ -650,7 +679,7 @@ static int parse_funce(tuple_t *tuple, cistpl_funce_t *f)
     for (i = 1; i < tuple->TupleDataLen; i++)
 	f->data[i-1] = p[i];
     return CS_SUCCESS;
-} /* parse_funce */
+}
 
 /*====================================================================*/
 
@@ -676,7 +705,7 @@ static int parse_config(tuple_t *tuple, cistpl_config_t *config)
 	config->rmask[i>>2] += p[i] << (8*(i%4));
     config->subtuples = tuple->TupleDataLen - (rasz+rmsz+4);
     return CS_SUCCESS;
-} /* parse_config */
+}
 
 /*======================================================================
 
@@ -690,7 +719,7 @@ static u_char *parse_power(u_char *p, u_char *q,
 			   cistpl_power_t *pwr)
 {
     int i;
-    u_long scale;
+    u_int scale;
 
     if (p == q) return NULL;
     pwr->present = *p;
@@ -717,7 +746,7 @@ static u_char *parse_power(u_char *p, u_char *q,
 	    p++;
 	}
     return p;
-} /* parse_power */
+}
 
 /*====================================================================*/
 
@@ -747,7 +776,7 @@ static u_char *parse_timing(u_char *p, u_char *q,
     }
     p++;
     return p;
-} /* parse_timing */
+}
 
 /*====================================================================*/
 
@@ -786,14 +815,14 @@ static u_char *parse_io(u_char *p, u_char *q, cistpl_io_t *io)
 	}
     }
     return p;
-} /* parse_io */
+}
 
 /*====================================================================*/
 
 static u_char *parse_mem(u_char *p, u_char *q, cistpl_mem_t *mem)
 {
     int i, j, asz, lsz, has_ha;
-    u_long len, ca, ha;
+    u_int len, ca, ha;
 
     if (p == q) return NULL;
 
@@ -823,7 +852,7 @@ static u_char *parse_mem(u_char *p, u_char *q, cistpl_mem_t *mem)
 	mem->win[i].host_addr = (caddr_t)(ha << 8);
     }
     return p;
-} /* parse_mem */
+}
 
 /*====================================================================*/
 
@@ -837,7 +866,7 @@ static u_char *parse_irq(u_char *p, u_char *q, cistpl_irq_t *irq)
 	p += 2;
     }
     return p;
-} /* parse_irq */
+}
 
 /*====================================================================*/
 
@@ -863,8 +892,7 @@ static int parse_cftable_entry(tuple_t *tuple,
 	if (*p & 0x80)
 	    entry->flags |= CISTPL_CFTABLE_MWAIT;
 	entry->interface = *p & 0x0f;
-    }
-    else
+    } else
 	entry->interface = 0;
 
     /* Process optional features */
@@ -875,25 +903,24 @@ static int parse_cftable_entry(tuple_t *tuple,
     if ((features & 3) > 0) {
 	p = parse_power(p, q, &entry->vcc);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else entry->vcc.present = 0;
+    } else
+	entry->vcc.present = 0;
     if ((features & 3) > 1) {
 	p = parse_power(p, q, &entry->vpp1);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else entry->vpp1.present = 0;
+    } else
+	entry->vpp1.present = 0;
     if ((features & 3) > 2) {
 	p = parse_power(p, q, &entry->vpp2);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else entry->vpp2.present = 0;
+    } else
+	entry->vpp2.present = 0;
 
     /* Timing options */
     if (features & 0x04) {
 	p = parse_timing(p, q, &entry->timing);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else {
+    } else {
 	entry->timing.wait = 0;
 	entry->timing.ready = 0;
 	entry->timing.reserved = 0;
@@ -903,16 +930,14 @@ static int parse_cftable_entry(tuple_t *tuple,
     if (features & 0x08) {
 	p = parse_io(p, q, &entry->io);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else
+    } else
 	entry->io.nwin = 0;
     
     /* Interrupt options */
     if (features & 0x10) {
 	p = parse_irq(p, q, &entry->irq);
 	if (p == NULL) return CS_BAD_TUPLE;
-    }
-    else
+    } else
 	entry->irq.IRQInfo1 = 0;
 
     switch (features & 0x60) {
@@ -944,12 +969,7 @@ static int parse_cftable_entry(tuple_t *tuple,
     /* Misc features */
     if (features & 0x80) {
 	if (p == q) return CS_BAD_TUPLE;
-	if (*p & 0x08)
-	    entry->flags |= CISTPL_CFTABLE_AUDIO;
-	if (*p & 0x10)
-	    entry->flags |= CISTPL_CFTABLE_READONLY;
-	if (*p & 0x20)
-	    entry->flags |= CISTPL_CFTABLE_PWRDOWN;
+	entry->flags |= (*p << 8);
 	while (*p & 0x80)
 	    if (++p == q) return CS_BAD_TUPLE;
 	p++;
@@ -958,7 +978,110 @@ static int parse_cftable_entry(tuple_t *tuple,
     entry->subtuples = q-p;
     
     return CS_SUCCESS;
-} /* parse_cftable_entry */
+}
+
+/*====================================================================*/
+
+#ifdef CONFIG_CARDBUS
+
+static int parse_bar(tuple_t *tuple, cistpl_bar_t *bar)
+{
+    u_char *p;
+    if (tuple->TupleDataLen < 6)
+	return CS_BAD_TUPLE;
+    p = (char *)tuple->TupleData;
+    bar->attr = *p;
+    p += 2;
+    bar->size = p[0] + (p[1]<<8) + (p[2]<<16) + (p[3]<<24);
+    return CS_SUCCESS;
+}
+
+static int parse_config_cb(tuple_t *tuple, cistpl_config_t *config)
+{
+    u_char *p;
+    
+    p = (char *)tuple->TupleData;
+    if ((*p != 3) || (tuple->TupleDataLen < 6))
+	return CS_BAD_TUPLE;
+    config->last_idx = *(++p);
+    p++;
+    config->base = p[0] + (p[1]<<8) + (p[2]<<16) + (p[3]<<24);
+    config->subtuples = tuple->TupleDataLen - 6;
+    return CS_SUCCESS;
+}
+
+static int parse_cftable_entry_cb(tuple_t *tuple,
+				  cistpl_cftable_entry_cb_t *entry)
+{
+    u_char *p, *q, features;
+
+    p = tuple->TupleData;
+    q = p + tuple->TupleDataLen;
+    entry->index = *p & 0x3f;
+    entry->flags = 0;
+    if (*p & 0x40)
+	entry->flags |= CISTPL_CFTABLE_DEFAULT;
+
+    /* Process optional features */
+    if (++p == q) return CS_BAD_TUPLE;
+    features = *p; p++;
+
+    /* Power options */
+    if ((features & 3) > 0) {
+	p = parse_power(p, q, &entry->vcc);
+	if (p == NULL) return CS_BAD_TUPLE;
+    } else
+	entry->vcc.present = 0;
+    if ((features & 3) > 1) {
+	p = parse_power(p, q, &entry->vpp1);
+	if (p == NULL) return CS_BAD_TUPLE;
+    } else
+	entry->vpp1.present = 0;
+    if ((features & 3) > 2) {
+	p = parse_power(p, q, &entry->vpp2);
+	if (p == NULL) return CS_BAD_TUPLE;
+    } else
+	entry->vpp2.present = 0;
+
+    /* I/O window options */
+    if (features & 0x08) {
+	if (p == q) return CS_BAD_TUPLE;
+	entry->io = *p; p++;
+    } else
+	entry->io = 0;
+    
+    /* Interrupt options */
+    if (features & 0x10) {
+	p = parse_irq(p, q, &entry->irq);
+	if (p == NULL) return CS_BAD_TUPLE;
+    } else
+	entry->irq.IRQInfo1 = 0;
+
+    if (features & 0x20) {
+	if (p == q) return CS_BAD_TUPLE;
+	entry->mem = *p; p++;
+    } else
+	entry->mem = 0;
+
+    /* Misc features */
+    if (features & 0x80) {
+	if (p == q) return CS_BAD_TUPLE;
+	entry->flags |= (*p << 8);
+	if (*p & 0x80) {
+	    if (++p == q) return CS_BAD_TUPLE;
+	    entry->flags |= (*p << 16);
+	}
+	while (*p & 0x80)
+	    if (++p == q) return CS_BAD_TUPLE;
+	p++;
+    }
+
+    entry->subtuples = q-p;
+    
+    return CS_SUCCESS;
+}
+
+#endif
 
 /*====================================================================*/
 
@@ -982,7 +1105,7 @@ static int parse_device_geo(tuple_t *tuple, cistpl_device_geo_t *geo)
     }
     geo->ngeo = n;
     return CS_SUCCESS;
-} /* parse_device_geo */
+}
 
 /*====================================================================*/
 
@@ -1004,7 +1127,7 @@ static int parse_vers_2(tuple_t *tuple, cistpl_vers_2_t *v2)
     v2->nhdr = p[8];
     p += 9;
     return parse_strings(p, q, 2, v2->str, &v2->vendor, NULL);
-} /* parse_vers_2 */
+}
 
 /*====================================================================*/
 
@@ -1024,7 +1147,7 @@ static int parse_org(tuple_t *tuple, cistpl_org_t *org)
 	if (++p == q) return CS_BAD_TUPLE;
     }
     return CS_SUCCESS;
-} /* parse_org */
+}
 
 /*====================================================================*/
 
@@ -1039,6 +1162,17 @@ int parse_tuple(client_handle_t handle, tuple_t *tuple, cisparse_t *parse)
     case CISTPL_DEVICE_A:
 	ret = parse_device(tuple, &parse->device);
 	break;
+#ifdef CONFIG_CARDBUS
+    case CISTPL_BAR:
+	ret = parse_bar(tuple, &parse->bar);
+	break;
+    case CISTPL_CONFIG_CB:
+	ret = parse_config_cb(tuple, &parse->config);
+	break;
+    case CISTPL_CFTABLE_ENTRY_CB:
+	ret = parse_cftable_entry_cb(tuple, &parse->cftable_entry_cb);
+	break;
+#endif
     case CISTPL_CHECKSUM:
 	ret = parse_checksum(tuple, &parse->checksum);
 	break;
@@ -1089,7 +1223,7 @@ int parse_tuple(client_handle_t handle, tuple_t *tuple, cisparse_t *parse)
 	break;
     }
     return ret;
-} /* parse_tuple */
+}
 
 /*======================================================================
 
@@ -1104,7 +1238,7 @@ int read_tuple(client_handle_t handle, cisdata_t code, void *parse)
     int ret;
     
     tuple.DesiredTuple = code;
-    tuple.Attributes = 0;
+    tuple.Attributes = TUPLE_RETURN_COMMON;
     ret = CardServices(GetFirstTuple, handle, &tuple, NULL);
     if (ret != CS_SUCCESS) return ret;
     tuple.TupleData = buf;
@@ -1137,7 +1271,7 @@ int validate_cis(client_handle_t handle, cisinfo_t *info)
     
     info->Chains = reserved = errors = 0;
     tuple.DesiredTuple = RETURN_FIRST_TUPLE;
-    tuple.Attributes = 0;
+    tuple.Attributes = TUPLE_RETURN_COMMON;
     ret = get_first_tuple(handle, &tuple);
     if (ret != CS_SUCCESS)
 	return CS_SUCCESS;
@@ -1148,9 +1282,11 @@ int validate_cis(client_handle_t handle, cisinfo_t *info)
     /* All cards should have a MANFID tuple */
     if (read_tuple(handle, CISTPL_MANFID, &parse) != CS_SUCCESS)
 	errors++;
-    /* All cards should have either a VERS_1 or a VERS_2 tuple */
+    /* All cards should have either a VERS_1 or a VERS_2 tuple.  But
+       at worst, we'll accept a CFTABLE_ENTRY that parses. */
     if ((read_tuple(handle, CISTPL_VERS_1, &parse) != CS_SUCCESS) &&
-	(read_tuple(handle, CISTPL_VERS_2, &parse) != CS_SUCCESS))
+	(read_tuple(handle, CISTPL_VERS_2, &parse) != CS_SUCCESS) &&
+	(read_tuple(handle, CISTPL_CFTABLE_ENTRY, &parse) != CS_SUCCESS))
 	errors++;
     if (errors > 1)
 	return CS_SUCCESS;
@@ -1167,5 +1303,5 @@ int validate_cis(client_handle_t handle, cisinfo_t *info)
 	info->Chains = 0;
     
     return CS_SUCCESS;
-} /* validate_cis */
+}
 
