@@ -38,7 +38,7 @@ int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"ds.c 1.70 1997/06/01 05:03:16 (David Hinds)";
+"ds.c 1.76 1997/12/29 17:36:38 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -49,10 +49,12 @@ static void handle_removal(u_long i);
 
 static int ds_event(event_t event, int priority,
 		    event_callback_args_t *args);
-static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
-			 char *buf, U_FS_SIZE_T count);
-static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
-			  CONST char *buf, U_FS_SIZE_T count);
+static FS_SIZE_T ds_read FOPS(struct inode *inode,
+			      struct file *file, char *buf,
+			      U_FS_SIZE_T count, loff_t *ppos);
+static FS_SIZE_T ds_write FOPS(struct inode *inode,
+			       struct file *file, CONST char *buf,
+			       U_FS_SIZE_T count, loff_t *ppos);
 #if (LINUX_VERSION_CODE < VERSION(2,1,23))
 static int ds_select(struct inode *inode, struct file *file,
 		     int sel_type, select_table *wait);
@@ -65,16 +67,16 @@ static int ds_open(struct inode *inode, struct file *file);
 static FS_RELEASE_T ds_release(struct inode *inode, struct file *file);
 
 typedef struct driver_info_t {
-    dev_info_t *dev_info;
-    int use_count;
-    dev_link_t *(*attach)(void);
-    void (*detach)(dev_link_t *);
+    dev_info_t		*dev_info;
+    int			use_count;
+    dev_link_t		*(*attach)(void);
+    void		(*detach)(dev_link_t *);
     struct driver_info_t *next;
 } driver_info_t;
 
 typedef struct socket_bind_t {
-    driver_info_t *driver;
-    dev_link_t *instance;
+    driver_info_t	*driver;
+    dev_link_t		*instance;
     struct socket_bind_t *next;
 } socket_bind_t;
 
@@ -84,21 +86,21 @@ typedef struct socket_bind_t {
 #define CHECK_USER(u) \
     (((u) == NULL) || ((u)->user_magic != USER_MAGIC))
 typedef struct user_info_t {
-    u_long user_magic;
-    int event_head, event_tail;
-    event_t event[MAX_EVENTS];
-    struct user_info_t *next;
+    u_int		user_magic;
+    int			event_head, event_tail;
+    event_t		event[MAX_EVENTS];
+    struct user_info_t	*next;
 } user_info_t;
 
 /* Socket state information */
 typedef struct socket_info_t {
-    client_handle_t handle;
-    long state;
-    user_info_t *user;
-    int req_pending, req_result;
-    struct wait_queue *queue, *request;
-    struct timer_list removal;
-    socket_bind_t *bind;
+    client_handle_t	handle;
+    int			state;
+    user_info_t		*user;
+    int			req_pending, req_result;
+    struct wait_queue	*queue, *request;
+    struct timer_list	removal;
+    socket_bind_t	*bind;
 } socket_info_t;
 
 #define SOCKET_PRESENT		0x01
@@ -161,9 +163,10 @@ EXPORT_SYMBOL(bind_mtd);
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, "ds", (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -263,7 +266,7 @@ static int handle_request(socket_info_t *s, event_t event)
     handle_event(s, event);
     if (s->req_pending > 0) {
 	interruptible_sleep_on(&s->request);
-	if (current->signal & ~current->blocked)
+	if (signal_pending(current))
 	    return CS_IN_USE;
 	else
 	    return s->req_result;
@@ -348,9 +351,9 @@ int bind_mtd(int i, mtd_info_t *mtd_info)
     bind_req.CardOffset = mtd_info->CardOffset;
     ret = CardServices(BindMTD, &bind_req);
     if (ret != CS_SUCCESS) {
-	cs_error(BindMTD, ret);
+	cs_error(NULL, BindMTD, ret);
 	printk(KERN_NOTICE "ds: unable to bind MTD '%s' to socket %d"
-	       " offset 0x%lx\n",
+	       " offset 0x%x\n",
 	       (char *)bind_req.dev_info, i, bind_req.CardOffset);
 	return -ENODEV;
     }
@@ -394,7 +397,7 @@ int bind_request(int i, bind_info_t *bind_info)
     bind_req.dev_info = driver->dev_info;
     ret = CardServices(BindDevice, &bind_req);
     if (ret != CS_SUCCESS) {
-	cs_error(BindDevice, ret);
+	cs_error(NULL, BindDevice, ret);
 	printk(KERN_NOTICE "ds: unable to bind '%s' to socket %d\n",
 	       (char *)dev_info, i);
 	return -ENODEV;
@@ -499,11 +502,6 @@ static int unbind_request(int i, bind_info_t *bind_info)
 
     The user-mode PCMCIA device interface
 
-    I use a really simple scheme for reporting card status changes.
-    When a status change happens, the SOCKET_EVENT_PENDING flag is
-    set.  Reading from the socket device returns the current status,
-    and resets the PENDING flag.
-    
 ======================================================================*/
 
 static int ds_open(struct inode *inode, struct file *file)
@@ -570,10 +568,11 @@ static FS_RELEASE_T ds_release(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
-			 char *buf, U_FS_SIZE_T count)
+static FS_SIZE_T ds_read FOPS(struct inode *inode,
+			      struct file *file, char *buf,
+			      U_FS_SIZE_T count, loff_t *ppos)
 {
-    socket_t i = MINOR(inode->i_rdev);
+    socket_t i = MINOR(F_INODE(file)->i_rdev);
     socket_info_t *s;
     user_info_t *user;
 
@@ -590,19 +589,20 @@ static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
     
     if (queue_empty(user)) {
 	interruptible_sleep_on(&s->queue);
-	if (current->signal & ~current->blocked)
+	if (signal_pending(current))
 	    return -EINTR;
     }
-    put_user(get_queued_event(user), (long *)buf);
+    put_user(get_queued_event(user), (int *)buf);
     return 4;
 } /* ds_read */
 
 /*====================================================================*/
 
-static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
-			  CONST char *buf, U_FS_SIZE_T count)
+static FS_SIZE_T ds_write FOPS(struct inode *inode,
+			       struct file *file, CONST char *buf,
+			       U_FS_SIZE_T count, loff_t *ppos)
 {
-    socket_t i = MINOR(inode->i_rdev);
+    socket_t i = MINOR(F_INODE(file)->i_rdev);
     socket_info_t *s;
     user_info_t *user;
 
@@ -621,11 +621,10 @@ static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
 
     if (s->req_pending) {
 	s->req_pending--;
-	get_user(s->req_result, (long *)buf);
+	get_user(s->req_result, (int *)buf);
 	if ((s->req_result != 0) || (s->req_pending == 0))
 	    wake_up_interruptible(&s->request);
-    }
-    else
+    } else
 	return -EIO;
 
     return 4;
@@ -662,7 +661,7 @@ static int ds_select(struct inode *inode, struct file *file,
 
 static unsigned int ds_poll(struct file *file, poll_table *wait)
 {
-    socket_t i = MINOR(file->f_inode->i_rdev);
+    socket_t i = MINOR(F_INODE(file)->i_rdev);
     socket_info_t *s;
     user_info_t *user;
 
@@ -879,14 +878,14 @@ int init_module(void)
 	bind.Function = BIND_FN_ALL;
 	ret = CardServices(BindDevice, &bind);
 	if (ret != CS_SUCCESS) {
-	    cs_error(BindDevice, ret);
+	    cs_error(NULL, BindDevice, ret);
 	    break;
 	}
 	client_reg.event_callback_args.client_data = &socket_table[i];
 	ret = CardServices(RegisterClient, &socket_table[i].handle,
 			   &client_reg);
 	if (ret != CS_SUCCESS) {
-	    cs_error(RegisterClient, ret);
+	    cs_error(NULL, RegisterClient, ret);
 	    break;
 	}
     }
