@@ -43,7 +43,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"fdomain_cs.c 1.18 1997/06/01 05:03:16 (David Hinds)";
+"fdomain_cs.c 1.25 1998/01/31 19:55:16 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -53,9 +53,11 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /*====================================================================*/
 
@@ -81,9 +83,10 @@ static dev_info_t dev_info = "fdomain_cs";
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*====================================================================*/
@@ -92,7 +95,7 @@ static dev_link_t *fdomain_attach(void)
 {
     client_reg_t client_reg;
     dev_link_t *link;
-    int ret;
+    int i, ret;
     
     DEBUG(0, "fdomain_attach()\n");
 
@@ -109,7 +112,11 @@ static dev_link_t *fdomain_attach(void)
     link->io.IOAddrLines = 10;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
@@ -129,7 +136,7 @@ static dev_link_t *fdomain_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	fdomain_detach(link);
 	return NULL;
     }
@@ -172,17 +179,23 @@ static void fdomain_detach(dev_link_t *link)
 
 /*====================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void fdomain_config(dev_link_t *link)
 {
     client_handle_t handle;
     scsi_info_t *info;
     tuple_t tuple;
     cisparse_t parse;
-    int i, ints[3];
+    int i, last_ret, last_fn, ints[3];
     u_char tuple_data[64];
 #ifdef GET_SCSI_INFO
     Scsi_Device *dev;
     dev_node_t *node, **tail;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,75))
+    struct Scsi_Host *host;
+#endif
 #endif
     
     handle = link->handle;
@@ -190,24 +203,14 @@ static void fdomain_config(dev_link_t *link)
 
     DEBUG(0, "fdomain_config(0x%p)\n", link);
 
-    do {
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	tuple.TupleData = tuple_data;
-	tuple.TupleDataMax = 64;
-	tuple.TupleOffset = 0;
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i != CS_SUCCESS) break;
-	link->conf.ConfigBase = parse.config.base;
-    } while (0);
-    if (i != CS_SUCCESS) {
-	cs_error(ParseTuple, i);
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    tuple.TupleData = tuple_data;
+    tuple.TupleDataMax = 64;
+    tuple.TupleOffset = 0;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
+    link->conf.ConfigBase = parse.config.base;
 
     /* Configure card */
 #if (LINUX_VERSION_CODE >= VERSION(2,1,23))
@@ -216,45 +219,28 @@ static void fdomain_config(dev_link_t *link)
     driver_template.usage_count = &MOD_USE_COUNT;
 #endif
     link->state |= DEV_CONFIG;
-
-    do {
-
-	tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	while (i == CS_SUCCESS) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    if (i != CS_SUCCESS) break;
-	    link->conf.ConfigIndex = parse.cftable_entry.index;
-	    link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i != CS_IN_USE) break;
-	    i = CardServices(GetNextTuple, handle, &tuple);
-	}
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIO, i);
-	    break;
-	}
-	
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    break;
-	}
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, i);
-	    break;
-	}
-
-    } while (0);
-
-    if (i != 0) {
-	fdomain_release((u_long)link);
-	return;
+    
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    i = CardServices(GetFirstTuple, handle, &tuple);
+    while (i == CS_SUCCESS) {
+	i = CardServices(GetTupleData, handle, &tuple);
+	if (i != CS_SUCCESS) break;
+	i = CardServices(ParseTuple, handle, &tuple, &parse);
+	if (i != CS_SUCCESS) break;
+	link->conf.ConfigIndex = parse.cftable_entry.index;
+	link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
+	i = CardServices(RequestIO, handle, &link->io);
+	if (i != CS_IN_USE) break;
+	i = CardServices(GetNextTuple, handle, &tuple);
+    }
+    if (i != CS_SUCCESS) {
+	cs_error(handle, RequestIO, i);
+	goto failed;
     }
 
+    CS_CHECK(RequestIRQ, handle, &link->irq);
+    CS_CHECK(RequestConfiguration, handle, &link->conf);
+    
     /* A bad hack... */
     release_region(link->io.BasePort1, link->io.NumPorts1);
 
@@ -269,8 +255,14 @@ static void fdomain_config(dev_link_t *link)
 #ifdef GET_SCSI_INFO
     tail = &link->dev;
     info->ndev = 0;
+#if (LINUX_VERSION_CODE < VERSION(2,1,75))
     for (dev = scsi_devices; dev != NULL; dev = dev->next)
 	if (dev->host->hostt == &driver_template) {
+#else
+    for (host = scsi_hostlist; host; host = host->next)
+	if (host->hostt == &driver_template)
+	    for (dev = host->host_queue; dev; dev = dev->next) {
+#endif
 	    u_long arg[2], id;
 	    kernel_scsi_ioctl(dev, SCSI_IOCTL_GET_IDLUN, arg);
 	    id = (arg[0]&0x0f) + ((arg[0]>>4)&0xf0) +
@@ -309,6 +301,13 @@ static void fdomain_config(dev_link_t *link)
 #endif /* GET_SCSI_INFO */
     
     link->state &= ~DEV_CONFIG_PENDING;
+    return;
+    
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    fdomain_release((u_long)link);
+    return;
     
 } /* fdomain_config */
 
