@@ -36,7 +36,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"fixed_cs.c 1.13 1997/06/01 05:03:16 (David Hinds)";
+"fixed_cs.c 1.20 1998/02/10 11:38:12 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -46,9 +46,11 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /*====================================================================*/
 
@@ -76,9 +78,10 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -93,7 +96,7 @@ static dev_link_t *fixed_attach(void)
 {
     client_reg_t client_reg;
     dev_link_t *link;
-    int ret;
+    int i, ret;
     
     DEBUG(0, "fixed_attach()\n");
 
@@ -107,7 +110,11 @@ static dev_link_t *fixed_attach(void)
     link->io.IOAddrLines = 3;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
@@ -128,7 +135,7 @@ static dev_link_t *fixed_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != CS_SUCCESS) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	fixed_detach(link);
 	return NULL;
     }
@@ -173,7 +180,7 @@ static void fixed_detach(dev_link_t *link)
     if (link->handle) {
 	ret = CardServices(DeregisterClient, link->handle);
 	if (ret != CS_SUCCESS)
-	    cs_error(DeregisterClient, ret);
+	    cs_error(link->handle, DeregisterClient, ret);
     }
     
     /* Unlink device structure, free bits */
@@ -191,6 +198,9 @@ static void fixed_detach(dev_link_t *link)
 
 ======================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 void fixed_config(dev_link_t *link)
 {
     client_handle_t handle;
@@ -198,7 +208,8 @@ void fixed_config(dev_link_t *link)
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
-    int i, hd, io, ctl;
+    cistpl_cftable_entry_t *cfg = &parse.cftable_entry;
+    int i, last_ret, last_fn, hd, io, ctl;
 
     sti();
     handle = link->handle;
@@ -208,104 +219,87 @@ void fixed_config(dev_link_t *link)
     
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
-    do {
-	tuple.Attributes = 0;
-	/* Get configuration register information */
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i != CS_SUCCESS) break;
-	link->conf.ConfigBase = parse.config.base;
-	link->conf.Present = parse.config.rmask[0];
-    } while (0);
-    
-    if (i != 0) {
-	printk(KERN_NOTICE "fixed_cs: unable to parse CIS\n");
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
+    tuple.Attributes = 0;
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
+    link->conf.ConfigBase = parse.config.base;
+    link->conf.Present = parse.config.rmask[0];
     
     /* Configure card */
     link->state |= DEV_CONFIG;
 
-    do {
-
-	io = ctl = 0;
-	tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-	tuple.Attributes = 0;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	while (i == CS_SUCCESS) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    /* Sanity checks */
-	    if ((i == CS_SUCCESS) && (parse.cftable_entry.io.nwin > 0)) {
-		link->conf.ConfigIndex = parse.cftable_entry.index;
-		link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
-		if (parse.cftable_entry.io.nwin == 2) {
-		    link->io.NumPorts1 = 8;
-		    link->io.BasePort2 = parse.cftable_entry.io.win[1].base;
-		    link->io.NumPorts2 = 1;
-		    i = CardServices(RequestIO, link->handle, &link->io);
-		    io = link->io.BasePort1;
-		    ctl = link->io.BasePort2;
-		}
-		else if (parse.cftable_entry.io.nwin == 1) {
-		    link->io.NumPorts1 = 16;
-		    link->io.NumPorts2 = 0;
-		    i = CardServices(RequestIO, link->handle, &link->io);
-		    io = link->io.BasePort1;
-		    ctl = link->io.BasePort1+0x0e;
-		}
-		else continue;
-		if (i == CS_SUCCESS) break;
+    io = ctl = 0;
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    tuple.Attributes = 0;
+    i = CardServices(GetFirstTuple, handle, &tuple);
+    while (i == CS_SUCCESS) {
+	i = CardServices(GetTupleData, handle, &tuple);
+	if (i != CS_SUCCESS) break;
+	i = CardServices(ParseTuple, handle, &tuple, &parse);
+	if ((i == CS_SUCCESS) &&
+	    (cfg->vpp1.present & (1<<CISTPL_POWER_VNOM)) &&
+	    (cfg->vpp1.param[CISTPL_POWER_VNOM] == 120))
+	    link->conf.Vpp1 = link->conf.Vpp2 = 120;
+	/* Sanity checks */
+	if ((i == CS_SUCCESS) && (cfg->io.nwin > 0)) {
+	    link->conf.ConfigIndex = cfg->index;
+	    link->io.BasePort1 = cfg->io.win[0].base;
+	    if (cfg->io.nwin == 2) {
+		link->io.NumPorts1 = 8;
+		link->io.BasePort2 = cfg->io.win[1].base;
+		link->io.NumPorts2 = 1;
+		i = CardServices(RequestIO, link->handle, &link->io);
+		io = link->io.BasePort1;
+		ctl = link->io.BasePort2;
 	    }
-	    i = CardServices(GetNextTuple, handle, &tuple);
-        }
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIO, i);
-	    break;
+	    else if (cfg->io.nwin == 1) {
+		link->io.NumPorts1 = 16;
+		link->io.NumPorts2 = 0;
+		i = CardServices(RequestIO, link->handle, &link->io);
+		io = link->io.BasePort1;
+		ctl = link->io.BasePort1+0x0e;
+	    }
+	    else continue;
+	    if (i == CS_SUCCESS) break;
 	}
-	
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    break;
-	}
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, i);
-	    break;
-	}
-
-	release_region(link->io.BasePort1, link->io.NumPorts1);
-	if (link->io.NumPorts2)
-	    release_region(link->io.BasePort2, link->io.NumPorts2);
-	hd = ide_register(io, ctl, link->irq.AssignedIRQ);
-	
-	if (hd < 0) {
-	    printk(KERN_NOTICE "fixed_cs: ide_register() at 0x%3x &"
-		   " 0x%3x, irq %ld failed\n", io, ctl,
+	i = CardServices(GetNextTuple, handle, &tuple);
+    }
+    if (i != CS_SUCCESS) {
+	cs_error(link->handle, RequestIO, i);
+	goto failed;
+    }
+    
+    CS_CHECK(RequestIRQ, handle, &link->irq);
+    CS_CHECK(RequestConfiguration, handle, &link->conf);
+    
+    release_region(link->io.BasePort1, link->io.NumPorts1);
+    if (link->io.NumPorts2)
+	release_region(link->io.BasePort2, link->io.NumPorts2);
+    hd = ide_register(io, ctl, link->irq.AssignedIRQ);
+    
+    if (hd < 0) {
+	printk(KERN_NOTICE "fixed_cs: ide_register() at 0x%3x &"
+	       " 0x%3x, irq %u failed\n", io, ctl,
 		   link->irq.AssignedIRQ);
-	    i = -1;
-	    break;
-	}
-	
-	info->ndev = 1;
-	sprintf(info->node.dev_name, "hd%c", 'a'+(hd*2));
-	info->node.major = ide_major[hd];
-	info->node.minor = 0;
-	info->hd = hd;
-	link->dev = &info->node;
-	
-    } while (0);
+	goto failed;
+    }
+    
+    info->ndev = 1;
+    sprintf(info->node.dev_name, "hd%c", 'a'+(hd*2));
+    info->node.major = ide_major[hd];
+    info->node.minor = 0;
+    info->hd = hd;
+    link->dev = &info->node;
 
     link->state &= ~DEV_CONFIG_PENDING;
-    if (i != 0)
-	fixed_release((u_long)link);
+    return;
+    
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    fixed_release((u_long)link);
 
 } /* fixed_config */
 
