@@ -70,13 +70,15 @@ static fmvj18x_debug = 2;
 
 /* Bit map of interrupts to choose from */
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 /* SRAM configuration */
 /* 0:4KB*2 TX buffer   else:8KB*2 TX buffer */
 static int sram_config = 0;
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(sram_config, "i");
 
 /*====================================================================*/
@@ -235,9 +237,10 @@ typedef struct local_info_t {
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*====================================================================*/
@@ -247,7 +250,7 @@ static dev_link_t *fmvj18x_attach(void)
     client_reg_t client_reg;
     dev_link_t *link;
     struct device *dev;
-    int ret;
+    int i, ret;
     
     DEBUG(0, "fmvj18x_attach()\n");
 
@@ -266,7 +269,11 @@ static dev_link_t *fmvj18x_attach(void)
     /* Interrupt setup */
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->irq.Handler = &fjn_interrupt;
     
     /* General socket configuration */
@@ -291,7 +298,7 @@ static dev_link_t *fmvj18x_attach(void)
     dev->open = &fjn_open;
     dev->stop = &fjn_close;
     dev->tbusy = 0xFF;
-    link->priv = dev;
+    link->priv = link->irq.Instance = dev;
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -307,7 +314,7 @@ static dev_link_t *fmvj18x_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	fmvj18x_detach(link);
 	return NULL;
     }
@@ -356,6 +363,9 @@ static void fmvj18x_detach(dev_link_t *link)
 
 /*====================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void fmvj18x_config(dev_link_t *link)
 {
     client_handle_t handle;
@@ -363,11 +373,11 @@ static void fmvj18x_config(dev_link_t *link)
     cisparse_t parse;
     struct device *dev;
     u_short buf[32];
-    int i;
+    int i, last_fn, last_ret;
     short ioaddr;
-    int ret;
     cardtype_t cardtype;
     char *card_name = "unknown";
+    u_char *node_id;
     
     handle = link->handle;
     dev =link->priv;
@@ -378,22 +388,13 @@ static void fmvj18x_config(dev_link_t *link)
        This reads the card's CONFIG tuple to find its configuration
        registers.
     */
-    do {
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	ret = CardServices(GetFirstTuple, handle, &tuple);
-	if (ret != CS_SUCCESS) break;
-	tuple.TupleData = (u_char *)buf;
-	tuple.TupleDataMax = 64;
-	tuple.TupleOffset = 0;
-	ret = CardServices(GetTupleData, handle, &tuple);
-	if (ret != CS_SUCCESS) break;
-	ret = CardServices(ParseTuple, handle, &tuple, &parse);
-    } while (0);
-    if (ret != CS_SUCCESS) {
-	cs_error(ParseTuple, ret);
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    tuple.TupleData = (u_char *)buf;
+    tuple.TupleDataMax = 64;
+    tuple.TupleOffset = 0;
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
     
     /* Configure card */
     link->state |= DEV_CONFIG;
@@ -405,60 +406,48 @@ static void fmvj18x_config(dev_link_t *link)
     tuple.TupleOffset = 0;
     if (CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) {
 	/* Yes, I have CISTPL_FUNCE. Let's check CISTPL_MANFID */
-	tuple.DesiredTuple = CISTPL_MANFID ;
-	if (CardServices(GetFirstTuple, handle, &tuple) != CS_SUCCESS) {
-	    fmvj18x_release((u_long)link);
-	    return;
-	}
-	if (CardServices(GetTupleData, handle, &tuple) != CS_SUCCESS) {
-	    fmvj18x_release((u_long)link);
-	    return;
-	}
+	tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+	CS_CHECK(GetFirstTuple, handle, &tuple);
+	CS_CHECK(GetTupleData, handle, &tuple);
+	CS_CHECK(ParseTuple, handle, &tuple, &parse);
+	link->conf.ConfigIndex = parse.cftable_entry.index;
+	tuple.DesiredTuple = CISTPL_MANFID;
+	CS_CHECK(GetFirstTuple, handle, &tuple);
+	CS_CHECK(GetTupleData, handle, &tuple);
+
 	switch (buf[0]) {
 	case MANFID_TDK:
-	    cardtype = TDK; link->conf.ConfigIndex = 0x04;
+	    cardtype = TDK;
 	    break;
 	case MANFID_CONTEC:
-	    cardtype = CONTEC; link->conf.ConfigIndex = 1;
+	    cardtype = CONTEC;
 	    break;
 	case MANFID_FUJITSU:
-	    cardtype = LA501; link->conf.ConfigIndex = 1;
+	    if (buf[1] == PRODID_FUJITSU_MBH10302)
+		cardtype = MBH10302;
+	    else if (buf[1] == PRODID_FUJITSU_MBH10304)
+		cardtype = MBH10304;
+	    else
+		cardtype = LA501;
 	    break;
 	default:
 	    cardtype = MBH10304;
-	    link->conf.ConfigIndex = 0x60;
 	}
     } else {
 	/* old type card */
 	cardtype = MBH10302;
 	link->conf.ConfigIndex = 1;
     }
-    do {
-	ret = CardServices(RequestIO, link->handle, &link->io);
-	if (ret != CS_SUCCESS) {
-	    cs_error(RequestIO, ret);
-	    goto end;
-	}
-	ret = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (ret != CS_SUCCESS) {
-	    cs_error(RequestIRQ, ret);
-	    goto end;
-	}
-	ret = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (ret != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, ret);
-	    goto end;
-	}
-	dev->irq = link->irq.AssignedIRQ;
-	irq2dev_map[dev->irq] = dev;
-	dev->base_addr = link->io.BasePort1;
-	dev->tbusy = 0;
-	ret = register_netdev(dev);
-	if (ret != 0) {
-	    printk(KERN_NOTICE "fmvj18x_cs: register_netdev() failed\n");
-	    goto end;
-	}
-    } while (0);
+    CS_CHECK(RequestIO, link->handle, &link->io);
+    CS_CHECK(RequestIRQ, link->handle, &link->irq);
+    CS_CHECK(RequestConfiguration, link->handle, &link->conf);
+    dev->irq = link->irq.AssignedIRQ;
+    dev->base_addr = link->io.BasePort1;
+    dev->tbusy = 0;
+    if (register_netdev(dev) != 0) {
+	printk(KERN_NOTICE "fmvj18x_cs: register_netdev() failed\n");
+	goto failed;
+    }
 
     ioaddr = dev->base_addr;
 
@@ -476,40 +465,32 @@ static void fmvj18x_config(dev_link_t *link)
     case TDK:
     case LA501:
     case CONTEC:
-	do {
-	    u_char *node_id;
-	    
-	    tuple.DesiredTuple = CISTPL_FUNCE;
-	    tuple.TupleOffset = 0;
-	    ret = CardServices(GetFirstTuple, handle, &tuple);
-	    if (ret != CS_SUCCESS) goto end;
-	    tuple.TupleOffset = 0;
-	    ret = CardServices(GetTupleData, handle, &tuple);
-	    if (ret != CS_SUCCESS) goto end;
-	    if (cardtype == MBH10304) {
-		/* MBH10304's CIS_FUNCE is corrupted */
-		node_id = &(tuple.TupleData[5]);
-		card_name = "FMV-J182";
-	    }else {
-		while (tuple.TupleData[0] != CISTPL_FUNCE_LAN_NODE_ID ) {
-		    ret = CardServices(GetNextTuple, handle, &tuple) ;
-		    if (ret != CS_SUCCESS) goto end;
-		    ret = CardServices(GetTupleData, handle, &tuple) ;
-		    if (ret != CS_SUCCESS) goto end;
-		}
-		node_id = &(tuple.TupleData[2]);
-		if( cardtype == TDK ) {
-		    card_name = "TDK LAK-CD021";
-		} else if( cardtype == LA501 ) {
-		    card_name = "LA501";
-		} else {
-		    card_name = "C-NET(PC)C";
-		}
+	tuple.DesiredTuple = CISTPL_FUNCE;
+	tuple.TupleOffset = 0;
+	CS_CHECK(GetFirstTuple, handle, &tuple);
+	tuple.TupleOffset = 0;
+	CS_CHECK(GetTupleData, handle, &tuple);
+	if (cardtype == MBH10304) {
+	    /* MBH10304's CIS_FUNCE is corrupted */
+	    node_id = &(tuple.TupleData[5]);
+	    card_name = "FMV-J182";
+	} else {
+	    while (tuple.TupleData[0] != CISTPL_FUNCE_LAN_NODE_ID ) {
+		CS_CHECK(GetNextTuple, handle, &tuple) ;
+		CS_CHECK(GetTupleData, handle, &tuple) ;
 	    }
-	    /* Read MACID from CIS */
-	    for (i = 0; i < 6; i++)
-		dev->dev_addr[i] = node_id[i];
-	} while (0);
+	    node_id = &(tuple.TupleData[2]);
+	    if( cardtype == TDK ) {
+		card_name = "TDK LAK-CD021";
+	    } else if( cardtype == LA501 ) {
+		card_name = "LA501";
+	    } else {
+		card_name = "C-NET(PC)C";
+	    }
+	}
+	/* Read MACID from CIS */
+	for (i = 0; i < 6; i++)
+	    dev->dev_addr[i] = node_id[i];
 	break;
     case MBH10302:
     default:
@@ -529,15 +510,16 @@ static void fmvj18x_config(dev_link_t *link)
 	   dev->name, card_name, sram_config == 0 ? "4K TX*2" : "8K TX*2", 
 	   dev->base_addr, dev->irq);
     for (i = 0; i < 6; i++)
-        printk(" %02X", dev->dev_addr[i]);
-    printk("\n");
+	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
+
+    return;
+    
+cs_failed:
+    /* All Card Services errors end up here */
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    fmvj18x_release((u_long)link);
 
-end:
-    /* If any step failed, release any partially configured state */
-    if (ret != 0) {
-	fmvj18x_release((u_long)link);
-	return;
-    }
 } /* fmvj18x_config */
  
 /*====================================================================*/
@@ -668,7 +650,7 @@ void cleanup_module(void)
 
 static void fjn_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 {
-    struct device *dev = (struct device *)(irq2dev_map[irq]);
+    struct device *dev = (struct device *)DEV_ID;
     int ioaddr;
     local_info_t *lp;
     unsigned short tx_stat, rx_stat;
@@ -851,7 +833,7 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	outb(D_TX_INTR, ioaddr + TX_INTR);
 	outb(D_RX_INTR, ioaddr + RX_INTR);
     }
-    dev_kfree_skb (skb, FREE_WRITE);
+    DEV_KFREE_SKB (skb);
 
     return 0;
 } /* fjn_start_xmit */
@@ -1150,7 +1132,7 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 
     if (num_addrs) {
 	outb(RECV_ALL, ioaddr + RX_MODE);
-    }else {
+    } else {
 	outb(ID_MATCHED, ioaddr + RX_MODE);
     }
 }
