@@ -23,6 +23,7 @@
 #include <linux/major.h>
 #include <linux/fs.h>
 #include <linux/ioctl.h>
+#include <linux/hdreg.h>
 #include <asm/io.h>
 #include <asm/system.h>
 #include <asm/segment.h>
@@ -40,19 +41,31 @@
 #include <pcmcia/ds.h>
 #include <pcmcia/ftl.h>
 
+/*====================================================================*/
+
+/* Parameters that can be set with 'insmod' */
+
 /* Major device # for FTL device */
 static int major_dev = 0;
 
+static int shuffle_freq = 50;
+
+MODULE_PARM(major_dev, "i");
+MODULE_PARM(shuffle_freq, "i");
+
+/*====================================================================*/
+
 /* Funky stuff for setting up a block device */
 #define MAJOR_NR		major_dev
-#define DEVICE_NAME		"FTL"
+#define DEVICE_NAME		"ftl"
 #define DEVICE_REQUEST		do_ftl_request
 #define DEVICE_ON(device)
 #define DEVICE_OFF(device)
 
-#define DEVICE_NR(minor)	((minor)>>3)
-#define REGION_NR(minor)	((minor)&7)
-#define MINOR_NR(dev,rgn)	(((dev)<<3)+(rgn))
+#define DEVICE_NR(minor)	((minor)>>5)
+#define REGION_NR(minor)	(((minor)>>3)&3)
+#define PART_NR(minor)		((minor)&7)
+#define MINOR_NR(dev,reg,part)	(((dev)<<5)+((reg)<<3)+(part))
 
 #include BLK_DEV_HDR
 
@@ -61,17 +74,22 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ftl_cs.c 1.29 1997/06/01 05:03:16 (David Hinds)";
+"ftl_cs.c 1.34 1997/12/29 21:42:22 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
 
-static int shuffle_freq = 50;
-
 /*====================================================================*/
 
 /* Maximum number of separate memory devices we'll allow */
-#define MAX_DEV		8
+#define MAX_DEV		4
+
+/* Maximum number of regions per device */
+#define MAX_REGION	4
+
+/* Maximum number of partitions in an FTL region */
+#define PART_BITS	3
+#define MAX_PART	8
 
 /* Maximum number of outstanding erase requests per socket */
 #define MAX_ERASE	8
@@ -93,29 +111,30 @@ static void do_ftl_request(void);
 /* Each memory region corresponds to a minor device */
 typedef struct partition_t {
     dev_node_t		dev;
-    u_long		state;
-    u_long		*VirtualBlockMap;
-    u_long		*VirtualPageMap;
-    u_long		FreeTotal;
+    u_int		state;
+    u_int		*VirtualBlockMap;
+    u_int		*VirtualPageMap;
+    u_int		FreeTotal;
     struct eun_info_t {
-	u_long			Offset;
-	u_long			EraseCount;
-	u_long			Free;
-	u_long			Deleted;
+	u_int			Offset;
+	u_int			EraseCount;
+	u_int			Free;
+	u_int			Deleted;
     } *EUNInfo;
     struct xfer_info_t {
-	u_long			Offset;
-	u_long			EraseCount;
+	u_int			Offset;
+	u_int			EraseCount;
 	u_short			state;
     } *XferInfo;
     u_short		bam_index;
-    u_long		*bam_cache;
+    u_int		*bam_cache;
     u_short		DataUnits;
-    u_long		BlocksPerUnit;
+    u_int		BlocksPerUnit;
     erase_unit_header_t	header;
     region_info_t	region;
     memory_handle_t	handle;
     int			open;
+    int			locked;
 } partition_t;
 
 /* Partition state flags */
@@ -132,14 +151,31 @@ typedef struct ftl_dev_t {
     eraseq_handle_t	eraseq_handle;
     eraseq_entry_t	eraseq[MAX_ERASE];
     struct wait_queue	*erase_pending;
-    partition_t		minor[2*CISTPL_MAX_DEVICES];
+    partition_t		minor[CISTPL_MAX_DEVICES];
 } ftl_dev_t;
 
 static dev_info_t dev_info = "ftl_cs";
 static dev_link_t *dev_table[MAX_DEV] = { NULL, /* ... */ };
 
-static int ftl_blocksizes[MINOR_NR(MAX_DEV, 0)] =
-{ 0, /* ... */ };
+static struct hd_struct ftl_hd[MINOR_NR(MAX_DEV, 0, 0)];
+static int ftl_sizes[MINOR_NR(MAX_DEV, 0, 0)];
+static int ftl_blocksizes[MINOR_NR(MAX_DEV, 0, 0)];
+
+static struct wait_queue *ftl_wait_open = NULL;
+
+static struct gendisk ftl_gendisk = {
+    0,			/* Major device number */
+    "ftl",
+    PART_BITS,		/* Bits in partition # */
+    MAX_PART,		/* Partitions per device */
+    MAX_DEV*MAX_PART,	/* maximum number of devices */
+    NULL,		/* init function */
+    ftl_hd,		/* hd_struct */
+    ftl_sizes,		/* block sizes */
+    MAX_DEV*MAX_PART,	/* number of "real" devices */
+    NULL,
+    NULL
+};
 
 /*====================================================================*/
 
@@ -147,6 +183,7 @@ static int ftl_ioctl(struct inode *inode, struct file *file,
 		     u_int cmd, u_long arg);
 static int ftl_open(struct inode *inode, struct file *file);
 static FS_RELEASE_T ftl_close(struct inode *inode, struct file *file);
+static int ftl_reread_partitions(int minor);
 
 static struct file_operations ftl_blk_fops = {
     NULL,		/* lseek */
@@ -165,7 +202,12 @@ static struct file_operations ftl_blk_fops = {
 
 static void cs_error(int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    int i;
+    error_info_t err = { func, ret };
+    
+    for (i = 0; i < MAX_DEV; i++)
+	if (dev_table[i] != NULL) break;
+    CardServices(ReportError, dev_table[i]->handle, &err);
 }
 
 /*======================================================================
@@ -314,7 +356,7 @@ static void ftl_config(dev_link_t *link)
 	minor->region = region;
 	sprintf(minor->dev.dev_name, "ftl%dc%d", i, nr);
 	minor->dev.major = major_dev;
-	minor->dev.minor = MINOR_NR(i, nr);
+	minor->dev.minor = MINOR_NR(i, nr, 0);
 	*tail = &minor->dev; tail = &minor->dev.next;
 	minor++; nr++;
 	ret = CardServices(GetNextRegion, link->handle, &region);
@@ -330,9 +372,9 @@ static void ftl_config(dev_link_t *link)
 	minor = dev->minor;
 	for (i = 0; i < nr; i++) {
 	    if (minor[i].region.RegionSize & 0xfffff)
-		printk(" %ld kb", minor[i].region.RegionSize >> 10);
+		printk(" %u kb", minor[i].region.RegionSize >> 10);
 	    else
-		printk(" %ld mb", minor[i].region.RegionSize >> 20);
+		printk(" %u mb", minor[i].region.RegionSize >> 20);
 	}
 	printk("\n");
     }
@@ -424,20 +466,18 @@ static int ftl_event(event_t event, int priority,
 
 /*======================================================================
 
-    Scan_partition() checks to see if a memory region contains an FTL
-    partition.  It reads all the erase unit headers, builds the erase
-    unit map, and then builds the virtual page map.
+    Scan_header() checks to see if a memory region contains an FTL
+    partition.  build_maps() reads all the erase unit headers, builds
+    the erase unit map, and then builds the virtual page map.
     
 ======================================================================*/
 
-static int scan_partition(partition_t *part)
+static int scan_header(partition_t *part)
 {
     erase_unit_header_t header;
     mem_op_t req;
-    u_short xvalid, xtrans, i;
-    u_long blocks, j;
-    int hdr_ok, ret;
-
+    int ret;
+    
     /* Search first megabyte for a valid FTL header */
     req.Attributes = MEM_OP_BUFFER_KERNEL;
     req.Count = sizeof(header);
@@ -462,17 +502,30 @@ static int scan_partition(partition_t *part)
 	return -1;
     }
     part->header = header;
+    return 0;
+}
+
+static int build_maps(partition_t *part)
+{
+    erase_unit_header_t header;
+    mem_op_t req;
+    u_short xvalid, xtrans, i;
+    u_int blocks, j;
+    int hdr_ok, ret;
 
     /* Set up erase unit maps */
-    part->DataUnits = header.NumEraseUnits - header.NumTransferUnits;
+    part->DataUnits = part->header.NumEraseUnits -
+	part->header.NumTransferUnits;
     part->EUNInfo = kmalloc(part->DataUnits * sizeof(struct eun_info_t),
 			    GFP_KERNEL);
     for (i = 0; i < part->DataUnits; i++)
 	part->EUNInfo[i].Offset = 0xffffffff;
     part->XferInfo =
-	kmalloc(header.NumTransferUnits * sizeof(struct xfer_info_t),
+	kmalloc(part->header.NumTransferUnits * sizeof(struct xfer_info_t),
 		GFP_KERNEL);
 
+    req.Attributes = MEM_OP_BUFFER_KERNEL;
+    req.Count = sizeof(header);
     xvalid = xtrans = 0;
     for (i = 0; i < part->header.NumEraseUnits; i++) {
 	req.Offset = ((i + part->header.FirstPhysicalEUN)
@@ -490,8 +543,7 @@ static int scan_partition(partition_t *part)
 	    part->EUNInfo[header.LogicalEUN].EraseCount =
 		header.EraseCount;
 	    xvalid++;
-	}
-	else {
+	} else {
 	    if (xtrans == part->header.NumTransferUnits) {
 		printk(KERN_NOTICE "ftl_cs: format error: too many "
 		       "transfer units!\n");
@@ -500,8 +552,7 @@ static int scan_partition(partition_t *part)
 	    if (hdr_ok && (header.LogicalEUN == 0xffff)) {
 		part->XferInfo[xtrans].state = XFER_PREPARED;
 		part->XferInfo[xtrans].EraseCount = header.EraseCount;
-	    }
-	    else {
+	    } else {
 		part->XferInfo[xtrans].state = XFER_UNKNOWN;
 		/* Pick anything reasonable for the erase count */
 		part->XferInfo[xtrans].EraseCount =
@@ -522,12 +573,12 @@ static int scan_partition(partition_t *part)
     
     /* Set up virtual page map */
     blocks = header.FormattedSize >> header.BlockSize;
-    part->VirtualBlockMap = vmalloc(blocks * sizeof(u_long));
-    memset(part->VirtualBlockMap, 0xff, blocks * sizeof(u_long));
+    part->VirtualBlockMap = vmalloc(blocks * sizeof(u_int));
+    memset(part->VirtualBlockMap, 0xff, blocks * sizeof(u_int));
     part->BlocksPerUnit = (1 << header.EraseUnitSize) >> header.BlockSize;
-    req.Count = part->BlocksPerUnit * sizeof(u_long);
+    req.Count = part->BlocksPerUnit * sizeof(u_int);
 
-    part->bam_cache = kmalloc(part->BlocksPerUnit * sizeof(u_long),
+    part->bam_cache = kmalloc(part->BlocksPerUnit * sizeof(u_int),
 			      GFP_KERNEL);
     part->bam_index = 0xffff;
     part->FreeTotal = 0;
@@ -545,8 +596,7 @@ static int scan_partition(partition_t *part)
 	    if (BLOCK_FREE(part->bam_cache[j])) {
 		part->EUNInfo[i].Free++;
 		part->FreeTotal++;
-	    }
-	    else if ((BLOCK_TYPE(part->bam_cache[j]) == BLOCK_DATA) &&
+	    } else if ((BLOCK_TYPE(part->bam_cache[j]) == BLOCK_DATA) &&
 		     (BLOCK_NUMBER(part->bam_cache[j]) < blocks))
 		part->VirtualBlockMap[BLOCK_NUMBER(part->bam_cache[j])] =
 		    (i << header.EraseUnitSize) + (j << header.BlockSize);
@@ -557,7 +607,7 @@ static int scan_partition(partition_t *part)
     
     return 0;
     
-} /* scan_partition */
+} /* build_maps */
 
 /*======================================================================
 
@@ -573,7 +623,7 @@ static int erase_xfer(ftl_dev_t *dev, partition_t *part,
     struct xfer_info_t *xfer;
 
     xfer = &part->XferInfo[xfernum];
-    DEBUG(1, "ftl_cs: erasing xfer unit at 0x%lx\n", xfer->Offset);
+    DEBUG(1, "ftl_cs: erasing xfer unit at 0x%x\n", xfer->Offset);
     xfer->state = XFER_ERASING;
     /* Is there a free erase slot? */
     for (;;) {
@@ -637,12 +687,12 @@ static void prepare_xfer(partition_t *part, int i)
     mem_op_t req;
     struct xfer_info_t *xfer;
     int nbam, ret;
-    u_long ctl;
+    u_int ctl;
 
     xfer = &part->XferInfo[i];
     xfer->state = XFER_FAILED;
     
-    DEBUG(1, "ftl_cs: preparing xfer unit at 0x%lx\n", xfer->Offset);
+    DEBUG(1, "ftl_cs: preparing xfer unit at 0x%x\n", xfer->Offset);
 
     /* Write the transfer unit header */
     header = part->header;
@@ -658,12 +708,12 @@ static void prepare_xfer(partition_t *part, int i)
     }
 
     /* Write the BAM stub */
-    nbam = (part->BlocksPerUnit * sizeof(u_long) +
+    nbam = (part->BlocksPerUnit * sizeof(u_int) +
 	    part->header.BAMOffset + SECTOR_SIZE - 1) / SECTOR_SIZE;
     req.Offset = xfer->Offset + part->header.BAMOffset;
-    req.Count = sizeof(u_long);
+    req.Count = sizeof(u_int);
     ctl = BLOCK_CONTROL;
-    for (i = 0; i < nbam; i++, req.Offset += sizeof(u_long)) {
+    for (i = 0; i < nbam; i++, req.Offset += sizeof(u_int)) {
 	ret = CardServices(WriteMemory, part->handle, &req, &ctl);
 	if (ret != CS_SUCCESS) {
 	    cs_error(WriteMemory, ret);
@@ -693,13 +743,13 @@ static int copy_erase_unit(partition_t *part, u_short srcunit,
     struct eun_info_t *eun;
     struct xfer_info_t *xfer;
     mem_op_t req;
-    u_long src, dest, free, i;
+    u_int src, dest, free, i;
     u_short unit;
     int ret;
 
     eun = &part->EUNInfo[srcunit];
     xfer = &part->XferInfo[xferunit];
-    DEBUG(2, "ftl_cs: copying block 0x%lx to 0x%lx\n",
+    DEBUG(2, "ftl_cs: copying block 0x%x to 0x%x\n",
 	  eun->Offset, xfer->Offset);
 	
     req.Attributes = MEM_OP_BUFFER_KERNEL;
@@ -707,7 +757,7 @@ static int copy_erase_unit(partition_t *part, u_short srcunit,
     /* Read current BAM */
     if (part->bam_index != srcunit) {
 	req.Offset = eun->Offset + part->header.BAMOffset;
-	req.Count = part->BlocksPerUnit * sizeof(u_long);
+	req.Count = part->BlocksPerUnit * sizeof(u_int);
 	ret = CardServices(ReadMemory, part->handle, &req, part->bam_cache);
 	/* mark the cache bad, in case we get an error later */
 	part->bam_index = 0xffff;
@@ -807,7 +857,7 @@ write_error:
 static int reclaim_block(ftl_dev_t *dev, partition_t *part)
 {
     u_short i, eun, xfer;
-    u_long best;
+    u_int best;
     int queued, ret;
 
     DEBUG(0, "ftl_cs: reclaiming space...\n");
@@ -834,8 +884,7 @@ static int reclaim_block(ftl_dev_t *dev, partition_t *part)
 		DEBUG(1, "ftl_cs: waiting for transfer "
 		      "unit to be prepared...\n");
 		sleep_on(&dev->erase_pending);
-	    }
-	    else {
+	    } else {
 		static int ne = 0;
 		if (++ne < 5)
 		    printk(KERN_NOTICE "ftl_cs: reclaim failed: no "
@@ -854,8 +903,7 @@ static int reclaim_block(ftl_dev_t *dev, partition_t *part)
 		best = part->EUNInfo[i].EraseCount;
 		eun = i;
 	    }
-    }
-    else {
+    } else {
 	best = 0;
 	for (i = 0; i < part->DataUnits; i++)
 	    if (part->EUNInfo[i].Deleted >= best) {
@@ -901,10 +949,10 @@ static void dump_lists(partition_t *part)
 }
 #endif
 
-static u_long find_free(partition_t *part)
+static u_int find_free(partition_t *part)
 {
     u_short stop, eun;
-    u_long blk;
+    u_int blk;
     mem_op_t req;
     int ret;
     
@@ -925,7 +973,7 @@ static u_long find_free(partition_t *part)
 	/* Invalidate cache */
 	part->bam_index = 0xffff;
 	req.Attributes = MEM_OP_BUFFER_KERNEL;
-	req.Count = part->BlocksPerUnit * sizeof(u_long);
+	req.Count = part->BlocksPerUnit * sizeof(u_int);
 	req.Offset = part->EUNInfo[eun].Offset + part->header.BAMOffset;
 	ret = CardServices(ReadMemory, part->handle, &req,
 			   part->bam_cache);
@@ -948,7 +996,7 @@ static u_long find_free(partition_t *part)
 	printk(KERN_NOTICE "ftl_cs: bad free list!\n");
 	return 0;
     }
-    DEBUG(2, "ftl_cs: found free block at %ld in %d\n", blk, eun);
+    DEBUG(2, "ftl_cs: found free block at %d in %d\n", blk, eun);
     return blk;
     
 } /* find_free */
@@ -979,6 +1027,9 @@ static int ftl_open(struct inode *inode, struct file *file)
     partition = &dev->minor[REGION_NR(minor)];
     if (partition->region.RegionSize == 0)
 	return -ENODEV;
+    while (partition->locked)
+	sleep_on(&ftl_wait_open);
+    
     if (partition->handle == NULL) {
 	partition->handle = (memory_handle_t)link->handle;
 	open.Attributes = partition->region.Attributes;
@@ -988,14 +1039,15 @@ static int ftl_open(struct inode *inode, struct file *file)
 	    cs_error(OpenMemory, ret);
 	    return -ENODEV;
 	}
-	if (scan_partition(partition) == 0) {
+	if ((scan_header(partition) == 0) &&
+	    (build_maps(partition) == 0)) {
 	    partition->state = FTL_FORMATTED;
+	    ftl_reread_partitions(minor);
 #ifdef PCMCIA_DEBUG
-	    printk(KERN_INFO "ftl_cs: opening %ld kb FTL partition\n",
+	    printk(KERN_INFO "ftl_cs: opening %d kb FTL partition\n",
 		   partition->header.FormattedSize >> 10);
 #endif
-	}
-	else {
+	} else {
 	    CardServices(CloseMemory, partition->handle);
 	    printk(KERN_NOTICE "ftl_cs: FTL partition is invalid.\n");
 	    return -ENODEV;
@@ -1022,9 +1074,7 @@ static FS_RELEASE_T ftl_close(struct inode *inode, struct file *file)
 
     /* Flush all writes */
     fsync_dev(inode->i_rdev);
-#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
-    invalidate_inodes(inode->i_rdev);
-#endif
+    INVALIDATE_INODES(inode->i_rdev);
     invalidate_buffers(inode->i_rdev);
     
     link = dev_table[DEVICE_NR(minor)];
@@ -1080,7 +1130,8 @@ static int ftl_read(partition_t *part, caddr_t buffer,
 		    u_long sector, u_long nblocks)
 {
     mem_op_t req;
-    u_long log_addr, bsize, i;
+    u_int log_addr, bsize;
+    u_long i;
     int ret;
     
     DEBUG(2, "ftl_cs: ftl_read(0x%p, 0x%lx, %ld)\n",
@@ -1120,25 +1171,25 @@ static int ftl_read(partition_t *part, caddr_t buffer,
     
 ======================================================================*/
 
-static int set_bam_entry(partition_t *part, u_long log_addr,
-			 u_long virt_addr)
+static int set_bam_entry(partition_t *part, u_int log_addr,
+			 u_int virt_addr)
 {
     mem_op_t req;
-    u_long bsize, blk;
+    u_int bsize, blk;
 #ifdef PSYCHO_DEBUG
-    u_long old_addr;
+    u_int old_addr;
 #endif
     u_short eun;
     int ret;
     
-    DEBUG(2, "ftl_cs: set_bam_entry(0x%p, 0x%lx, 0x%lx)\n",
+    DEBUG(2, "ftl_cs: set_bam_entry(0x%p, 0x%x, 0x%x)\n",
 	  part->handle, log_addr, virt_addr);
     bsize = part->region.BlockSize;
     eun = log_addr / bsize;
     blk = (log_addr % bsize) / SECTOR_SIZE;
     req.Attributes = MEM_OP_BUFFER_KERNEL;
-    req.Count = sizeof(u_long);
-    req.Offset = (part->EUNInfo[eun].Offset + blk * sizeof(u_long) +
+    req.Count = sizeof(u_int);
+    req.Offset = (part->EUNInfo[eun].Offset + blk * sizeof(u_int) +
 		  part->header.BAMOffset);
     
 #ifdef PSYCHO_DEBUG
@@ -1149,8 +1200,8 @@ static int set_bam_entry(partition_t *part, u_long log_addr,
 	static int ne = 0;
 	if (++ne < 5) {
 	    printk(KERN_NOTICE "ftl_cs: set_bam_entry() inconsistency!\n");
-	    printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%lx, old = 0x"
-		   "%lx, new = 0x%lx\n", log_addr, old_addr, virt_addr);
+	    printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%x, old = 0x%x"
+		   ", new = 0x%x\n", log_addr, old_addr, virt_addr);
 	}
 	return CS_GENERAL_FAILURE;
     }
@@ -1162,8 +1213,8 @@ static int set_bam_entry(partition_t *part, u_long log_addr,
 	    if (++ne < 5) {
 		printk(KERN_NOTICE "ftl_cs: set_bam_entry() "
 		       "inconsistency!\n");
-		printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%lx, cache"
-		       " = 0x%lx, card = 0x%lx\n",
+		printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%x, cache"
+		       " = 0x%x, card = 0x%x\n",
 		       part->bam_cache[blk], old_addr);
 	    }
 	    return CS_GENERAL_FAILURE;
@@ -1176,8 +1227,8 @@ static int set_bam_entry(partition_t *part, u_long log_addr,
 #ifdef PSYCHO_DEBUG
     if (ret != CS_SUCCESS) {
 	printk(KERN_NOTICE "ftl_cs: set_bam_entry() failed!\n");
-	printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%lx, old = 0x%lx,"
-	       " new = 0x%lx\n", log_addr, old_addr, virt_addr);
+	printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%x, old = 0x%x,"
+	       " new = 0x%x\n", log_addr, old_addr, virt_addr);
 	cs_error(WriteMemory, ret);
     }
 #endif
@@ -1188,7 +1239,8 @@ static int ftl_write(ftl_dev_t *dev, partition_t *part, caddr_t buffer,
 		     u_long sector, u_long nblocks)
 {
     mem_op_t req;
-    u_long bsize, log_addr, virt_addr, old_addr, blk, i;
+    u_int bsize, log_addr, virt_addr, old_addr, blk;
+    u_long i;
     int ret;
 
     DEBUG(2, "ftl_cs: ftl_write(0x%p, %ld, %ld)\n",
@@ -1236,8 +1288,8 @@ static int ftl_write(ftl_dev_t *dev, partition_t *part, caddr_t buffer,
 	ret = CardServices(WriteMemory, part->handle, &req, buffer);
 	if (ret != CS_SUCCESS) {
 	    printk(KERN_NOTICE "ftl_cs: block write failed!\n");
-	    printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%lx, virt_addr"
-		   " = 0x%lx, Offset = 0x%lx\n", log_addr, virt_addr,
+	    printk(KERN_NOTICE "ftl_cs:   log_addr = 0x%x, virt_addr"
+		   " = 0x%x, Offset = 0x%x\n", log_addr, virt_addr,
 		   req.Offset);
 	    cs_error(WriteMemory, ret);
 	    return -EIO;
@@ -1273,31 +1325,44 @@ static int ftl_write(ftl_dev_t *dev, partition_t *part, caddr_t buffer,
 static int ftl_ioctl(struct inode *inode, struct file *file,
 		     u_int cmd, u_long arg)
 {
-    int minor = MINOR(inode->i_rdev);
+    struct hd_geometry *geo = (struct hd_geometry *)arg;
+    int ret = 0, minor = MINOR(inode->i_rdev);
     dev_link_t *link;
     ftl_dev_t *dev;
-    partition_t *partition;
-    int ret = 0;
+    partition_t *part;
+    u_long sect;
 
     link = dev_table[DEVICE_NR(minor)];
     if (!DEV_OK(link)) return -ENODEV;
     dev = (ftl_dev_t *)link->priv;
-    partition = &dev->minor[REGION_NR(minor)];
+    part = &dev->minor[REGION_NR(minor)];
 
     switch (cmd) {
+    case HDIO_GETGEO:
+	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(*geo));
+	if (ret) return ret;
+	/* Sort of arbitrary: round size down to 4K boundary */
+	sect = part->header.FormattedSize/SECTOR_SIZE;
+	put_user(1, (char *)&geo->heads);
+	put_user(8, (char *)&geo->sectors);
+	put_user((sect>>3), (short *)&geo->cylinders);
+	put_user(ftl_hd[minor].start_sect, (u_long *)&geo->start);
+	break;
     case BLKGETSIZE:
-	if (!(partition->state & FTL_FORMATTED)) return -EIO;
 	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
 	if (ret) return ret;
-	put_user(partition->header.FormattedSize/SECTOR_SIZE,
+	put_user(part->header.FormattedSize/SECTOR_SIZE,
 		 (long *)arg);
 	break;
     case BLKFLSBUF:
 	if (!suser()) return -EACCES;
-	if (!(inode->i_rdev)) return -EINVAL;
 	fsync_dev(inode->i_rdev);
 	invalidate_buffers(inode->i_rdev);
 	break;
+    case BLKRRPART:
+	ret = ftl_reread_partitions(minor);
+	break;
+    RO_IOCTLS(inode->i_rdev, arg);
     default:
 	ret = -EINVAL;
     }
@@ -1311,6 +1376,57 @@ static int ftl_ioctl(struct inode *inode, struct file *file,
 
 ======================================================================*/
 
+static int ftl_reread_partitions(int minor)
+{
+    int d = DEVICE_NR(minor), r = REGION_NR(minor);
+    ftl_dev_t *dev = dev_table[d]->priv;
+    partition_t *part = &(dev->minor[r]);
+    int i, whole;
+
+    DEBUG(0, "ftl_cs: ftl_reread_partition(%d)\n", minor);
+    if (part->locked || (part->open > 1))
+	return -EBUSY;
+    part->locked = 1;
+    
+    whole = minor & ~(MAX_PART-1);
+    for (i = 0; i < MAX_PART; i++) {
+	if (ftl_hd[whole+i].nr_sects > 0) {
+	    kdev_t rdev = MKDEV(major_dev, whole+i);
+	    sync_dev(rdev);
+	    INVALIDATE_INODES(rdev);
+	    invalidate_buffers(rdev);
+	}
+	ftl_hd[whole+i].start_sect = 0;
+	ftl_hd[whole+i].nr_sects = 0;
+    }
+
+    if (scan_header(part) == 0)
+	ftl_hd[whole].nr_sects =
+	    part->header.FormattedSize/SECTOR_SIZE;
+    else
+	ftl_hd[whole].start_sect = -1;
+    resetup_one_dev(&ftl_gendisk, whole);
+    
+#ifdef PCMCIA_DEBUG
+    for (i = 0; i < MAX_PART; i++) {
+	if (ftl_hd[whole+i].nr_sects > 0)
+	    printk(KERN_INFO "  %d: start %ld size %ld\n", i,
+		   ftl_hd[whole+i].start_sect,
+		   ftl_hd[whole+i].nr_sects);
+    }
+#endif
+    
+    part->locked = 0;
+    wake_up(&ftl_wait_open);
+    return 0;
+}
+
+/*======================================================================
+
+    Handler for block device requests
+
+======================================================================*/
+
 static void do_ftl_request(void)
 {
     int ret, minor;
@@ -1334,12 +1450,14 @@ static void do_ftl_request(void)
 	switch (CURRENT->cmd) {
 	    
 	case READ:
-	    ret = ftl_read(part, CURRENT->buffer, CURRENT->sector,
+	    ret = ftl_read(part, CURRENT->buffer,
+			   CURRENT->sector+ftl_hd[minor].start_sect,
 			   CURRENT->current_nr_sectors);
 	    break;
 	    
 	case WRITE:
-	    ret = ftl_write(dev, part, CURRENT->buffer, CURRENT->sector,
+	    ret = ftl_write(dev, part, CURRENT->buffer,
+			    CURRENT->sector+ftl_hd[minor].start_sect,
 			    CURRENT->current_nr_sectors);
 	    break;
 	    
@@ -1347,15 +1465,7 @@ static void do_ftl_request(void)
 	    panic("ftl_cs: unknown block command!\n");
 	    
 	}
-	if (ret == 0)
-	    end_request(1);
-	else {
-	    static int ne = 0;
-	    if (++ne < 5)
-		end_request(0);
-	    else
-		end_request(1);
-	}
+	end_request((ret == 0) ? 1 : 0);
     } while (1);
 } /* do_ftl_request */
 
@@ -1377,15 +1487,23 @@ int init_module(void)
     
     register_pcmcia_driver(&dev_info, &ftl_attach, &ftl_detach);
 
-    major_dev = register_blkdev(0, "ftl", &ftl_blk_fops);
+    major_dev = register_blkdev(major_dev, "ftl", &ftl_blk_fops);
     if (major_dev == 0)
 	printk(KERN_NOTICE "ftl_cs: unable to grab major "
 	       "device number!\n");
-
-    blk_dev[major_dev].request_fn = DEVICE_REQUEST;
-    for (i = 0; i < MINOR_NR(MAX_DEV, 0); i++)
+    
+    for (i = 0; i < MINOR_NR(MAX_DEV, 0, 0); i++)
 	ftl_blocksizes[i] = 1024;
+    for (i = 0; i < MAX_DEV*MAX_PART; i++) {
+	ftl_hd[i].nr_sects = 0;
+	ftl_hd[i].start_sect = -1;
+    }
     blksize_size[major_dev] = ftl_blocksizes;
+    ftl_gendisk.major = major_dev;
+    blk_dev[major_dev].request_fn = DEVICE_REQUEST;
+    ftl_gendisk.next = gendisk_head;
+    gendisk_head = &ftl_gendisk;
+    init_waitqueue(&ftl_wait_open);
     
     return 0;
 }
@@ -1394,6 +1512,7 @@ void cleanup_module(void)
 {
     int i;
     dev_link_t *link;
+    struct gendisk *gd, **gdp;
 
     DEBUG(0, "ftl_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
@@ -1407,4 +1526,11 @@ void cleanup_module(void)
 	    ftl_detach(link);
 	}
     }
+    blk_dev[major_dev].request_fn = NULL;
+    blksize_size[major_dev] = NULL;
+    for (gdp = &gendisk_head; *gdp; gdp = &((*gdp)->next))
+	if (*gdp == &ftl_gendisk) {
+	    gd = *gdp; *gdp = gd->next;
+	    break;
+	}
 }
