@@ -35,8 +35,6 @@
 #include <pcmcia/cistpl.h>
 #include <pcmcia/ds.h>
 
-extern struct device *irq2dev_map[16];
-
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
@@ -52,7 +50,8 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 /* MMIO base address */
 static u_long mmiobase = 0;
@@ -70,6 +69,7 @@ static int ringspeed = 16;
 static int hw_addr[6] = { 0, /* ... */ };
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(mmiobase, "i");
 MODULE_PARM(srambase, "i");
 MODULE_PARM(sramsize, "i");
@@ -105,9 +105,10 @@ typedef struct ibmtr_dev_t {
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -126,7 +127,7 @@ static dev_link_t *ibmtr_attach(void)
     ibmtr_dev_t *info;
     struct device *dev;
     struct tok_info *ti;
-    int ret;
+    int i, ret;
 
     DEBUG(0, "ibmtr_attach()\n");
 
@@ -140,7 +141,11 @@ static dev_link_t *ibmtr_attach(void)
     link->io.IOAddrLines = 16;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->irq.Handler = &tok_interrupt;
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
@@ -149,7 +154,7 @@ static dev_link_t *ibmtr_attach(void)
 
     info = kmalloc(sizeof(struct ibmtr_dev_t), GFP_KERNEL);
     memset(info, 0, sizeof(struct ibmtr_dev_t));
-    dev = &info->dev;
+    link->irq.Instance = dev = &info->dev;
 
 #if (LINUX_KERNEL_VERSION > VERSION(2,1,16))
     init_trdev(dev, 0);
@@ -178,7 +183,7 @@ static dev_link_t *ibmtr_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-        cs_error(RegisterClient, ret);
+        cs_error(link->handle, RegisterClient, ret);
         ibmtr_detach(link);
         return NULL;
     }
@@ -247,8 +252,8 @@ static void ibmtr_detach(dev_link_t *link)
 
 ======================================================================*/
 
-#define CS_EXIT_TEST(ret, svc, label) \
-    if (ret != CS_SUCCESS) { cs_error(svc, ret); goto label; }
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void ibmtr_config(dev_link_t *link)
 {
@@ -260,7 +265,7 @@ static void ibmtr_config(dev_link_t *link)
     ibmtr_dev_t *info;
     struct device *dev;
     struct tok_info *ti;
-    int i;
+    int i, last_ret, last_fn;
     u_char buf[64];
     unsigned char Shared_Ram_Base;
 
@@ -272,16 +277,13 @@ static void ibmtr_config(dev_link_t *link)
     DEBUG(0, "ibmtr_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
-    tuple.DesiredTuple = CISTPL_CONFIG;
-    i = CardServices(GetFirstTuple, handle, &tuple);
-    CS_EXIT_TEST(i, GetFirstTuple, config_failed);
     tuple.TupleData = buf;
     tuple.TupleDataMax = 64;
     tuple.TupleOffset = 0;
-    i = CardServices(GetTupleData, handle, &tuple);
-    CS_EXIT_TEST(i, GetTupleData, config_failed);
-    i = CardServices(ParseTuple, handle, &tuple, &parse);
-    CS_EXIT_TEST(i, ParseTuple, config_failed);
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
     link->conf.ConfigBase = parse.config.base;
 
     /* Configure card */
@@ -296,27 +298,19 @@ static void ibmtr_config(dev_link_t *link)
     i = CardServices(RequestIO, link->handle, &link->io);
     if (i == CS_SUCCESS) {
 	memcpy(info->node.dev_name, "tr0\0", 4);
-    }
-    else {
+    } else {
 	/* Couldn't get 0xA20-0xA23.  Try ALTERNATE at 0xA24-0xA27. */
 	link->io.BasePort1 = 0xA24;
-	i = CardServices(RequestIO, link->handle, &link->io);
-	if (i == CS_SUCCESS) {
-	    memcpy(info->node.dev_name, "tr1\0", 4);
-	}
-	else {
-	    CS_EXIT_TEST(i, RequestIO, config_failed);
-	}
+	CS_CHECK(RequestIO, link->handle, &link->io);
+	memcpy(info->node.dev_name, "tr1\0", 4);
     }
 
     dev->base_addr = link->io.BasePort1;
 
-    i = CardServices(RequestIRQ, link->handle, &link->irq);
-    CS_EXIT_TEST(i, RequestIRQ, config_failed);
+    CS_CHECK(RequestIRQ, link->handle, &link->irq);
     dev->irq = link->irq.AssignedIRQ;
     ti->irq = link->irq.AssignedIRQ;
     ti->global_int_enable=GLOBAL_INT_ENABLE+((dev->irq==9) ? 2 : dev->irq);
-    irq2dev_map[info->dev.irq] = dev;
 
     /* Allocate the MMIO memory window */
     req.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
@@ -325,13 +319,11 @@ static void ibmtr_config(dev_link_t *link)
     req.Size = 0x2000;
     req.AccessSpeed = 0x81;
     link->win = (window_handle_t)link->handle;
-    i = CardServices(RequestWindow, &link->win, &req);
-    CS_EXIT_TEST(i, RequestWindow, config_failed);
+    CS_CHECK(RequestWindow, &link->win, &req);
 
     mem.CardOffset = (u_long) req.Base;
     mem.Page = 0;
-    i = CardServices(MapMemPage, link->win, &mem);
-    CS_EXIT_TEST(i, MapMemPage, config_failed);
+    CS_CHECK(MapMemPage, link->win, &mem);
     ti->mmio = (u_long)req.Base;
 
     /* Allocate the SRAM memory window */
@@ -341,20 +333,17 @@ static void ibmtr_config(dev_link_t *link)
     req.Size = sramsize * 1024;
     req.AccessSpeed = 0x81;
     info->sram_win_handle = (window_handle_t)link->handle;
-    i = CardServices(RequestWindow, &info->sram_win_handle, &req);
-    CS_EXIT_TEST(i, RequestWindow, config_failed);
+    CS_CHECK(RequestWindow, &info->sram_win_handle, &req);
 
     mem.CardOffset = (u_long) req.Base;
     mem.Page = 0;
-    i = CardServices(MapMemPage, info->sram_win_handle, &mem);
-    CS_EXIT_TEST(i, MapMemPage, config_failed);
+    CS_CHECK(MapMemPage, info->sram_win_handle, &mem);
     Shared_Ram_Base = (int) req.Base >> 12;
     /*  By setting the ti->sram to NULL, the RRR gets written by ibmtr.c */
     ti->sram = 0;
     ti->sram_base = Shared_Ram_Base;
 
-    i = CardServices(RequestConfiguration, link->handle, &link->conf);
-    CS_EXIT_TEST(i, RequestConfiguration, config_failed);
+    CS_CHECK(RequestConfiguration, link->handle, &link->conf);
 
     /*  Set up the Token-Ring Controller Configuration Register and
         turn on the card.  Check the "Local Area Network Credit Card
@@ -374,7 +363,7 @@ static void ibmtr_config(dev_link_t *link)
 #else
         printk(KERN_NOTICE "ibmtr_cs: register_trdev() failed\n");
 #endif
-        goto config_failed;
+	goto failed;
     }
 
     link->dev = &info->node;
@@ -390,7 +379,9 @@ static void ibmtr_config(dev_link_t *link)
     printk("\n");
     return;
 
-config_failed:    /* CS_EXIT_TEST() calls jump to here... */
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
     ibmtr_release((u_long)link);
 
 } /* ibmtr_config */
@@ -430,8 +421,6 @@ static void ibmtr_release(u_long arg)
     CardServices(ReleaseIRQ, link->handle, &link->irq);
     CardServices(ReleaseWindow, link->win);
     CardServices(ReleaseWindow, info->sram_win_handle);
-    if (info->dev.irq != 0)
-        irq2dev_map[info->dev.irq] = NULL;
 
     link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
     if (link->state & DEV_STALE_LINK)
