@@ -44,7 +44,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"iflash2+_mtd.c 1.31 1997/06/01 05:03:16 (David Hinds)";
+"iflash2+_mtd.c 1.36 1998/01/09 04:19:01 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -59,7 +59,7 @@ static int write_timeout	= HZ/10;	/* in ticks */
 static int erase_timeout	= 100;		/* in ms */
 static int erase_limit		= 10*HZ;	/* in ticks */
 static int retry_limit		= 4;		/* write retries */
-static u_long max_tries       	= 4096;		/* status polling */
+static u_int max_tries       	= 4096;		/* status polling */
 
 MODULE_PARM(vpp_timeout_period, "i");
 MODULE_PARM(vpp_settle, "i");
@@ -88,12 +88,12 @@ static void flash_detach(dev_link_t *);
    simultaneous erases if they are in different cells */
 typedef struct flash_region_t {
     region_info_t	region;
-    u_long		cell_size;
+    u_int		cell_size;
     struct flash_cell_t {
-	u_long		state;
-	u_long		erase_time;
-	u_long		erase_addr;
-	u_long		erase_retries;
+	u_int		state;
+	u_int		erase_time;
+	u_int		erase_addr;
+	u_int		erase_retries;
     } cell[MAX_CELLS];
 } flash_region_t;
 
@@ -102,7 +102,7 @@ typedef struct flash_dev_t {
     window_handle_t	ESRwin;
     caddr_t		ESRbase;
     int			vpp_usage;
-    u_long		vpp_start;
+    u_int		vpp_start;
     struct timer_list	vpp_timeout;
     flash_region_t	*flash[2*CISTPL_MAX_DEVICES];
 } flash_dev_t;
@@ -117,9 +117,10 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 #ifdef PCMCIA_DEBUG
@@ -241,16 +242,14 @@ static int page_setup(struct wait_queue **queue, volatile u_short *esr,
     if (nw == 0) {
 	/* Special case of single byte write */
 	writeb(LOW(IF_SINGLE_LOAD), address);
-    }
-    else {
+    } else {
 	writew(IF_SEQ_LOAD, address);
 	/* Tricky: split count into low bytes and high bytes */
 	nw = (count-nw-1) | ((nw-1) << 8);
 	if ((u_long)address & 2) {
 	    writew(0, address);
 	    writew(nw, address);
-	}
-	else {
+	} else {
 	    writew(nw, address);
 	    writew(0, address);
 	}
@@ -277,16 +276,14 @@ static int page_write(volatile u_short *esr, volatile u_short *address,
 	writeb(LOW(IF_PAGE_WRITE), address);
 	writeb(0, address);
 	writeb(0, address);
-    }
-    else {
+    } else {
 	writew(IF_PAGE_WRITE, address);
 	/* Tricky: split count into low bytes and high bytes */
 	nw = (count-nw-1) | ((nw-1) << 8);
 	if ((u_long)address & 2) {
 	    writew(0, address);
 	    writew(nw, address);
-	}
-	else {
+	} else {
 	    writew(nw, address);
 	    writew(0, address);
 	}
@@ -475,7 +472,7 @@ static dev_link_t *flash_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	flash_detach(link);
 	return NULL;
     }
@@ -520,7 +517,7 @@ static void flash_detach(dev_link_t *link)
     if (link->handle) {
 	ret = CardServices(DeregisterClient, link->handle);
 	if (ret != CS_SUCCESS)
-	    cs_error(DeregisterClient, ret);
+	    cs_error(link->handle, DeregisterClient, ret);
     }
     
     /* Unlink device structure, free bits */
@@ -537,14 +534,14 @@ static void flash_detach(dev_link_t *link)
     
 ======================================================================*/
 
-static void printk_size(u_long sz)
+static void printk_size(u_int sz)
 {
     if (sz & 0x3ff)
-	printk("%ld bytes", sz);
+	printk("%u bytes", sz);
     else if (sz & 0x0fffff)
-	printk("%ld kb", sz >> 10);
+	printk("%u kb", sz >> 10);
     else
-	printk("%ld mb", sz >> 20);
+	printk("%u mb", sz >> 20);
 }
 
 static void flash_config(dev_link_t *link)
@@ -562,9 +559,9 @@ static void flash_config(dev_link_t *link)
     req.Base = NULL; req.Size = WINDOW_SIZE;
     req.AccessSpeed = 0;
     link->win = (window_handle_t)link->handle;
-    ret = CardServices(RequestWindow, &link->win, &req);
+    ret = MTDHelperEntry(MTDRequestWindow, &link->win, &req);
     if (ret != 0) {
-	cs_error(RequestWindow, ret);
+	cs_error(link->handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	flash_release((u_long)link);
 	return;
@@ -575,9 +572,9 @@ static void flash_config(dev_link_t *link)
     /* Allocate a 4K memory window for ESR accesses*/
     req.Base = NULL; req.Size = WINDOW_SIZE;
     dev->ESRwin = (window_handle_t)link->handle;
-    ret = CardServices(RequestWindow, &dev->ESRwin, &req);
+    ret = MTDHelperEntry(MTDRequestWindow, &dev->ESRwin, &req);
     if (ret != 0) {
-	cs_error(RequestWindow, ret);
+	cs_error(link->handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	flash_release((u_long)link);
 	return;
@@ -602,10 +599,10 @@ static void flash_config(dev_link_t *link)
 		kfree_s(dev->flash[i], sizeof(struct flash_region_t));
 		break;
 	    }
-	    printk(KERN_INFO "iflash2+_mtd: %s at 0x%lx, ",
+	    printk(KERN_INFO "iflash2+_mtd: %s at 0x%x, ",
 		   attr ? "attr" : "common", region.CardOffset);
 	    printk_size(region.RegionSize);
-	    printk(", %ld ns\n", region.AccessSpeed);
+	    printk(", %u ns\n", region.AccessSpeed);
 	    memset(dev->flash[i], 0, sizeof(struct flash_region_t));
 	    dev->flash[i]->region = region;
 	    /* Distinguish between 4MB..20MB cards and 40MB cards */
@@ -638,15 +635,15 @@ static void flash_release(u_long arg)
 
     link->state &= ~DEV_CONFIG;
     if (link->win) {
-	int ret = CardServices(ReleaseWindow, link->win);
+	int ret = MTDHelperEntry(MTDReleaseWindow, link->win);
 	if (ret != CS_SUCCESS)
-	    cs_error(ReleaseWindow, ret);
+	    cs_error(link->handle, ReleaseWindow, ret);
     }
     dev = link->priv;
     if (dev->ESRwin) {
-	int ret = CardServices(ReleaseWindow, dev->ESRwin);
+	int ret = MTDHelperEntry(MTDReleaseWindow, dev->ESRwin);
 	if (ret != CS_SUCCESS)
-	    cs_error(ReleaseWindow, ret);
+	    cs_error(link->handle, ReleaseWindow, ret);
     }
     if (dev->vpp_usage == 0)
 	del_timer(&dev->vpp_timeout);
@@ -675,14 +672,14 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     flash_region_t *flash;
     region_info_t *region;
     mtd_mod_win_t mod;
-    u_long from, length, nb, cell;
+    u_int from, length, nb, cell;
     int ret;
 #ifdef PCMCIA_DEBUG
     u_long time;
 #endif
     
-    DEBUG(2, "iflash2+_mtd: flash_read(0x%p, 0x%ld, 0x%p, 0x%lx, "
-	  "0x%lx)\n", link, req->MediaID, buf, req->SrcCardOffset,
+    DEBUG(2, "iflash2+_mtd: flash_read(0x%p, 0x%lx, 0x%p, 0x%x, "
+	  "0x%x)\n", link, req->MediaID, buf, req->SrcCardOffset,
 	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
@@ -766,7 +763,7 @@ done:
 ======================================================================*/
 
 static int basic_write(struct wait_queue **queue, char *esr, char *dest,
-		       char *buf, u_long nb, u_long is_krnl)
+		       char *buf, u_int nb, u_int is_krnl)
 {
     u_short npb;
     int ret;
@@ -830,7 +827,7 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_rdy_req_t rdy;
     flash_region_t *flash;
     region_info_t *region;
-    u_long from, length, nb, retry, cell;
+    u_int from, length, nb, retry, cell;
     status_t status;
     int ret;
 #ifdef PCMCIA_DEBUG
@@ -838,13 +835,13 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 #endif
 
     DEBUG(2, "iflash2+_mtd: flash_write(0x%p, 0x%lx, "
-	  "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
+	  "0x%p, 0x%x, 0x%x)\n", link, req->MediaID, buf,
 	  req->DestCardOffset, req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
     if (ret != CS_SUCCESS) {
-	cs_error(GetStatus, ret);
+	cs_error(link->handle, GetStatus, ret);
 	return CS_GENERAL_FAILURE;
     }
     if (status.CardState & CS_EVENT_WRITE_PROTECT)
@@ -948,9 +945,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_mod_win_t mod;
     int i, ret;
 
-    DEBUG(2, "iflash2+_mtd: flash_erase(0x%p, 0x%lx, "
-	  "0x%lx, 0x%lx)\n", link, req->MediaID,
-	  req->DestCardOffset, req->TransferLength);
+    DEBUG(2, "iflash2+_mtd: flash_erase(0x%p, 0x%lx, 0x%x, 0x%x)\n",
+	  link, req->MediaID, req->DestCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -968,7 +965,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 	/* Check card write protect status */
 	ret = CardServices(GetStatus, link->handle, &status);
 	if (ret != CS_SUCCESS) {
-	    cs_error(GetStatus, ret);
+	    cs_error(link->handle, GetStatus, ret);
 	    return CS_GENERAL_FAILURE;
 	}
 	if (status.CardState & CS_EVENT_WRITE_PROTECT)
@@ -1000,8 +997,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 	/* Disable busy signal during the erase */
 	set_rdy_mode((u_short *)dev->Base, IF_RDY_DISABLE);
 	block_erase((u_short *)dev->Base);
-    }
-    else {
+    } else {
 	/* Check on an already started erase */
 	ret = check_erase((u_short *)dev->Base);
 	if (ret == CS_SUCCESS)
@@ -1011,8 +1007,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 		printk(KERN_NOTICE "iflash2+_mtd: erase failed: "
 		       "too many retries!\n");
 		goto done;
-	    }
-	    else {
+	    } else {
 		flash->cell[i].erase_time = jiffies;
 		abort_cmd((u_short *)dev->Base);
 		block_erase((u_short *)dev->Base);
