@@ -5,7 +5,7 @@
 
     This driver provides the equivalent of /dev/mem for a PCMCIA
     card's attribute and common memory.  It includes character
-    and block devices.
+    and block device support.
 
     Written by David Hinds, dhinds@allegro.stanford.edu
     
@@ -35,6 +35,7 @@
 #include <pcmcia/cistpl.h>
 #include <pcmcia/ds.h>
 #include <pcmcia/memory.h>
+#include <pcmcia/mem_op.h>
 
 /* Major device #'s for memory device */
 static int major_dev = 0;
@@ -46,9 +47,12 @@ static int major_dev = 0;
 #define DEVICE_ON(device)
 #define DEVICE_OFF(device)
 
-#define DEVICE_NR(minor)	((minor)>>3)
+#define DEVICE_NR(minor)	((minor)>>4)
+#define IS_DIRECT(minor)	(((minor)&8)>>3)
+#define REGION_AM(minor)	(((minor)&4)>>2)
 #define REGION_NR(minor)	((minor)&7)
-#define MINOR_NR(dev,attr,rgn)	(((dev)<<3)+((attr)<<2)+(rgn))
+#define MINOR_NR(dev,dir,attr,rgn) \
+(((dev)<<4)+((dir)<<3)+((attr)<<2)+(rgn))
 
 #include BLK_DEV_HDR
 
@@ -57,15 +61,31 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cs.c 1.33 1997/06/01 05:03:16 (David Hinds)";
+"memory_cs.c 1.44 1998/02/11 21:28:16 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
 
+/* Parameters that can be set with 'insmod' */
+
+/* 1 = do 16-bit transfers, 0 = do 8-bit transfers */
+static int word_width = 1;
+
+/* Speed of memory accesses, in ns */
+static int mem_speed = 0;
+
+MODULE_PARM(word_width, "i");
+MODULE_PARM(mem_speed, "i");
+
+/*====================================================================*/
+
 /* Maximum number of separate memory devices we'll allow */
-#define MAX_DEV		8
+#define MAX_DEV		4
+
+/* Maximum number of partitions per memory space */
+#define MAX_PART	4
 
 /* Maximum number of outstanding erase requests per socket */
 #define MAX_ERASE	8
@@ -73,6 +93,12 @@ static char *version =
 /* Sector size -- shouldn't need to change */
 #define SECTOR_SIZE	512
 
+/* Size of the PCMCIA address space: 26 bits = 64 MB */
+#define HIGH_ADDR	0x4000000
+
+/* Size of the direct-access memory window: 4K */
+#define WINDOW_SIZE	0x1000
+
 static void memory_config(dev_link_t *link);
 static void memory_release(u_long arg);
 static int memory_event(event_t event, int priority,
@@ -82,34 +108,46 @@ static dev_link_t *memory_attach(void);
 static void memory_detach(dev_link_t *);
 
 /* Each memory region corresponds to a minor device */
-typedef struct minor_dev_t {
-    dev_node_t		dev;
+typedef struct minor_dev_t {		/* For normal regions */
     region_info_t	region;
     memory_handle_t	handle;
     int			open;
 } minor_dev_t;
 
+typedef struct direct_dev_t {		/* For direct access */
+    int			flags;
+    int			open;
+    caddr_t		Base;
+    u_long		size;
+} direct_dev_t;
+
 typedef struct memory_dev_t {
+    dev_node_t		node;
     eraseq_handle_t	eraseq_handle;
     eraseq_entry_t	eraseq[MAX_ERASE];
     struct wait_queue	*erase_pending;
-    minor_dev_t		minor[2*CISTPL_MAX_DEVICES];
+    direct_dev_t	direct;
+    minor_dev_t		minor[2*MAX_PART];
 } memory_dev_t;
 
+#define MEM_WRPROT	1
+
 static dev_info_t dev_info = "memory_cs";
 static dev_link_t *dev_table[MAX_DEV] = { NULL, /* ... */ };
 
-static int memory_blocksizes[MINOR_NR(MAX_DEV, 0, 0)] =
+static int memory_blocksizes[MINOR_NR(MAX_DEV, 0, 0, 0)] =
 { 0, /* ... */ };
     
 /*====================================================================*/
 
 static int memory_ioctl(struct inode *inode, struct file *file,
 			u_int cmd, u_long arg);
-static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
-			     char *buf, U_FS_SIZE_T count);
-static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
-			      CONST char *buf, U_FS_SIZE_T count);
+static FS_SIZE_T memory_read FOPS(struct inode *inode,
+				  struct file *file, char *buf,
+				  U_FS_SIZE_T count, loff_t *ppos);
+static FS_SIZE_T memory_write FOPS(struct inode *inode,
+				   struct file *file, CONST char *buf,
+				   U_FS_SIZE_T count, loff_t *ppos);
 static int memory_open(struct inode *inode, struct file *file);
 static FS_RELEASE_T memory_close(struct inode *inode,
 				 struct file *file);
@@ -144,9 +182,10 @@ static struct file_operations memory_blk_fops = {
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -198,7 +237,7 @@ static dev_link_t *memory_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	memory_detach(link);
 	return NULL;
     }
@@ -210,7 +249,7 @@ static dev_link_t *memory_attach(void)
     dev->eraseq_handle = (void *)link->handle;
     ret = CardServices(RegisterEraseQueue, &dev->eraseq_handle, &eraseq_hdr);
     if (ret != 0) {
-	cs_error(RegisterEraseQueue, ret);
+	cs_error(link->handle, RegisterEraseQueue, ret);
 	dev->eraseq_handle = NULL;
 	memory_detach(link);
 	return NULL;
@@ -231,14 +270,14 @@ static dev_link_t *memory_attach(void)
 static void memory_detach(dev_link_t *link)
 {
     memory_dev_t *dev;
-    int i;
+    int nd;
 
     DEBUG(0, "memory_detach(0x%p)\n", link);
     
-    /* Locate device structure */
-    for (i = 0; i < MAX_DEV; i++)
-	if (dev_table[i] == link) break;
-    if (i == MAX_DEV)
+    /* Verify device address */
+    for (nd = 0; nd < MAX_DEV; nd++)
+	if (dev_table[nd] == link) break;
+    if (nd == MAX_DEV)
 	return;
 
     if (link->state & DEV_CONFIG) {
@@ -256,12 +295,73 @@ static void memory_detach(dev_link_t *link)
 	CardServices(DeregisterClient, link->handle);
     
     /* Unlink device structure, free bits */
-    dev_table[i] = NULL;
+    dev_table[nd] = NULL;
     kfree_s(dev, sizeof(struct memory_dev_t));
     kfree_s(link, sizeof(struct dev_link_t));
     
 } /* memory_detach */
 
+/*======================================================================
+
+    Figure out the size of a simple SRAM card
+    
+======================================================================*/
+
+static void get_size(dev_link_t *link, direct_dev_t *direct)
+{
+    modwin_t mod;
+    memreq_t mem;
+    u_char buf[26];
+    int s, t, ret;
+
+    mod.Attributes = WIN_ENABLE | WIN_MEMORY_TYPE_CM;
+    mod.AccessSpeed = 0;
+    ret = CardServices(ModifyWindow, link->win, &mod);
+    if (ret != CS_SUCCESS)
+	cs_error(link->handle, ModifyWindow, ret);
+
+    /* Look for wrap-around or dead end */
+    mem.Page = 0;
+    for (s = 12; s < 26; s++) {
+	mem.CardOffset = 1<<s;
+	CardServices(MapMemPage, link->win, &mem);
+	buf[s] = readb(direct->Base);
+	writeb(~buf[s], direct->Base);
+	for (t = 12; t < s; t++) {
+	    mem.CardOffset = 1<<t;
+	    CardServices(MapMemPage, link->win, &mem);
+	    if (readb(direct->Base) != buf[t]) {
+		writeb(buf[t], direct->Base);
+		break;
+	    }
+	}
+	if (t < s) break;
+	mem.CardOffset = 1<<s;
+	CardServices(MapMemPage, link->win, &mem);
+	if (readb(direct->Base) != (0xff & ~buf[s])) break;
+	writeb(buf[s], direct->Base);
+    }
+
+    /* Restore that last byte on wrap-around */
+    if (t < s) {
+	mem.CardOffset = 1<<t;
+	CardServices(MapMemPage, link->win, &mem);
+	writeb(buf[t], direct->Base);
+    }
+
+    direct->size = (t > 15) ? (1<<t) : 0;
+} /* get_size */
+
+static void print_size(u_long sz)
+{
+    if (sz & 0x03ff)
+	printk("%ld bytes", sz);
+    else if (sz & 0x0fffff)
+	printk("%ld kb", sz >> 10);
+    else
+	printk("%ld mb", sz >> 20);
+}
+
 /*======================================================================
 
     memory_config() is scheduled to run after a CARD_INSERTION event
@@ -270,67 +370,98 @@ static void memory_detach(dev_link_t *link)
     
 ======================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void memory_config(dev_link_t *link)
 {
     memory_dev_t *dev;
     minor_dev_t *minor;
-    dev_node_t **tail;
     region_info_t region;
-    int i, attr, ret, nr[2];
+    status_t status;
+    win_req_t req;
+    int nd, i, last_ret, last_fn, attr, ret, nr[2];
 
     DEBUG(0, "memory_config(0x%p)\n", link);
 
     /* Configure card */
     link->state |= DEV_CONFIG;
 
-    for (i = 0; i < MAX_DEV; i++)
-	if (dev_table[i] == link) break;
+    for (nd = 0; nd < MAX_DEV; nd++)
+	if (dev_table[nd] == link) break;
     
     dev = (memory_dev_t *)link->priv;
-    tail = &link->dev;
+
+    /* Allocate a 4K memory window for direct access */
+    if (word_width)
+	req.Attributes = WIN_DATA_WIDTH_16;
+    else
+	req.Attributes = WIN_DATA_WIDTH_8;
+    req.Base = NULL; req.Size = WINDOW_SIZE;
+    req.AccessSpeed = mem_speed;
+    link->win = (window_handle_t)link->handle;
+    CS_CHECK(RequestWindow, &link->win, &req);
+    /* Get write protect status */
+    CS_CHECK(GetStatus, link->handle, &status);
+    
+    dev->direct.Base = req.Base;
+    dev->direct.size = 0;
+
     for (attr = 0; attr < 2; attr++) {
-	minor = dev->minor + attr*CISTPL_MAX_DEVICES;
 	nr[attr] = 0;
+	minor = dev->minor + attr*MAX_PART;
 	region.Attributes =
 	    (attr) ? REGION_TYPE_AM : REGION_TYPE_CM;
 	ret = CardServices(GetFirstRegion, link->handle, &region);
 	while (ret == CS_SUCCESS) {
 	    minor->region = region;
-	    sprintf(minor->dev.dev_name, "mem%d%c%d",
-		    i, attr ? 'a' : 'c', nr[attr]);
-	    minor->dev.major = major_dev;
-	    minor->dev.minor = MINOR_NR(i, attr, nr[attr]);
-	    *tail = &minor->dev; tail = &minor->dev.next;
 	    minor++; nr[attr]++;
 	    ret = CardServices(GetNextRegion, link->handle, &region);
 	}
     }
-    *tail = NULL;
     
+    sprintf(dev->node.dev_name, "mem%d", nd);
+    dev->node.major = major_dev;
+    dev->node.minor = MINOR_NR(nd, 0, 0, 0);
+    link->dev = &dev->node;
     link->state &= ~DEV_CONFIG_PENDING;
     
-    if ((nr[0] == 0) && (nr[1] == 0))
-	printk(KERN_NOTICE "memory_cs: no regions found!\n");
-    else {
-	printk(KERN_INFO "memory_cs: mem%d:", i);
+    printk(KERN_INFO "memory_cs: mem%d:", nd);
+    if ((nr[0] == 0) && (nr[1] == 0)) {
+	cisinfo_t cisinfo;
+	if ((CardServices(ValidateCIS, link->handle, &cisinfo)
+	     == CS_SUCCESS) && (cisinfo.Chains == 0)) {
+	    get_size(link, &dev->direct);
+	    printk(" anonymous: ");
+	    if (dev->direct.size == 0) {
+		dev->direct.size = HIGH_ADDR;
+		printk("unknown size");
+	    } else {
+		print_size(dev->direct.size);
+	    }
+	} else {
+	    printk(" no regions found.");
+	}
+    } else {
 	for (attr = 0; attr < 2; attr++) {
-	    minor = dev->minor + attr*CISTPL_MAX_DEVICES;
+	    minor = dev->minor + attr*MAX_PART;
 	    if (attr && nr[0] && nr[1])
 		printk(",");
 	    if (nr[attr])
 		printk(" %s", attr ? "attribute" : "common");
 	    for (i = 0; i < nr[attr]; i++) {
-		if (minor[i].region.RegionSize & 0x03ff)
-		    printk(" %ld bytes", minor[i].region.RegionSize);
-		else if (minor[i].region.RegionSize & 0x0fffff)
-		    printk(" %ld kb", minor[i].region.RegionSize >> 10);
-		else
-		    printk(" %ld mb", minor[i].region.RegionSize >> 20);
+		printk(" ");
+		print_size(minor[i].region.RegionSize);
 	    }
 	}
-	printk("\n");
     }
-    
+    printk("\n");
+    return;
+
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+    memory_release((u_long)link);
+    return;
 } /* memory_config */
 
 /*======================================================================
@@ -344,15 +475,15 @@ static void memory_config(dev_link_t *link)
 static void memory_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
-    int i;
+    int nd;
 
     DEBUG(0, "memory_release(0x%p)\n", link);
 
-    for (i = 0; i < MAX_DEV; i++)
-	if (dev_table[i] == link) break;
+    for (nd = 0; nd < MAX_DEV; nd++)
+	if (dev_table[nd] == link) break;
     if (link->open) {
 	DEBUG(0, "memory_cs: release postponed, 'mem%d'"
-	      " still open\n", i);
+	      " still open\n", nd);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -442,20 +573,28 @@ static int memory_open(struct inode *inode, struct file *file)
 	return -ENODEV;
     
     dev = (memory_dev_t *)link->priv;
-    minor_dev = &dev->minor[REGION_NR(minor)];
-    if (minor_dev->region.RegionSize == 0)
-	return -ENODEV;
-    if (minor_dev->handle == NULL) {
-	minor_dev->handle = (memory_handle_t)link->handle;
-	open.Attributes = minor_dev->region.Attributes;
-	open.Offset = minor_dev->region.CardOffset;
-	ret = CardServices(OpenMemory, &minor_dev->handle, &open);
-	if (ret != CS_SUCCESS)
-	    return -ENOMEM;
+
+    if (IS_DIRECT(minor) || (dev->direct.size > 0)) {
+	if ((file->f_mode & 2) && (dev->direct.flags & MEM_WRPROT))
+	    return -EROFS;
+	dev->direct.open++;
+	file->private_data = NULL;
+    } else {
+	minor_dev = &dev->minor[REGION_NR(minor)];
+	if (minor_dev->region.RegionSize == 0)
+	    return -ENODEV;
+	if (minor_dev->handle == NULL) {
+	    minor_dev->handle = (memory_handle_t)link->handle;
+	    open.Attributes = minor_dev->region.Attributes;
+	    open.Offset = minor_dev->region.CardOffset;
+	    ret = CardServices(OpenMemory, &minor_dev->handle, &open);
+	    if (ret != CS_SUCCESS)
+		return -ENOMEM;
+	}
+	minor_dev->open++;
+	file->private_data = minor_dev;
     }
-    file->private_data = minor_dev;
     
-    minor_dev->open++;
     link->open++;
     MOD_INC_USE_COUNT;
     return 0;
@@ -475,14 +614,18 @@ static FS_RELEASE_T memory_close(struct inode *inode,
 
     link = dev_table[DEVICE_NR(minor)];
     dev = (memory_dev_t *)link->priv;
-    minor_dev = &dev->minor[REGION_NR(minor)];
-    link->open--;
-    minor_dev->open--;
-    if (minor_dev->open == 0) {
-	CardServices(CloseMemory, minor_dev->handle);
-	minor_dev->handle = NULL;
+    if (IS_DIRECT(minor) || (dev->direct.size > 0)) {
+	dev->direct.open--;
+    } else {
+	minor_dev = &dev->minor[REGION_NR(minor)];
+	minor_dev->open--;
+	if (minor_dev->open == 0) {
+	    CardServices(CloseMemory, minor_dev->handle);
+	    minor_dev->handle = NULL;
+	}
     }
 
+    link->open--;
     MOD_DEC_USE_COUNT;
     return (FS_RELEASE_T)0;
 } /* memory_close */
@@ -491,9 +634,7 @@ static FS_RELEASE_T memory_blk_close(struct inode *inode,
 				     struct file *file)
 {
     fsync_dev(inode->i_rdev);
-#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
-    invalidate_inodes(inode->i_rdev);
-#endif
+    INVALIDATE_INODES(inode->i_rdev);
     invalidate_buffers(inode->i_rdev);
     return memory_close(inode, file);
 }
@@ -504,24 +645,91 @@ static FS_RELEASE_T memory_blk_close(struct inode *inode,
     
 ======================================================================*/
 
-static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
-			     char *buf, U_FS_SIZE_T count)
+static FS_SIZE_T direct_read FOPS(struct inode *inode,
+				  struct file *file, char *buf,
+				  U_FS_SIZE_T count, loff_t *ppos)
+{
+    int minor = MINOR(F_INODE(file)->i_rdev);
+    dev_link_t *link;
+    memory_dev_t *dev;
+    U_FS_SIZE_T size, pos, read, from, nb;
+    int ret;
+    modwin_t mod;
+    memreq_t mem;
+
+    DEBUG(2, "direct_read(%d, 0x%lx, %ld)\n", minor,
+	  (u_long)FPOS, (u_long)count);
+
+    link = dev_table[DEVICE_NR(minor)];
+    
+    if (!DEV_OK(link))
+	return -ENODEV;
+    dev = (memory_dev_t *)link->priv;
+
+    /* Boundary checks */
+    if (count < 0)
+	return -EINVAL;
+    pos = FPOS;
+    size = (IS_DIRECT(minor)) ? HIGH_ADDR : dev->direct.size;
+    if (pos >= size)
+	return 0;
+    if (count > size - pos)
+	count = size - pos;
+
+    mod.Attributes = WIN_ENABLE;
+    mod.Attributes |= (REGION_AM(minor)) ? WIN_MEMORY_TYPE_AM : 0;
+    mod.AccessSpeed = 0;
+    ret = CardServices(ModifyWindow, link->win, &mod);
+    if (ret != CS_SUCCESS) {
+	cs_error(link->handle, ModifyWindow, ret);
+	return -EIO;
+    }
+    
+    mem.CardOffset = pos & ~(WINDOW_SIZE-1);
+    mem.Page = 0;
+    from = pos & (WINDOW_SIZE-1);
+    for (read = 0; count > 0; count -= nb, read += nb) {
+	ret = CardServices(MapMemPage, link->win, &mem);
+	if (ret != CS_SUCCESS) {
+	    cs_error(link->handle, MapMemPage, ret);
+	    return -EIO;
+	}
+	nb = (from+count > WINDOW_SIZE) ? WINDOW_SIZE-from : count;
+	copy_pc_to_user(buf, dev->direct.Base+from, nb);
+	buf += nb;
+        from = 0;
+	mem.CardOffset += WINDOW_SIZE;
+    }
+
+    FPOS += read;
+    return read;
+} /* direct_read */
+
+static FS_SIZE_T memory_read FOPS(struct inode *inode,
+				  struct file *file, char *buf,
+				  U_FS_SIZE_T count, loff_t *ppos)
 {
     minor_dev_t *minor;
     mem_op_t req;
     int ret;
 
     minor = file->private_data;
+    if (minor == NULL)
+	return direct_read FOPS(inode, file, buf, count, ppos);
+    
     DEBUG(2, "memory_read(0x%p, 0x%lx, %ld)\n", minor->handle,
-	  (u_long)file->f_pos, (u_long)count);
+	  (u_long)FPOS, (u_long)count);
+    
     req.Attributes = MEM_OP_BUFFER_USER;
-    req.Offset = file->f_pos;
+    req.Offset = FPOS;
     req.Count = count;
     ret = CardServices(ReadMemory, minor->handle, &req, buf);
     if (ret == CS_SUCCESS) {
-	file->f_pos += count;
+	FPOS += count;
 	return count;
-    } else
+    } else if (ret == CS_BAD_OFFSET)
+	return 0;
+    else
 	return -EIO;
 } /* memory_read */
 
@@ -533,9 +741,11 @@ static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
     
 ======================================================================*/
 
-static int memory_erase(memory_dev_t *dev, minor_dev_t *minor_dev,
-			u_long f_pos, int count)
+static int memory_erase(int minor, u_long f_pos, U_FS_SIZE_T count)
 {
+    dev_link_t *link = dev_table[DEVICE_NR(minor)];
+    memory_dev_t *dev = link->priv;
+    minor_dev_t *minor_dev = &dev->minor[REGION_NR(minor)];
     int i, ret;
 
     /* Find a free erase slot, or wait for one to become available */
@@ -545,7 +755,7 @@ static int memory_erase(memory_dev_t *dev, minor_dev_t *minor_dev,
 	if (i < MAX_ERASE) break;
 	DEBUG(2, "waiting for erase slot...\n");
 	interruptible_sleep_on(&dev->erase_pending);
-	if (current->signal & ~current->blocked)
+	if (signal_pending(current))
 	    return -ERESTARTSYS;
     }
 
@@ -556,7 +766,7 @@ static int memory_erase(memory_dev_t *dev, minor_dev_t *minor_dev,
     dev->eraseq[i].Size = count;
     ret = CardServices(CheckEraseQueue, dev->eraseq_handle);
     if (ret != CS_SUCCESS) {
-	cs_error(CheckEraseQueue, ret);
+	cs_error(link->handle, CheckEraseQueue, ret);
 	return -EIO;
     }
 
@@ -575,36 +785,101 @@ static int memory_erase(memory_dev_t *dev, minor_dev_t *minor_dev,
     
 ======================================================================*/
 
-static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
-			      CONST char *buf, U_FS_SIZE_T count)
+static FS_SIZE_T direct_write FOPS(struct inode *inode,
+				   struct file *file, CONST char *buf,
+				   U_FS_SIZE_T count, loff_t *ppos)
 {
-    int minor = MINOR(inode->i_rdev);
+    int minor = MINOR(F_INODE(file)->i_rdev);
+    dev_link_t *link;
     memory_dev_t *dev;
-    minor_dev_t *minor_dev;
+    U_FS_SIZE_T size, pos, wrote, to, nb;
+    int ret;
+    modwin_t mod;
+    memreq_t mem;
+    
+    DEBUG(2, "direct_write(%d, 0x%lx, %ld)\n", minor,
+	  (u_long)FPOS, (u_long)count);
+    
+    link = dev_table[DEVICE_NR(minor)];
+    
+    if (!DEV_OK(link))
+	return -ENODEV;
+    
+    dev = (memory_dev_t *)link->priv;
+    
+    /* Check for write protect */
+    if (dev->direct.flags & MEM_WRPROT)
+	return -EROFS;
+
+    /* Boundary checks */
+    if (count < 0)
+	return -EINVAL;
+    size = (IS_DIRECT(minor)) ? HIGH_ADDR : dev->direct.size;
+    pos = FPOS;
+    if (pos >= size)
+        return -ENOSPC;
+    if (count > size - pos)
+	count = size - pos;
+
+    mod.Attributes = WIN_ENABLE;
+    mod.Attributes |= (REGION_AM(minor)) ? WIN_MEMORY_TYPE_AM : 0;
+    mod.AccessSpeed = 0;
+    ret = CardServices(ModifyWindow, link->win, &mod);
+    if (ret != CS_SUCCESS) {
+	cs_error(link->handle, ModifyWindow, ret);
+	return -EIO;
+    }
+    
+    mem.CardOffset = pos & ~(WINDOW_SIZE-1);
+    mem.Page = 0;
+    to = pos & (WINDOW_SIZE-1);
+    for (wrote = 0; count > 0; count -= nb, wrote += nb) {
+	ret = CardServices(MapMemPage, link->win, &mem);
+	if (ret != CS_SUCCESS) {
+	    cs_error(link->handle, MapMemPage, ret);
+	    return -EIO;
+	}
+	nb = (to+count > WINDOW_SIZE) ? WINDOW_SIZE-to : count;
+	copy_user_to_pc(dev->direct.Base+to, buf, nb);
+	buf += nb;
+        to = 0;
+	mem.CardOffset += WINDOW_SIZE;
+    }
+
+    FPOS += wrote;
+    return wrote;
+} /* direct_write */
+
+static FS_SIZE_T memory_write FOPS(struct inode *inode,
+				   struct file *file, CONST char *buf,
+				   U_FS_SIZE_T count, loff_t *ppos)
+{
+    minor_dev_t *minor;
     mem_op_t req;
     int ret;
 
-    dev = (memory_dev_t *)dev_table[DEVICE_NR(minor)]->priv;
-    minor_dev = &dev->minor[REGION_NR(minor)];
-    DEBUG(2, "memory_write(0x%p, 0x%lx, %ld)\n", minor_dev->handle,
-	  (u_long)file->f_pos, (u_long)count);
-    if ((minor_dev->region.BlockSize > 1) &&
-	((file->f_pos & (minor_dev->region.BlockSize-1)) == 0) &&
-	((count & (minor_dev->region.BlockSize-1)) == 0)) {
-	ret = memory_erase(dev, minor_dev, file->f_pos, count);
+    minor = file->private_data;
+    if (minor == NULL)
+	return direct_write FOPS(inode, file, buf, count, ppos);
+    
+    DEBUG(2, "memory_write(0x%p, 0x%lx, %ld)\n", minor->handle,
+	  (u_long)FPOS, (u_long)count);
+    if ((minor->region.BlockSize > 1) &&
+	((FPOS & (minor->region.BlockSize-1)) == 0) &&
+	((count & (minor->region.BlockSize-1)) == 0)) {
+	ret = memory_erase(MINOR(F_INODE(file)->i_rdev), FPOS, count);
 	if (ret != 0)
 	    return ret;
     }
     
     req.Attributes = 0;
-    req.Offset = file->f_pos;
+    req.Offset = FPOS;
     req.Count = count;
-    ret = CardServices(WriteMemory, minor_dev->handle, &req, buf);
+    ret = CardServices(WriteMemory, minor->handle, &req, buf);
     if (ret == CS_SUCCESS) {
-	file->f_pos += count;
+	FPOS += count;
 	return count;
-    }
-    else
+    } else
 	return -EIO;
 } /* memory_write */
 
@@ -615,7 +890,7 @@ static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
 ======================================================================*/
 
 static int memory_ioctl(struct inode *inode, struct file *file,
-		       u_int cmd, u_long arg)
+			u_int cmd, u_long arg)
 {
     int minor = MINOR(inode->i_rdev);
     dev_link_t *link;
@@ -644,8 +919,11 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 
     switch (cmd) {
     case BLKGETSIZE:
-	put_user(minor_dev->region.RegionSize/SECTOR_SIZE,
-		 (long *)arg);
+	if (!IS_DIRECT(minor))
+	    put_user(dev->direct.size/SECTOR_SIZE, (long *)arg);
+	else
+	    put_user(minor_dev->region.RegionSize/SECTOR_SIZE,
+		     (long *)arg);
 	break;
     case BLKFLSBUF:
 	if (!suser()) return -EACCES;
@@ -654,13 +932,17 @@ static int memory_ioctl(struct inode *inode, struct file *file,
 	invalidate_buffers(inode->i_rdev);
 	break;
     case MEMGETINFO:
-	copy_to_user((region_info_t *)arg, &minor_dev->region,
-		     sizeof(struct region_info_t));
+	if (!IS_DIRECT(minor)) {
+	    copy_to_user((region_info_t *)arg, &minor_dev->region,
+			 sizeof(struct region_info_t));
+	} else ret = -EINVAL;
 	break;
     case MEMERASE:
-	copy_from_user(&erase, (erase_info_t *)arg,
-		       sizeof(struct erase_info_t));
-	ret = memory_erase(dev, minor_dev, erase.Offset, erase.Size);
+	if (!IS_DIRECT(minor)) {
+	    copy_from_user(&erase, (erase_info_t *)arg,
+			   sizeof(struct erase_info_t));
+	    ret = memory_erase(minor, erase.Offset, erase.Size);
+	} else ret = -EINVAL;
 	break;
     default:
 	ret = -EINVAL;
@@ -675,6 +957,60 @@ static int memory_ioctl(struct inode *inode, struct file *file,
     
 ======================================================================*/
 
+
+static void do_direct_request(dev_link_t *link)
+{
+    int addr, len, from, nb, ret;
+    char *buf;
+    memory_dev_t *dev;
+    modwin_t mod;
+    memreq_t mem;
+    
+    dev = (memory_dev_t *)link->priv;
+    
+    addr = CURRENT->sector * SECTOR_SIZE;
+    len = CURRENT->current_nr_sectors * SECTOR_SIZE;
+    if ((addr + len) > dev->direct.size) {
+	end_request(0);
+	return;
+    }
+    
+    mod.Attributes = WIN_ENABLE;
+    mod.AccessSpeed = 0;
+    ret = CardServices(ModifyWindow, link->win, &mod);
+    if (ret != CS_SUCCESS) {
+	cs_error(link->handle, ModifyWindow, ret);
+	end_request(0);
+	return;
+    }
+
+    buf = CURRENT->buffer;
+    mem.Page = 0;
+    mem.CardOffset = addr & ~(WINDOW_SIZE-1);
+    from = addr & (WINDOW_SIZE-1);
+    ret = 0;
+    
+    if ((CURRENT->cmd == READ) || (CURRENT->cmd == WRITE))
+	for ( ; len > 0; len -= nb, buf += nb, from = 0) {
+	    ret = CardServices(MapMemPage, link->win, &mem);
+	    if (ret != CS_SUCCESS) break;
+	    nb = (from+len > WINDOW_SIZE) ? WINDOW_SIZE-from : len;
+	    if (CURRENT->cmd == READ)
+		copy_from_pc(buf, &dev->direct.Base[from], nb);
+	    else
+		copy_to_pc(&dev->direct.Base[from], buf, nb);
+	    mem.CardOffset += WINDOW_SIZE;
+	}
+    else panic("pcmem_cs: unknown block command!\n");
+    
+    if (ret == CS_SUCCESS)
+	end_request(1);
+    else {
+	cs_error(link->handle, MapMemPage, ret);
+	end_request(0);
+    }
+} /* do_direct_request */
+
 static void do_memory_request(void)
 {
     int ret, minor;
@@ -689,9 +1025,14 @@ static void do_memory_request(void)
 	INIT_REQUEST;
 
 	minor = MINOR(DEVICE(CURRENT));
-	
 	link = dev_table[DEVICE_NR(minor)];
 	dev = (memory_dev_t *)link->priv;
+
+	if (IS_DIRECT(minor) || (dev->direct.size > 0)) {
+	    do_direct_request(link);
+	    continue;
+	}
+	
 	minor_dev = &dev->minor[REGION_NR(minor)];
 
 	req.Attributes = MEM_OP_BUFFER_KERNEL;
@@ -700,26 +1041,19 @@ static void do_memory_request(void)
 	buf = CURRENT->buffer;
 	ret = CS_SUCCESS;
 	
-	switch (CURRENT->cmd) {
-	    
-	case READ:
+	if (CURRENT->cmd == READ) {
 	    ret = CardServices(ReadMemory, minor_dev->handle,
 			       &req, buf);
 	    if (ret != CS_SUCCESS)
-		cs_error(ReadMemory, ret);
-	    break;
-	    
-	case WRITE:
+		cs_error(link->handle, ReadMemory, ret);
+	} else if (CURRENT->cmd == WRITE) {
 	    ret = CardServices(WriteMemory, minor_dev->handle,
 			       &req, buf);
 	    if (ret != CS_SUCCESS)
-		cs_error(WriteMemory, ret);
-	    break;
-	    
-	default:
+		cs_error(link->handle, WriteMemory, ret);
+	} else
 	    panic("memory_cs: unknown block command!\n");
-	    
-	}
+	
 	if (ret == CS_SUCCESS)
 	    end_request(1);
 	else
@@ -759,7 +1093,7 @@ int init_module(void)
 	major_dev = i;
 
     blk_dev[major_dev].request_fn = DEVICE_REQUEST;
-    for (i = 0; i < MINOR_NR(MAX_DEV, 0, 0); i++)
+    for (i = 0; i < MINOR_NR(MAX_DEV, 0, 0, 0); i++)
 	memory_blocksizes[i] = 1024;
     blksize_size[major_dev] = memory_blocksizes;
     
