@@ -422,10 +422,12 @@ static int if_port=0; /* default=auto */
    */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask=0xdeb8; /* 3, 4, 5, 7, 9, 10, 11, 12, 14, 15 */
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(if_port, "i");
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /* ----------------------------------------------------------------------------
 Function Prototypes
@@ -460,11 +462,12 @@ static void nmclan_detach(dev_link_t *);
 cs_error
 	Report a Card Services related error.
 ---------------------------------------------------------------------------- */
-static void cs_error(int func, int ret)
-{
-  CardServices(ReportError, dev_info, (void *)func, (void *)ret);
-} /* cs_error */
 
+static void cs_error(client_handle_t handle, int func, int ret)
+{
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
+}
 
 /* ----------------------------------------------------------------------------
 nmclan_init
@@ -488,7 +491,7 @@ static dev_link_t *nmclan_attach(void)
   client_reg_t client_reg;
   dev_link_t *link;
   struct device *dev;
-  int ret;
+  int i, ret;
 
   DEBUG(0, "nmclan_attach()\n");
   DEBUG(1, "%s\n", rcsid);
@@ -503,7 +506,11 @@ static dev_link_t *nmclan_attach(void)
   link->io.IOAddrLines = 5;
   link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
   link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-  link->irq.IRQInfo2 = irq_mask;
+  if (irq_list[0] == -1)
+    link->irq.IRQInfo2 = irq_mask;
+  else
+    for (i = 0; i < 4; i++)
+      link->irq.IRQInfo2 |= 1 << irq_list[i];
   link->irq.Handler = &mace_interrupt;
   link->conf.Attributes = CONF_ENABLE_IRQ;
   link->conf.Vcc = 50;
@@ -529,7 +536,7 @@ static dev_link_t *nmclan_attach(void)
   dev->open = &mace_open;
   dev->stop = &mace_close;
   dev->tbusy = 0xFF;
-  link->priv = dev;
+  link->priv = link->irq.Instance = dev;
 
   /* Register with Card Services */
   link->next = dev_list;
@@ -545,7 +552,7 @@ static dev_link_t *nmclan_attach(void)
   client_reg.event_callback_args.client_data = link;
   ret = CardServices(RegisterClient, &link->handle, &client_reg);
   if (ret != 0) {
-    cs_error(RegisterClient, ret);
+    cs_error(link->handle, RegisterClient, ret);
     nmclan_detach(link);
     return NULL;
   }
@@ -727,6 +734,10 @@ nmclan_config
 	is received, to configure the PCMCIA socket, and to make the
 	ethernet device available to the system.
 ---------------------------------------------------------------------------- */
+
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void nmclan_config(dev_link_t *link)
 {
   client_handle_t handle;
@@ -734,7 +745,7 @@ static void nmclan_config(dev_link_t *link)
   tuple_t tuple;
   cisparse_t parse;
   u_char buf[64];
-  int i;
+  int i, last_ret, last_fn;
   short ioaddr, *phys_addr;
 
 
@@ -744,77 +755,41 @@ static void nmclan_config(dev_link_t *link)
 
   DEBUG(0, "nmclan_config(0x%p)\n", link);
 
-  do {
-    tuple.Attributes = 0;
-    tuple.DesiredTuple = CISTPL_CONFIG;
-    i = CardServices(GetFirstTuple, handle, &tuple);
-    if (i != CS_SUCCESS) break;
-    tuple.TupleData = buf;
-    tuple.TupleDataMax = 64;
-    tuple.TupleOffset = 0;
-    i = CardServices(GetTupleData, handle, &tuple);
-    if (i != CS_SUCCESS) break;
-    i = CardServices(ParseTuple, handle, &tuple, &parse);
-    if (i != CS_SUCCESS) break;
-    link->conf.ConfigBase = parse.config.base;
-  } while (0);
-  if (i != CS_SUCCESS) {
-    printk(KERN_NOTICE "nmclan_cs: unable to parse CIS\n");
-    link->state &= ~DEV_CONFIG_PENDING;
-    return;
-  }
+  tuple.Attributes = 0;
+  tuple.TupleData = buf;
+  tuple.TupleDataMax = 64;
+  tuple.TupleOffset = 0;
+  tuple.DesiredTuple = CISTPL_CONFIG;
+  CS_CHECK(GetFirstTuple, handle, &tuple);
+  CS_CHECK(GetTupleData, handle, &tuple);
+  CS_CHECK(ParseTuple, handle, &tuple, &parse);
+  link->conf.ConfigBase = parse.config.base;
 
   /* Configure card */
   link->state |= DEV_CONFIG;
 
-  do {
-    i = CardServices(RequestIO, link->handle, &link->io);
-    if (i != CS_SUCCESS) {
-      cs_error(RequestIO, i);
-      break;
-    }
-    i = CardServices(RequestIRQ, link->handle, &link->irq);
-    if (i != CS_SUCCESS) {
-      cs_error(RequestIRQ, i);
-      break;
-    }
-    i = CardServices(RequestConfiguration, link->handle, &link->conf);
-    if (i != CS_SUCCESS) {
-      cs_error(RequestConfiguration, i);
-      break;
-    }
-    dev->irq = link->irq.AssignedIRQ;
-    dev->base_addr = link->io.BasePort1;
-    dev->tbusy = 0;
-    i = register_netdev(dev);
-    if (i != 0) {
-      printk(KERN_NOTICE "nmclan_cs: register_netdev() failed\n");
-      break;
-    }
-  } while (0);
-
+  CS_CHECK(RequestIO, handle, &link->io);
+  CS_CHECK(RequestIRQ, handle, &link->irq);
+  CS_CHECK(RequestConfiguration, handle, &link->conf);
+  dev->irq = link->irq.AssignedIRQ;
+  dev->base_addr = link->io.BasePort1;
+  dev->tbusy = 0;
+  i = register_netdev(dev);
+  if (i != 0) {
+    printk(KERN_NOTICE "nmclan_cs: register_netdev() failed\n");
+    goto failed;
+  }
+  
   ioaddr = dev->base_addr;
 
-
   /* Read the ethernet address from the CIS. */
-  do {
-    tuple.DesiredTuple = 0x80 /* CISTPL_CFTABLE_ENTRY_MISC */;
-    i = CardServices(GetFirstTuple, handle, &tuple);
-    if (i != CS_SUCCESS) break;
-    tuple.TupleData = buf;
-    tuple.TupleDataMax = 64;
-    tuple.TupleOffset = 0;
-    i = CardServices(GetTupleData, handle, &tuple);
-    if (i != CS_SUCCESS) break;
-    memcpy(dev->dev_addr, tuple.TupleData, ETHER_ADDR_LEN);
-  } while (0);
-  if (i != CS_SUCCESS) {
-    printk(KERN_NOTICE
-	   "nmclan_cs: unable to parse Ethernet address from CIS\n");
-    link->state &= ~DEV_CONFIG_PENDING;
-    return;
-  }
-
+  tuple.DesiredTuple = 0x80 /* CISTPL_CFTABLE_ENTRY_MISC */;
+  tuple.TupleData = buf;
+  tuple.TupleDataMax = 64;
+  tuple.TupleOffset = 0;
+  CS_CHECK(GetFirstTuple, handle, &tuple);
+  CS_CHECK(GetTupleData, handle, &tuple);
+  memcpy(dev->dev_addr, tuple.TupleData, ETHER_ADDR_LEN);
 
   /* Verify configuration by reading the MACE ID. */
   {
@@ -864,12 +839,18 @@ static void nmclan_config(dev_link_t *link)
   link->dev = &((mace_private *)dev->priv)->node;
   link->state &= ~DEV_CONFIG_PENDING;
   
-  printk(KERN_INFO "%s: nmclan, io %#3lx, irq %d, %s port,", dev->name,
-	 dev->base_addr, dev->irq, if_names[dev->if_port]);
-  for (i = 0; i < ETHER_ADDR_LEN; i++)
-      printk(" %02X", dev->dev_addr[i]);
-  printk("\n");
+  printk(KERN_INFO "%s: nmclan: port %#3lx, irq %d, %s port, hw_addr ",
+	 dev->name, dev->base_addr, dev->irq, if_names[dev->if_port]);
+  for (i = 0; i < 6; i++)
+      printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
+  return;
 
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    nmclan_release((u_long)link);
+    return;
+    
 } /* nmclan_config */
 
 
@@ -899,8 +880,6 @@ static void nmclan_release(u_long arg)
   CardServices(ReleaseConfiguration, link->handle);
   CardServices(ReleaseIO, link->handle, &link->io);
   CardServices(ReleaseIRQ, link->handle, &link->irq);
-  if (dev->irq != 0)
-    irq2dev_map[dev->irq] = NULL;
 
   link->state &= ~DEV_CONFIG;
   if (link->state & DEV_STALE_LINK)
@@ -922,7 +901,7 @@ static int nmclan_event(event_t event, int priority,
   dev_link_t *link = args->client_data;
   struct device *dev = link->priv;
 
-  DEBUG(1 "nmclan_event(0x%06x)\n", event);
+  DEBUG(1, "nmclan_event(0x%06x)\n", event);
     
   switch (event) {
     case CS_EVENT_CARD_REMOVAL:
@@ -1069,8 +1048,6 @@ static int mace_open(struct device *dev)
 
   MACEBANK(0);
 
-  irq2dev_map[dev->irq] = dev;
-
   dev->interrupt = 0;
   dev->tbusy = 0;
   dev->start = 1;
@@ -1166,7 +1143,7 @@ static int mace_start_xmit(struct sk_buff *skb, struct device *dev)
 #endif
  
   DEBUG(3, "%s: mace_start_xmit(length = %ld) called.\n",
-	dev->name, skb->len);
+	dev->name, (long)skb->len);
   
   /* Avoid timer-based retransmission conflicts. */
   if (test_and_set_bit(TBUSY_UNSPECIFIED, (void*)&dev->tbusy) != 0) {
@@ -1223,7 +1200,7 @@ static int mace_start_xmit(struct sk_buff *skb, struct device *dev)
   outb(MACE_IMR_DEFAULT, ioaddr + AM2150_MACE_BASE + MACE_IMR);
 #endif /* #if (!TX_INTERRUPTABLE) */
 
-  dev_kfree_skb(skb, FREE_WRITE);
+  DEV_KFREE_SKB(skb);
     
   return 0;
 } /* mace_start_xmit */
@@ -1235,7 +1212,7 @@ mace_interrupt
 ---------------------------------------------------------------------------- */
 static void mace_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 {
-  struct device *dev = (struct device *)(irq2dev_map[irq]);
+  struct device *dev = (struct device *)DEV_ID;
   mace_private *lp = (mace_private *)dev->priv;
   int ioaddr = dev->base_addr;
   int status;
@@ -1837,16 +1814,15 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 
 static void restore_multicast_list(struct device *dev)
 {
-  mace_private *lp = (mace_private *)dev->priv;
   short ioaddr = dev->base_addr;
 
-  DEBUG(2, "%s: restoring Rx mode to %d addresses.\n",
-	dev->name, lp->multicast_num_addrs);
+  DEBUG(2, "%s: restoring Rx mode to %d addresses.\n", dev->name,
+	((mace_private *)(dev->priv))->multicast_num_addrs);
   
 #ifdef NEW_MULTICAST
   if (dev->flags & IFF_PROMISC) {
 #else
-  if (lp->multicast_num_addrs < 0) {
+  if (((mace_private *)(dev->priv))->multicast_num_addrs < 0) {
 #endif
     /* Promiscuous mode: receive all packets */
     mace_write(ioaddr, MACE_UTR, MACE_UTR_LOOP_EXTERNAL);
@@ -1870,7 +1846,7 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
   mace_private *lp = (mace_private *)dev->priv;
 
 #ifdef PCMCIA_DEBUG
-  if (nmclan_debug > 1) {
+  if (pc_debug > 1) {
     static int old = 0;
     if (num_addrs != old) {
       old = num_addrs;
