@@ -71,7 +71,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"pcnet_cs.c 1.52 1997/06/01 05:03:16 (David Hinds)";
+"pcnet_cs.c 1.63 1997/12/29 21:30:55 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -81,7 +81,8 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 /* Transceiver type, for Socket EA and IBM CC cards. */
 static int if_port = 1;
@@ -105,6 +106,7 @@ static int use_shmem = -1;
 static int hw_addr[6] = { 0, /* ... */ };
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(if_port, "i");
 MODULE_PARM(use_big_buf, "i");
 MODULE_PARM(mem_speed, "i");
@@ -142,7 +144,6 @@ static dev_link_t *dev_list;
 /*====================================================================*/
 
 typedef struct hw_info_t {
-    char	*id;
     u_long	offset;
     u_char	a0, a1, a2;
     u_long	flags;
@@ -156,59 +157,59 @@ typedef struct hw_info_t {
 #define IS_MULTIFN	0x20
 
 static hw_info_t hw_info[] = {
-    { "Accton EN2212", 0x0ff0, 0x00, 0x00, 0xe8, DELAY_OUTPUT },
-    { "Allied Telesis LA-PCM", 0x0ff0, 0x00, 0x00, 0xf4, USE_SHMEM },
-    { "APEX MultiCard", 0x03f4, 0x00, 0x20, 0xe5, 0 },
-    { "ASANTE FriendlyNet", 0x4910, 0x00, 0x00, 0x94,
+    { /* Accton EN2212 */ 0x0ff0, 0x00, 0x00, 0xe8, DELAY_OUTPUT }, 
+    { /* Allied Telesis LA-PCM */ 0x0ff0, 0x00, 0x00, 0xf4, USE_SHMEM },
+    { /* APEX MultiCard */ 0x03f4, 0x00, 0x20, 0xe5, 0 },
+    { /* ASANTE FriendlyNet */ 0x4910, 0x00, 0x00, 0x94,
       DELAY_OUTPUT | HAS_IBM_MISC },
-    { "Danpex EN-6200P2", 0x0110, 0x00, 0x40, 0xc7, 0 },
-    { "DataTrek NetCard", 0x0ff0, 0x00, 0x20, 0xe8, 0 },
-    { "Dayna CommuniCard E", 0x0110, 0x00, 0x80, 0x19, 0 },
-    { "D-Link DE-650", 0x0040, 0x00, 0x80, 0xc8, 0 },
-    { "EP-210 Ethernet", 0x0110, 0x00, 0x40, 0x33, 0 },
-    { "EP4000 Ethernet", 0x01c0, 0x00, 0x00, 0xb4, 0 },
-    { "Epson EEN10B", 0x0ff0, 0x00, 0x00, 0x48,
+    { /* Danpex EN-6200P2 */ 0x0110, 0x00, 0x40, 0xc7, 0 },
+    { /* DataTrek NetCard */ 0x0ff0, 0x00, 0x20, 0xe8, 0 },
+    { /* Dayna CommuniCard E */ 0x0110, 0x00, 0x80, 0x19, 0 },
+    { /* D-Link DE-650 */ 0x0040, 0x00, 0x80, 0xc8, 0 },
+    { /* EP-210 Ethernet */ 0x0110, 0x00, 0x40, 0x33, 0 },
+    { /* EP4000 Ethernet */ 0x01c0, 0x00, 0x00, 0xb4, 0 },
+    { /* Epson EEN10B */ 0x0ff0, 0x00, 0x00, 0x48,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "ELECOM Laneed LD-CDWA", 0xb8, 0x08, 0x00, 0x42, 0 },
-    { "Grey Cell GCS2220", 0x0000, 0x00, 0x47, 0x43, 0 },
-    { "Hypertec Ethernet", 0x01c0, 0x00, 0x40, 0x4c, 0 },
-    { "IBM CCAE", 0x0ff0, 0x08, 0x00, 0x5a,
+    { /* ELECOM Laneed LD-CDWA */ 0xb8, 0x08, 0x00, 0x42, 0 },
+    { /* Hypertec Ethernet */ 0x01c0, 0x00, 0x40, 0x4c, 0 },
+    { /* IBM CCAE */ 0x0ff0, 0x08, 0x00, 0x5a,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM CCAE", 0x0ff0, 0x00, 0x04, 0xac,
+    { /* IBM CCAE */ 0x0ff0, 0x00, 0x04, 0xac,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM CCAE", 0x0ff0, 0x00, 0x06, 0x29,
+    { /* IBM CCAE */ 0x0ff0, 0x00, 0x06, 0x29,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM FME", 0x0374, 0x08, 0x00, 0x5a,
+    { /* IBM FME */ 0x0374, 0x08, 0x00, 0x5a,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "IBM FME", 0x0374, 0x00, 0x04, 0xac,
+    { /* IBM FME */ 0x0374, 0x00, 0x04, 0xac,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "I-O DATA PCLA/T", 0x0ff0, 0x00, 0xa0, 0xb0, 0 },
-    { "Katron PE-520", 0x0110, 0x00, 0x40, 0xf6, 0 },
-    { "Kingston KNE-PCM/x", 0x0ff0, 0x00, 0xc0, 0xf0,
+    { /* I-O DATA PCLA/T */ 0x0ff0, 0x00, 0xa0, 0xb0, 0 },
+    { /* Katron PE-520 */ 0x0110, 0x00, 0x40, 0xf6, 0 },
+    { /* Kingston KNE-PCM/x */ 0x0ff0, 0x00, 0xc0, 0xf0,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "Kingston KNE-PCM/x", 0x0ff0, 0xe2, 0x0c, 0x0f,
+    { /* Kingston KNE-PCM/x */ 0x0ff0, 0xe2, 0x0c, 0x0f,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "Kingston KNE-PC2", 0x0180, 0x00, 0xc0, 0xf0, 0 },
-    { "Longshine LCS-8534", 0, 0x08, 0x00, 0x00, 0 },
-    { "Maxtech PCN2000", 0x5000, 0x00, 0x00, 0xe8, 0 },
-    { "NDC Instant-Link", 0x003a, 0x00, 0x80, 0xc6, 0 },
-    { "NE2000 Compatible", 0x0ff0, 0x00, 0xa0, 0x0c, 0 },
-    { "Network General Sniffer", 0x0ff0, 0x00, 0x00, 0x65,
+    { /* Kingston KNE-PC2 */ 0x0180, 0x00, 0xc0, 0xf0, 0 },
+    { /* Longshine LCS-8534 */ 0, 0x08, 0x00, 0x00, 0 },
+    { /* Maxtech PCN2000 */ 0x5000, 0x00, 0x00, 0xe8, 0 },
+    { /* NDC Instant-Link */ 0x003a, 0x00, 0x80, 0xc6, 0 },
+    { /* NE2000 Compatible */ 0x0ff0, 0x00, 0xa0, 0x0c, 0 },
+    { /* Network General Sniffer */ 0x0ff0, 0x00, 0x00, 0x65,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "Panasonic VEL211", 0x0ff0, 0x00, 0x80, 0x45, 
+    { /* Panasonic VEL211 */ 0x0ff0, 0x00, 0x80, 0x45, 
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "PreMax PE-200", 0x07f0, 0x00, 0x20, 0xe0, 0 },
-    { "RPTI EP400", 0x0110, 0x00, 0x40, 0x95, 0 },
-    { "SCM Ethernet", 0x0ff0, 0x00, 0x20, 0xcb, 0 },
-    { "Socket EA", 0x4000, 0x00, 0xc0, 0x1b,
+    { /* PreMax PE-200 */ 0x07f0, 0x00, 0x20, 0xe0, 0 },
+    { /* RPTI EP400 */ 0x0110, 0x00, 0x40, 0x95, 0 },
+    { /* SCM Ethernet */ 0x0ff0, 0x00, 0x20, 0xcb, 0 },
+    { /* Socket EA */ 0x4000, 0x00, 0xc0, 0x1b,
       DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF },
-    { "Volktek NPL-402CT", 0x0060, 0x00, 0x40, 0x05, 0 }
+    { /* SuperSocket RE450T */ 0x0110, 0x00, 0xe0, 0x98, 0 },
+    { /* Volktek NPL-402CT */ 0x0060, 0x00, 0x40, 0x05, 0 }
 };
 
 #define NR_INFO		(sizeof(hw_info)/sizeof(hw_info_t))
 
 static hw_info_t default_info =
-{ "Unknown NE2000 Clone", 0x00, 0x00, 0x00, 0 };
+{ /* Unknown NE2000 Clone */ 0x00, 0x00, 0x00, 0 };
 
 typedef struct pcnet_dev_t {
     struct device	dev;
@@ -218,9 +219,10 @@ typedef struct pcnet_dev_t {
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -249,7 +251,7 @@ static dev_link_t *pcnet_attach(void)
     dev_link_t *link;
     pcnet_dev_t *info;
     struct device *dev;
-    int ret;
+    int i, ret;
 
     DEBUG(0, "pcnet_attach()\n");
 
@@ -264,7 +266,11 @@ static dev_link_t *pcnet_attach(void)
     link->io.IOAddrLines = 5;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
@@ -294,8 +300,8 @@ static dev_link_t *pcnet_attach(void)
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
-    if (ret != 0) {
-	cs_error(RegisterClient, ret);
+    if (ret != CS_SUCCESS) {
+	cs_error(link->handle, RegisterClient, ret);
 	pcnet_detach(link);
 	return NULL;
     }
@@ -356,6 +362,27 @@ static void pcnet_detach(dev_link_t *link)
 
 } /* pcnet_detach */
 
+/*======================================================================
+
+    For the Linksys EtherFast 10/100 card
+
+======================================================================*/
+
+static hw_info_t *get_DL10019A(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    int i;
+    u_char sum;
+
+    for (sum = 0, i = 0x14; i < 0x1c; i++)
+	sum += inb_p(dev->base_addr + i);
+    if (sum != 0xff)
+	return NULL;
+    for (i = 0; i < 6; i++)
+	dev->dev_addr[i] = inb_p(dev->base_addr + 0x14 + i);
+    return &default_info;
+}
+
 /*======================================================================
 
     This probes for a card's hardware address, for card types that
@@ -378,8 +405,8 @@ static hw_info_t *get_hwinfo(dev_link_t *link)
     req.AccessSpeed = 0;
     link->win = (window_handle_t)link->handle;
     i = CardServices(RequestWindow, &link->win, &req);
-    if (i != 0) {
-	cs_error(RequestWindow, i);
+    if (i != CS_SUCCESS) {
+	cs_error(link->handle, RequestWindow, i);
 	return NULL;
     }
 
@@ -397,14 +424,13 @@ static hw_info_t *get_hwinfo(dev_link_t *link)
 	for (j = 0; j < 6; j++)
 	    dev->dev_addr[j] = readb(base + (j<<1));
 	j = CardServices(ReleaseWindow, link->win);
-	if (j != 0)
-	    cs_error(ReleaseWindow, j);
+	if (j != CS_SUCCESS)
+	    cs_error(link->handle, ReleaseWindow, j);
 	return hw_info+i;
-    }
-    else {
+    } else {
 	i = CardServices(ReleaseWindow, link->win);
-	if (i != 0)
-	    cs_error(ReleaseWindow, i);
+	if (i != CS_SUCCESS)
+	    cs_error(link->handle, ReleaseWindow, i);
 	return NULL;
     }
 } /* get_hwinfo */
@@ -458,15 +484,10 @@ static hw_info_t *get_prom(dev_link_t *link)
 	    (prom[4] == hw_info[i].a2))
 	    break;
     }
-    if (i < NR_INFO) {
+    if ((i < NR_INFO) || ((prom[28] == 0x57) && (prom[30] == 0x57))) {
 	for (j = 0; j < 6; j++)
 	    dev->dev_addr[j] = prom[j<<1];
-	return hw_info+i;
-    }
-    if ((prom[28] == 0x57) && (prom[30] == 0x57)) {
-	for (j = 0; j < 6; j++)
-	    dev->dev_addr[j] = prom[j<<1];
-	return &default_info;
+	return (i < NR_INFO) ? hw_info+i : &default_info;
     }
     return NULL;
 } /* get_prom */
@@ -503,8 +524,8 @@ static hw_info_t *get_hwired(dev_link_t *link)
 
 ======================================================================*/
 
-#define CS_EXIT_TEST(ret, svc, label) \
-    if (ret != CS_SUCCESS) { cs_error(svc, ret); goto label; }
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 static void pcnet_config(dev_link_t *link)
 {
@@ -513,7 +534,7 @@ static void pcnet_config(dev_link_t *link)
     cisparse_t parse;
     pcnet_dev_t *info;
     struct device *dev;
-    int i, j, start_pg, stop_pg, cm_offset;
+    int i, j, last_ret, last_fn, start_pg, stop_pg, cm_offset;
     int manfid = 0, multi = 0;
     u_short buf[64];
     hw_info_t *hw_info;
@@ -525,16 +546,13 @@ static void pcnet_config(dev_link_t *link)
     DEBUG(0, "pcnet_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
-    tuple.DesiredTuple = CISTPL_CONFIG;
-    i = CardServices(GetFirstTuple, handle, &tuple);
-    CS_EXIT_TEST(i, GetFirstTuple, config_failed);
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleDataMax = sizeof(buf);
     tuple.TupleOffset = 0;
-    i = CardServices(GetTupleData, handle, &tuple);
-    CS_EXIT_TEST(i, GetTupleData, config_failed);
-    i = CardServices(ParseTuple, handle, &tuple, &parse);
-    CS_EXIT_TEST(i, ParseTuple, config_failed);
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
     link->conf.ConfigBase = parse.config.base;
     link->conf.Present = parse.config.rmask[0];
 
@@ -573,7 +591,8 @@ static void pcnet_config(dev_link_t *link)
 	i = CardServices(GetTupleData, handle, &tuple);
 	if (i != CS_SUCCESS) break;
 	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i == CS_SUCCESS) {
+	if ((i == CS_SUCCESS) &&
+	    (parse.cftable_entry.index != 0)) {
 	    link->conf.ConfigIndex = parse.cftable_entry.index;
 	    link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
 	    if (link->io.BasePort1 != 0) {
@@ -596,17 +615,19 @@ static void pcnet_config(dev_link_t *link)
     }
     if ((i != CS_SUCCESS) && !multi) {
 	link->conf.ConfigIndex = 1;
-	for (j = 0x100; j < 0x400; j += 0x20) {
-	    link->io.BasePort1 = j;
-	    link->io.BasePort2 = j+0x10;
+	for (j = 0; j < 0x400; j += 0x20) {
+	    link->io.BasePort1 = j ^ 0x300;
+	    link->io.BasePort2 = (j ^ 0x300) + 0x10;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
-	CS_EXIT_TEST(i, RequestIO, config_failed);
     }
-    
-    i = CardServices(RequestIRQ, link->handle, &link->irq);
-    CS_EXIT_TEST(i, RequestIRQ, config_failed);
+    if (i != CS_SUCCESS) {
+	cs_error(handle, RequestIO, i);
+	goto failed;
+    }
+
+    CS_CHECK(RequestIRQ, handle, &link->irq);
     if (multi) {
 	link->conf.Attributes |= CONF_ENABLE_SPKR;
 	link->conf.Status = CCSR_AUDIO_ENA;
@@ -614,8 +635,7 @@ static void pcnet_config(dev_link_t *link)
     if (multi && (manfid == MANFID_IBM))
 	/* For IBM Home and Away Card */
 	link->conf.ConfigIndex |= 0x10;
-    i = CardServices(RequestConfiguration, link->handle, &link->conf);
-    CS_EXIT_TEST(i, RequestConfiguration, config_failed);
+    CS_CHECK(RequestConfiguration, handle, &link->conf);
     dev->irq = link->irq.AssignedIRQ;
     dev->base_addr = link->io.BasePort1;
     if ((if_port == 1) || (if_port == 2))
@@ -623,13 +643,14 @@ static void pcnet_config(dev_link_t *link)
     else
 	printk(KERN_NOTICE "pcnet_cs: invalid if_port requested\n");
     dev->tbusy = 0;
-    i = register_netdev(dev);
-    if (i != 0) {
+    if (register_netdev(dev) != 0) {
 	printk(KERN_NOTICE "pcnet_cs: register_netdev() failed\n");
-	goto config_failed;
+	goto failed;
     }
 
-    hw_info = get_hwinfo(link);
+    hw_info = get_DL10019A(link);
+    if (hw_info == NULL)
+	hw_info = get_hwinfo(link);
     if (hw_info == NULL)
 	hw_info = get_prom(link);
     if (hw_info == NULL)
@@ -642,58 +663,55 @@ static void pcnet_config(dev_link_t *link)
 
     info->flags = hw_info->flags | (multi ? IS_MULTIFN : 0);
     /* Check for user overrides */
-    if (delay_output)
-	info->flags |= DELAY_OUTPUT;
+    info->flags |= (delay_output) ? DELAY_OUTPUT : 0;
     if (!use_big_buf)
 	info->flags &= ~USE_BIG_BUF;
     if (use_shmem != -1) {
-	if (use_shmem)
-	    info->flags |= USE_SHMEM;
-	else
-	    info->flags &= ~USE_SHMEM;
+	info->flags &= ~USE_SHMEM;
+	info->flags |= (use_shmem) ? USE_SHMEM : 0;
     }
     
     if (info->flags & USE_BIG_BUF) {
 	start_pg = SOCKET_START_PG;
 	stop_pg = SOCKET_STOP_PG;
 	cm_offset = 0x10000;
-    }
-    else {
+    } else {
 	start_pg = PCNET_START_PG;
 	stop_pg = PCNET_STOP_PG;
 	cm_offset = 0;
     }
 
-    if (info->flags & USE_SHMEM)
-	i = setup_shmem_window(link, start_pg, stop_pg, cm_offset);
-    else
-	i = setup_dma_config(link, start_pg, stop_pg);
-    if (i != 0)
+    if ((info->flags & USE_SHMEM) ?
+	setup_shmem_window(link, start_pg, stop_pg, cm_offset) :
+	setup_dma_config(link, start_pg, stop_pg))
 	goto config_undo;
 
-    ei_status.name = hw_info->id;
+    ei_status.name = "NE2000";
     ei_status.word16 = 1;
     ei_status.reset_8390 = &pcnet_reset_8390;
 
     link->dev = &info->node;
     link->state &= ~DEV_CONFIG_PENDING;
 
-    printk(KERN_INFO "%s: %s, port %#3lx, irq %d,",
-	   dev->name, hw_info->id, dev->base_addr, dev->irq);
+    printk(KERN_INFO "%s: NE2000 Compatible: port %#3lx, irq %d,",
+	   dev->name, dev->base_addr, dev->irq);
     if (info->flags & USE_SHMEM)
 	printk (" mem %#5lx,", dev->mem_start);
     if (info->flags & HAS_MISC_REG)
 	printk(" %s port,", if_names[dev->if_port]);
-    for (i = 0; i < ETHER_ADDR_LEN; i++)
-	printk(" %02X", dev->dev_addr[i]);
-    printk("\n");
+    printk(" hw_addr ");
+    for (i = 0; i < 6; i++)
+	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
     return;
 
 config_undo:
     unregister_netdev(dev);
-config_failed:    /* CS_EXIT_TEST() calls jump to here... */
+    goto failed;
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
     pcnet_release((u_long)link);
-
+    return;
 } /* pcnet_config */
 
 /*======================================================================
@@ -718,17 +736,14 @@ static void pcnet_release(u_long arg)
 	return;
     }
 
-    if (link->dev)
-	unregister_netdev(&info->dev);
-    link->dev = NULL;
-
     if (info->flags & USE_SHMEM)
 	CardServices(ReleaseWindow, link->win);
     CardServices(ReleaseConfiguration, link->handle);
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
-    if (info->dev.irq != 0)
-	irq2dev_map[info->dev.irq] = NULL;
+    if (link->dev)
+	unregister_netdev(&info->dev);
+    link->dev = NULL;
 
     link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
     if (link->state & DEV_STALE_LINK)
@@ -835,7 +850,7 @@ static int pcnet_open(struct device *dev)
     set_misc_reg(dev);
     REQUEST_IRQ(dev->irq, ei_interrupt,
 		((info->flags & IS_MULTIFN) ? SA_SHIRQ : 0),
-		dev_info, dev_info);
+		dev_info, dev);
     return ei_open(dev);
 } /* pcnet_open */
 
@@ -852,7 +867,7 @@ static int pcnet_close(struct device *dev)
     if (link == NULL)
 	return -ENODEV;
 
-    FREE_IRQ(dev->irq, dev_info);
+    FREE_IRQ(dev->irq, dev);
     
     link->open--; dev->start = 0;
     if (link->state & DEV_STALE_CONFIG) {
@@ -1235,7 +1250,7 @@ static int setup_shmem_window(dev_link_t *link, int start_pg,
 {
     win_req_t req;
     memreq_t mem;
-    int offset = 0, i;
+    int offset = 0, last_ret, last_fn;
     struct device *dev = link->priv;
     int window_size;
 
@@ -1254,15 +1269,13 @@ static int setup_shmem_window(dev_link_t *link, int start_pg,
     req.Size = window_size;
     req.AccessSpeed = mem_speed;
     link->win = (window_handle_t)link->handle;
-    i = CardServices(RequestWindow, &link->win, &req);
-    CS_EXIT_TEST(i, RequestWindow, config_failed);
+    CS_CHECK(RequestWindow, &link->win, &req);
 
     mem.CardOffset = (start_pg << 8) + cm_offset;
     offset = mem.CardOffset % window_size;
     mem.CardOffset -= offset;
     mem.Page = 0;
-    i = CardServices(MapMemPage, link->win, &mem);
-    CS_EXIT_TEST(i, MapMemPage, config_failed);
+    CS_CHECK(MapMemPage, link->win, &mem);
 
     dev->mem_start = (u_long)req.Base + offset;
     dev->rmem_start = dev->mem_start + (TX_PAGES<<8);
@@ -1281,7 +1294,8 @@ static int setup_shmem_window(dev_link_t *link, int start_pg,
 
     return 0;
 
-config_failed:
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
     return 1;
 }
 
