@@ -1,4 +1,4 @@
-/* pd67xx.h 1.2: 1997/03/21 07:32:29 (David Hinds) */
+/* pd67xx.h 1.3: 1997/05/05 01:22:27 (David Hinds) */
 
 #ifndef _LINUX_PD67XX_H
 #define _LINUX_PD67XX_H
@@ -18,7 +18,7 @@
 
 /* PD6730 extension registers -- indexed in PD67_EXT_INDEX */
 #define PD67_EXT_CTL_1		0x03	/* Extension control 1 */
-#define PD67_MEM_TOP(n)		((n)+5)	/* PCI window bits 31:24 */
+#define PD67_MEM_PAGE(n)	((n)+5)	/* PCI window bits 31:24 */
 #define PD67_EXTERN_DATA	0x0a
 #define PD67_MISC_CTL_3		0x25
 #define PD67_SMB_PWR_CTL	0x26
