@@ -61,7 +61,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"qlogic_cs.c 1.47 1997/06/01 05:03:16 (David Hinds)";
+"qlogic_cs.c 1.55 1998/01/08 04:48:14 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -71,13 +71,16 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /*====================================================================*/
 
 typedef struct scsi_info_t {
+    u_short	manf_id;
     int		ndev;
     dev_node_t	node[8];
 } scsi_info_t;
@@ -95,13 +98,12 @@ static dev_link_t *dev_list = NULL;
 
 static dev_info_t dev_info = "qlogic_cs";
 
-static int manf_id = 0;
-
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*====================================================================*/
@@ -110,7 +112,7 @@ static dev_link_t *qlogic_attach(void)
 {
     client_reg_t client_reg;
     dev_link_t *link;
-    int ret;
+    int i, ret;
     
     DEBUG(0, "qlogic_attach()\n");
 
@@ -127,7 +129,11 @@ static dev_link_t *qlogic_attach(void)
     link->io.IOAddrLines = 10;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
@@ -147,7 +153,7 @@ static dev_link_t *qlogic_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	qlogic_detach(link);
 	return NULL;
     }
@@ -191,17 +197,23 @@ static void qlogic_detach(dev_link_t *link)
 
 /*====================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
+
 static void qlogic_config(dev_link_t *link)
 {
     client_handle_t handle;
     scsi_info_t *info;
     tuple_t tuple;
     cisparse_t parse;
-    int i;
-    u_char tuple_data[64];
+    int i, last_ret, last_fn;
+    u_short tuple_data[32];
 #ifdef GET_SCSI_INFO
     Scsi_Device *dev;
     dev_node_t **tail, *node;
+#if (LINUX_VERSION_CODE >= VERSION(2,1,75))
+    struct Scsi_Host *host;
+#endif
 #endif
     
     handle = link->handle;
@@ -209,33 +221,19 @@ static void qlogic_config(dev_link_t *link)
 
     DEBUG(0, "qlogic_config(0x%p)\n", link);
 
-    tuple.TupleData = tuple_data;
+    tuple.TupleData = (cisdata_t *)tuple_data;
     tuple.TupleDataMax = 64;
     tuple.TupleOffset = 0;
-    do {
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i != CS_SUCCESS) break;
-	link->conf.ConfigBase = parse.config.base;
-    } while (0);
-    if (i != CS_SUCCESS) {
-	cs_error(ParseTuple, i);
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
-
-    do {
-	tuple.DesiredTuple = CISTPL_MANFID;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(GetTupleData, handle, &tuple); 
-	if (i != CS_SUCCESS) break;
-	manf_id = tuple.TupleData[0];
-    } while (0);
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
+    link->conf.ConfigBase = parse.config.base;
+    
+    tuple.DesiredTuple = CISTPL_MANFID;
+    if ((CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) &&
+	(CardServices(GetTupleData, handle, &tuple) == CS_SUCCESS))
+	info->manf_id = tuple.TupleData[0];
 
     /* Configure card */
 #if (LINUX_VERSION_CODE >= VERSION(2,1,23))
@@ -245,47 +243,30 @@ static void qlogic_config(dev_link_t *link)
 #endif
     link->state |= DEV_CONFIG;
 
-    do {
-
-	tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	while (i == CS_SUCCESS) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    if (i != CS_SUCCESS) break;
-	    link->conf.ConfigIndex = parse.cftable_entry.index;
-	    link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
-	    if (link->io.BasePort1 != 0) {
-		i = CardServices(RequestIO, link->handle, &link->io);
-		if (i != CS_IN_USE) break;
-	    }
-	    i = CardServices(GetNextTuple, handle, &tuple);
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    i = CardServices(GetFirstTuple, handle, &tuple);
+    while (i == CS_SUCCESS) {
+	i = CardServices(GetTupleData, handle, &tuple);
+	if (i != CS_SUCCESS) break;
+	i = CardServices(ParseTuple, handle, &tuple, &parse);
+	if (i != CS_SUCCESS) break;
+	link->conf.ConfigIndex = parse.cftable_entry.index;
+	link->io.BasePort1 = parse.cftable_entry.io.win[0].base;
+	if (link->io.BasePort1 != 0) {
+	    i = CardServices(RequestIO, handle, &link->io);
+	    if (i != CS_IN_USE) break;
 	}
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIO, i);
-	    break;
-	}
-	
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    break;
-	}
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, i);
-	    break;
-	}
-
-    } while (0);
-
-    if (i != 0) {
-	qlogic_release((u_long)link);
-	return;
+	i = CardServices(GetNextTuple, handle, &tuple);
     }
+    if (i != CS_SUCCESS) {
+	cs_error(handle, RequestIO, i);
+	goto failed;
+    }
+    
+    CS_CHECK(RequestIRQ, handle, &link->irq);
+    CS_CHECK(RequestConfiguration, handle, &link->conf);
 
-    if (manf_id == MANFID_MACNICA) {
+    if (info->manf_id == MANFID_MACNICA) {
 	/* set ATAcmd */
 	outb( 0xb4, link->io.BasePort1+0xd);
 	outb( 0x24, link->io.BasePort1+0x9);
@@ -302,8 +283,14 @@ static void qlogic_config(dev_link_t *link)
 #ifdef GET_SCSI_INFO
     tail = &link->dev;
     info->ndev = 0;
+#if (LINUX_VERSION_CODE < VERSION(2,1,75))
     for (dev = scsi_devices; dev != NULL; dev = dev->next)
 	if (dev->host->hostt == &driver_template) {
+#else
+    for (host = scsi_hostlist; host; host = host->next)
+	if (host->hostt == &driver_template)
+	    for (dev = host->host_queue; dev; dev = dev->next) {
+#endif
 	    u_long arg[2], id;
 	    kernel_scsi_ioctl(dev, SCSI_IOCTL_GET_IDLUN, arg);
 	    id = (arg[0]&0x0f) + ((arg[0]>>4)&0xf0) +
@@ -342,7 +329,14 @@ static void qlogic_config(dev_link_t *link)
 #endif /* GET_SCSI_INFO */
     
     link->state &= ~DEV_CONFIG_PENDING;
-    
+    return;
+
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    qlogic_release((u_long)link);
+    return;
+
 } /* qlogic_config */
 
 /*====================================================================*/
@@ -409,8 +403,9 @@ static int qlogic_event(event_t event, int priority,
 	/* Fall through... */
     case CS_EVENT_CARD_RESET:
 	if (link->state & DEV_CONFIG) {
+	    scsi_info_t *info = link->priv;
 	    CardServices(RequestConfiguration, link->handle, &link->conf);
-	    if (manf_id == MANFID_MACNICA) {
+	    if (info->manf_id == MANFID_MACNICA) {
 		outb( 0x80, link->io.BasePort1+0xd);
 		outb( 0x24, link->io.BasePort1+0x9);
 		outb( 0x04, link->io.BasePort1+0xd);
