@@ -2,7 +2,7 @@
 
     Resource management routines
 
-    rsrc_mgr.c 1.2 1995/05/17 19:21:26 (David Hinds)
+    rsrc_mgr.c 1.29 1998/01/07 04:44:34 (David Hinds)
     
 ======================================================================*/
 
@@ -30,17 +30,17 @@
 /*====================================================================*/
 
 typedef struct memory_entry_t {
-    u_long base, num;
-    char *name;
-    struct memory_entry_t *next;
+    u_long			base, num;
+    char			*name;
+    struct memory_entry_t	*next;
 } memory_entry_t;
 
 /* An ordered linked list of allocated memory blocks */
 static memory_entry_t memlist = { 0, 0, NULL, NULL };
 
 typedef struct resource_entry_t {
-    u_long		base, num;
-    struct		resource_entry_t *next;
+    u_long			base, num;
+    struct resource_entry_t	*next;
 } resource_entry_t;
 
 /* Memory resource database */
@@ -50,9 +50,9 @@ static resource_entry_t mem_db = { 0, 0, &mem_db };
 static resource_entry_t io_db = { 0, 0, &io_db };
 
 typedef struct irq_info_t {
-    u_long		Attributes;
-    int			time_share, dyn_share;
-    struct socket_info_t *Socket;
+    u_int			Attributes;
+    int				time_share, dyn_share;
+    struct socket_info_t	*Socket;
 } irq_info_t;
 
 /* Table of IRQ assignments */
@@ -65,9 +65,11 @@ static irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
 /* Should we probe resources for conflicts? */
 static int probe_io = 1;
 static int probe_mem = 1;
+static int mem_limit = 0x10000;
 
 MODULE_PARM(probe_io, "i");
 MODULE_PARM(probe_mem, "i");
+MODULE_PARM(mem_limit, "i");
 
 /*======================================================================
 
@@ -78,7 +80,7 @@ MODULE_PARM(probe_mem, "i");
 static memory_entry_t *find_gap(memory_entry_t *root,
 				memory_entry_t *entry)
 {
-    unsigned long flags;
+    u_long flags;
     memory_entry_t *p;
     
     if (entry->base > entry->base+entry->num-1)
@@ -108,7 +110,7 @@ int check_mem_region(u_long base, u_long num)
 
 int register_mem_region(u_long base, u_long num, char *name)
 {
-    unsigned long flags;
+    u_long flags;
     memory_entry_t *p, *entry;
 
     entry = kmalloc(sizeof(memory_entry_t), GFP_ATOMIC);
@@ -132,7 +134,7 @@ int register_mem_region(u_long base, u_long num, char *name)
 
 int release_mem_region(u_long base, u_long num)
 {
-    unsigned long flags;
+    u_long flags;
     memory_entry_t *p, *q;
 
     save_flags(flags);
@@ -191,22 +193,18 @@ static int sub_interval(resource_entry_t *map, u_long base, u_long num)
 		    kfree_s(q, sizeof(resource_entry_t));
 		    /* don't advance the pointer yet */
 		    q = p;
-		}
-		else {
+		} else {
 		    /* Cut off bit from the front */
 		    q->num = q->base + q->num - base - num;
 		    q->base = base + num;
 		}
-	    }
-	    else if (q->base+q->num <= base+num) {
+	    } else if (q->base+q->num <= base+num) {
 		/* Cut off bit from the end */
 		q->num = base - q->base;
-	    }
-	    else {
+	    } else {
 		/* Split the block into two pieces */
 		p = kmalloc(sizeof(resource_entry_t), GFP_KERNEL);
 		p->base = base+num;
-		
 		p->num = q->base+q->num - p->base;
 		q->num = base - q->base;
 		p->next = q->next ; q->next = p;
@@ -229,21 +227,26 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     ioaddr_t i, j, bad, any;
     u_char *b, hole;
 
+    printk(KERN_INFO "cs: IO port probe 0x%04x-0x%04x:",
+	   base, base+num-1);
+    
     /* First, what does a floating port look like? */
     b = kmalloc(256, GFP_KERNEL);
     memset(b, 0, 256);
-    for (i = 0x100; i < 0x400; i += 8) {
+    for (i = base; i < base+num; i += 8) {
 	if (check_region(i, 8) != 0)
 	    continue;
-	b[inb_p(i)]++;
+	hole = inb_p(i);
+	for (j = 1; j < 8; j++)
+	    if (inb_p(i+j) != hole) break;
+	if (j == 8)
+	    b[hole]++;
     }
     hole = 0;
     for (i = 0; i < 256; i++)
 	if (b[i] > b[hole]) hole = i;
     kfree(b);
 
-    printk(KERN_INFO "cs: IO port probe 0x%04x-0x%04x:",
-	   base, base+num-1);
     bad = any = 0;
     for (i = base; i < base+num; i += 8) {
 	if (check_region(i, 8) != 0)
@@ -276,55 +279,72 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     printk(any ? "\n" : " clean.\n");
 }
 
-static int do_mem_probe(u_long base, u_long num, u_long step,
+/*======================================================================
+
+    The memory probe.  If the memory list includes a 64K-aligned block
+    below 1MB, we probe in 64K chunks, and as soon as we accumulate at
+    least mem_limit free space, we quit.
+    
+======================================================================*/
+
+#define STEP 8192
+
+static int do_mem_probe(u_long base, u_long num, int pass,
 			int (*is_valid)(u_long))
 {
-    u_long i, bad, any;
+    u_long i, j, bad;
 
     printk(KERN_INFO "cs: memory probe 0x%06lx-0x%06lx:",
 	   base, base+num-1);
-    bad = any = 0;
-    for (i = base; i < base+num; i += step) {
-	if (check_mem_region(i, step) != 0)
-	    continue;
-	if (!is_valid(i)) {
-	    if (!any)
-		printk(" excluding");
-	    if (!bad)
-		bad = any = i;
-	} else {
-	    if (bad) {
-		register_mem_region(bad, i-bad, "reserved");
-		printk(" %#05lx-%#05lx", bad, i-1);
-	    }
-	    bad = 0;
+    if (pass != 0) release_mem_region(base, num);
+    bad = 0;
+    for (i = base; i < base+num; i = j + STEP) {
+	for (j = i; j < base+num; j += STEP)
+	    if ((check_mem_region(j, STEP)) == 0 && is_valid(j))
+		break;
+	if (i != j) {
+	    if (!bad) printk(" excluding");
+	    printk(" %#05lx-%#05lx", i, j-1);
+	    register_mem_region(i, j-i, "reserved");
+	    bad += j-i;
 	}
     }
-    if (bad) {
-	if ((bad == base) && (i == base+num)) {
-	    printk(" nothing: probe failed.\n");
-	    return -1;
-	} else {
-	    register_mem_region(bad, i-bad, "reserved");
-	    printk(" %#05lx-%#05lx", bad, i-1);
-	}
-    }
-    printk(any ? "\n" : " clean.\n");
-    return 0;
+    printk(bad ? "\n" : " clean.\n");
+    return (num - bad);
 }
 
-int validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
+void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
 {
+    resource_entry_t *m;
+    static u_char order[] = { 0xd0, 0xe0, 0xc0, 0xf0 };
+    u_long b, ok = 0;
+    int i, pass;
+    
     if (!probe_mem)
-	return 0;
-    /* Make sure we have exactly one memory range resource */
-    if ((mem_db.next == &mem_db) || (mem_db.next->next != &mem_db))
-	return 0;
-    if (do_mem_probe(mem_db.next->base, mem_db.next->num, 4096,
-		     is_valid) == 0)
-	return 0;
-    return do_mem_probe(mem_db.next->base, mem_db.next->num, 8192,
-			do_cksum);
+	return;
+    /* We do up to two passes through the list */
+    for (pass = 0; pass < 2; pass++) {
+	for (m = mem_db.next; m != &mem_db; m = m->next) {
+	    /* Only probe < 1 MB */
+	    if (m->base >= 0x100000) continue;
+	    if ((m->base | m->num) & 0xffff) {
+		ok += do_mem_probe(m->base, m->num, pass, is_valid);
+		continue;
+	    }
+	    /* Special probe for 64K-aligned block */
+	    for (i = 0; i < 4; i++) {
+		b = order[i] << 12;
+		if ((b >= m->base) && (b+0x10000 <= m->base+m->num)) {
+		    if (ok >= mem_limit)
+			register_mem_region(b, 0x10000, "skipped");
+		    else
+			ok += do_mem_probe(b, 0x10000, pass, is_valid);
+		}
+	    }
+	}
+	if (ok) return;
+	is_valid = do_cksum;
+    }
 }
 
 /*======================================================================
@@ -336,7 +356,7 @@ int validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
 
 int find_io_region(ioaddr_t *base, ioaddr_t num, char *name)
 {
-    u_long align;
+    ioaddr_t align;
     resource_entry_t *m;
     
     if (*base != 0) {
@@ -398,18 +418,20 @@ int find_mem_region(u_long *base, u_long num, char *name)
     
 ======================================================================*/
 
-int try_irq(irq_req_t *req, int irq, int specific)
+static void fake_irq IRQ(int i, void *d, struct pt_regs *r) { }
+
+int try_irq(u_int Attributes, int irq, int specific)
 {
     irq_info_t *info = &irq_table[irq];
     if (info->Attributes & RES_ALLOCATED) {
-	switch (req->Attributes & IRQ_TYPE) {
+	switch (Attributes & IRQ_TYPE) {
 	case IRQ_TYPE_EXCLUSIVE:
 	    return CS_IN_USE;
 	case IRQ_TYPE_TIME:
 	    if ((info->Attributes & RES_IRQ_TYPE)
 		!= RES_IRQ_TYPE_TIME)
 		return CS_IN_USE;
-	    if (req->Attributes & IRQ_FIRST_SHARED)
+	    if (Attributes & IRQ_FIRST_SHARED)
 		return CS_BAD_ATTRIBUTE;
 	    info->Attributes |= RES_IRQ_TYPE_TIME | RES_ALLOCATED;
 	    info->time_share++;
@@ -418,30 +440,30 @@ int try_irq(irq_req_t *req, int irq, int specific)
 	    if ((info->Attributes & RES_IRQ_TYPE)
 		!= RES_IRQ_TYPE_DYNAMIC)
 		return CS_IN_USE;
-	    if (req->Attributes & IRQ_FIRST_SHARED)
+	    if (Attributes & IRQ_FIRST_SHARED)
 		return CS_BAD_ATTRIBUTE;
 	    info->Attributes |= RES_IRQ_TYPE_DYNAMIC | RES_ALLOCATED;
 	    info->dyn_share++;
 	    break;
 	}
-    }
-    else {
+    } else {
 	if ((info->Attributes & RES_RESERVED) && !specific)
 	    return CS_IN_USE;
-	if (REQUEST_IRQ(irq, NULL, 0, "bogus", NULL) == -EBUSY)
+	if (REQUEST_IRQ(irq, fake_irq, 0, "bogus", NULL) != 0)
 	    return CS_IN_USE;
-	switch (req->Attributes & IRQ_TYPE) {
+	FREE_IRQ(irq, NULL);
+	switch (Attributes & IRQ_TYPE) {
 	case IRQ_TYPE_EXCLUSIVE:
 	    info->Attributes |= RES_ALLOCATED;
 	    break;
 	case IRQ_TYPE_TIME:
-	    if (!(req->Attributes & IRQ_FIRST_SHARED))
+	    if (!(Attributes & IRQ_FIRST_SHARED))
 		return CS_BAD_ATTRIBUTE;
 	    info->Attributes |= RES_IRQ_TYPE_TIME | RES_ALLOCATED;
 	    info->time_share = 1;
 	    break;
 	case IRQ_TYPE_DYNAMIC_SHARING:
-	    if (!(req->Attributes & IRQ_FIRST_SHARED))
+	    if (!(Attributes & IRQ_FIRST_SHARED))
 		return CS_BAD_ATTRIBUTE;
 	    info->Attributes |= RES_IRQ_TYPE_DYNAMIC | RES_ALLOCATED;
 	    info->dyn_share = 1;
@@ -453,12 +475,12 @@ int try_irq(irq_req_t *req, int irq, int specific)
 
 /*====================================================================*/
 
-void undo_irq(irq_req_t *req)
+void undo_irq(u_int Attributes, int irq)
 {
     irq_info_t *info;
 
-    info = &irq_table[req->AssignedIRQ];
-    switch (req->Attributes & IRQ_TYPE) {
+    info = &irq_table[irq];
+    switch (Attributes & IRQ_TYPE) {
     case IRQ_TYPE_EXCLUSIVE:
 	info->Attributes &= RES_RESERVED;
 	break;
