@@ -34,7 +34,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"serial_cs.c 1.85 1997/06/01 05:03:16 (David Hinds)";
+"serial_cs.c 1.91 1998/02/06 04:55:15 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -44,38 +44,34 @@ static char *version =
 /* Parameters that can be set with 'insmod' */
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 /* Enable the speaker? */
 static int do_sound = 1;
 
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(do_sound, "i");
 
 /*====================================================================*/
 
-/* Table of multifunction card ID's */
+/* Table of multifunction and multi-port card ID's */
 
 typedef struct {
     u_short	manfid;
     u_short	prodid;
     int		multi;		/* 1 = multifunction, > 1 = # ports */
-    int		slave;		/* 1 = not compliant with MFC spec */
 } multi_id_t;
 
 static multi_id_t multi_id[] = {
-    { MANFID_IBM, PRODID_IBM_HOME_AND_AWAY, 1, 0 },
-    { MANFID_3COM, PRODID_3COM_3C562, 1, 0 },
-    { MANFID_LINKSYS, PRODID_LINKSYS_PCMLM28, 1, 1 },
-    { MANFID_OSITECH, PRODID_OSITECH_JACK_144, 1, 1 },
-    { MANFID_OSITECH, PRODID_OSITECH_JACK_288, 1, 1 },
-    { MANFID_OSITECH, PRODID_OSITECH_JACK_336, 1, 1 },
-    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_VARIOUS, 1, 1 },
-    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_EM3288, 1, 1 },
-    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2, 0 },
-    { MANFID_QUATECH, PRODID_QUATECH_QUAD_RS232, 4, 0 },
-    { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2, 0 },
-    { MANFID_INTEL, PRODID_INTEL_DUAL_RS232, 2, 0 }
+    { MANFID_IBM, PRODID_IBM_HOME_AND_AWAY, 1 },
+    { MANFID_3COM, PRODID_3COM_3C562, 1 },
+    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2 },
+    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232_D1, 2 },
+    { MANFID_QUATECH, PRODID_QUATECH_QUAD_RS232, 4 },
+    { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2 },
+    { MANFID_INTEL, PRODID_INTEL_DUAL_RS232, 2 }
 };
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
@@ -102,9 +98,10 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -119,7 +116,7 @@ static dev_link_t *serial_attach(void)
 {
     client_reg_t client_reg;
     dev_link_t *link;
-    int ret;
+    int i, ret;
     
     DEBUG(0, "serial_attach()\n");
 
@@ -133,7 +130,11 @@ static dev_link_t *serial_attach(void)
     link->io.IOAddrLines = 3;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
     if (do_sound) {
@@ -158,7 +159,7 @@ static dev_link_t *serial_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != CS_SUCCESS) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	serial_detach(link);
 	return NULL;
     }
@@ -203,7 +204,7 @@ static void serial_detach(dev_link_t *link)
     if (link->handle) {
 	ret = CardServices(DeregisterClient, link->handle);
 	if (ret != CS_SUCCESS)
-	    cs_error(DeregisterClient, ret);
+	    cs_error(link->handle, DeregisterClient, ret);
     }
     
     /* Unlink device structure, free bits */
@@ -224,7 +225,7 @@ static int setup_serial(serial_info_t *info, ioaddr_t port, int irq)
     serial.irq = irq;
     serial.flags = ASYNC_SKIP_TEST;
 #ifdef ASYNC_SHARE_IRQ
-    serial.flags |= (info->multi) ? ASYNC_SHARE_IRQ : 0;
+    serial.flags |= (info->multi || info->slave) ? ASYNC_SHARE_IRQ : 0;
 #endif
     line = register_serial(&serial);
     if (line < 0) {
@@ -271,8 +272,26 @@ static int simple_config(dev_link_t *link)
     u_char buf[256];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
+    config_info_t config;
     int i, j;
 
+    /* If the card is already configured, look up the port and irq */
+    i = CardServices(GetConfigurationInfo, handle, &config);
+    if ((i == CS_SUCCESS) &&
+	(config.Attributes & CONF_VALID_CLIENT)) {
+	ioaddr_t port = 0;
+	if ((config.BasePort2 != 0) && (config.NumPorts2 == 8)) {
+	    port = config.BasePort2;
+	    info->slave = 1;
+	} else if ((info->manfid == MANFID_OSITECH) &&
+		   (config.NumPorts1 == 0x40)) {
+	    port = config.BasePort1 + 0x28;
+	    info->slave = 1;
+	}
+	if (info->slave)
+	    return setup_serial(info, port, config.AssignedIRQ);
+    }
+    
     /* First pass: look for a config entry that looks normal. */
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
@@ -309,49 +328,26 @@ static int simple_config(dev_link_t *link)
 
 found_port:
     if (i != CS_SUCCESS) {
-	cs_error(RequestIO, i);
+	cs_error(link->handle, RequestIO, i);
 	return -1;
     }
     
     i = CardServices(RequestIRQ, link->handle, &link->irq);
     if (i != CS_SUCCESS) {
-	cs_error(RequestIRQ, i);
+	cs_error(link->handle, RequestIRQ, i);
 	link->irq.AssignedIRQ = 0;
     }
     if (info->multi && (info->manfid == MANFID_3COM))
 	link->conf.ConfigIndex &= ~(0x08);
     i = CardServices(RequestConfiguration, link->handle, &link->conf);
     if (i != CS_SUCCESS) {
-	cs_error(RequestConfiguration, i);
+	cs_error(link->handle, RequestConfiguration, i);
 	return -1;
     }
 
     return setup_serial(info, link->io.BasePort1, link->irq.AssignedIRQ);
 }
 
-static int slave_config(dev_link_t *link)
-{
-    client_handle_t handle = link->handle;
-    serial_info_t *info = link->priv;
-    config_info_t config;
-    ioaddr_t port;
-    int i;
-
-    /* If the card is already configured, look up the port and irq */
-    i = CardServices(GetConfigurationInfo, handle, &config);
-    if ((i == CS_SUCCESS) &&
-	(config.Attributes & CONF_VALID_CLIENT)) {
-	if (config.BasePort2 != 0)
-	    port = config.BasePort2;
-	else
-	    /* for Ositech cards */
-	    port = config.BasePort1 + 0x28;
-	info->slave = 1;
-	return setup_serial(info, port, config.AssignedIRQ);
-    } else
-	return simple_config(link);
-}
-
 static int multi_config(dev_link_t *link)
 {
     client_handle_t handle = link->handle;
@@ -402,18 +398,23 @@ static int multi_config(dev_link_t *link)
     }
     
     if (i != CS_SUCCESS) {
-	cs_error(RequestIO, i);
+	cs_error(link->handle, RequestIO, i);
 	return -1;
     }
     
     i = CardServices(RequestIRQ, link->handle, &link->irq);
     if (i != CS_SUCCESS) {
-	cs_error(RequestIRQ, i);
+	cs_error(link->handle, RequestIRQ, i);
 	link->irq.AssignedIRQ = 0;
     }
+    /* Socket Dual IO: this enables irq's for second port */
+    if (info->multi && (info->manfid == MANFID_SOCKET)) {
+	link->conf.Present |= PRESENT_EXT_STATUS;
+	link->conf.ExtStatus = ESR_REQ_ATTN_ENA;
+    }
     i = CardServices(RequestConfiguration, link->handle, &link->conf);
     if (i != CS_SUCCESS) {
-	cs_error(RequestConfiguration, i);
+	cs_error(link->handle, RequestConfiguration, i);
 	return -1;
     }
     
@@ -432,8 +433,8 @@ static int multi_config(dev_link_t *link)
 
 ======================================================================*/
 
-#define CS_EXIT_TEST(ret, svc, label) \
-if (ret != CS_SUCCESS) { cs_error(svc, ret); goto label; }
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn), args))!=0) goto cs_failed
 
 void serial_config(dev_link_t *link)
 {
@@ -443,7 +444,7 @@ void serial_config(dev_link_t *link)
     u_short buf[128];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
-    int i;
+    int i, last_ret, last_fn;
 
     sti();
     handle = link->handle;
@@ -456,8 +457,11 @@ void serial_config(dev_link_t *link)
     tuple.Attributes = 0;
     /* Get configuration register information */
     tuple.DesiredTuple = CISTPL_CONFIG;
-    i = first_tuple(handle, &tuple, &parse);
-    CS_EXIT_TEST(i, GetFirstTuple, config_failed);
+    last_ret = first_tuple(handle, &tuple, &parse);
+    if (last_ret != CS_SUCCESS) {
+	last_fn = ParseTuple;
+	goto cs_failed;
+    }
     link->conf.ConfigBase = parse.config.base;
     link->conf.Present = parse.config.rmask[0];
     
@@ -473,10 +477,8 @@ void serial_config(dev_link_t *link)
 	    if ((buf[0] == multi_id[i].manfid) &&
 		(buf[1] == multi_id[i].prodid))
 		break;
-	if (i < MULTI_COUNT) {
+	if (i < MULTI_COUNT)
 	    info->multi = multi_id[i].multi;
-	    info->slave = multi_id[i].slave;
-	}
     }
 
     /* Another check for dual-serial cards */
@@ -490,31 +492,27 @@ void serial_config(dev_link_t *link)
 
     if (info->multi > 1)
 	multi_config(link);
-    else if (info->slave)
-	slave_config(link);
     else
 	simple_config(link);
     
     if (info->ndev == 0)
-	goto config_failed;
+	goto failed;
     
     if (info->manfid == MANFID_IBM) {
 	conf_reg_t reg = { 0, CS_READ, 0x800, 0 };
-	i = CardServices(AccessConfigurationRegister,
-			 link->handle, &reg);
-	CS_EXIT_TEST(i, AccessConfigurationRegister, config_failed);
+	CS_CHECK(AccessConfigurationRegister, link->handle, &reg);
 	reg.Action = CS_WRITE;
 	reg.Value = reg.Value | 1;
-	i = CardServices(AccessConfigurationRegister,
-			 link->handle, &reg);
-	CS_EXIT_TEST(i, AccessConfigurationRegister, config_failed);
+	CS_CHECK(AccessConfigurationRegister, link->handle, &reg);
     }
 
     link->dev = &info->node[0];
     link->state &= ~DEV_CONFIG_PENDING;
     return;
 
-config_failed:
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
     serial_release((u_long)link);
 
 } /* serial_config */
@@ -540,10 +538,12 @@ void serial_release(u_long arg)
 	unregister_serial(info->line[i]);
     }
     link->dev = NULL;
-    
-    CardServices(ReleaseConfiguration, link->handle);
-    CardServices(ReleaseIO, link->handle, &link->io);
-    CardServices(ReleaseIRQ, link->handle, &link->irq);
+
+    if (!info->slave) {
+	CardServices(ReleaseConfiguration, link->handle);
+	CardServices(ReleaseIO, link->handle, &link->io);
+	CardServices(ReleaseIRQ, link->handle, &link->irq);
+    }
     
     link->state &= ~DEV_CONFIG;
 
