@@ -36,7 +36,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args);
 static char *version =
-"skeleton.c 1.37 1997/06/01 05:03:16 (David Hinds)";
+"skeleton.c 1.40 1997/12/29 17:01:08 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -47,7 +47,7 @@ static char *version =
 
 /* Bit map of interrupts to choose from */
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
 
 MODULE_PARM(irq_mask, "i");
 
@@ -128,9 +128,10 @@ typedef struct local_info_t {
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -199,7 +200,7 @@ static dev_link_t *skeleton_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	skeleton_detach(link);
 	return NULL;
     }
@@ -264,13 +265,16 @@ static void skeleton_detach(dev_link_t *link)
     
 ======================================================================*/
 
+#define CS_CHECK(fn, args...) \
+while ((last_ret=CardServices(last_fn=(fn),args))!=0) goto cs_failed
+
 static void skeleton_config(dev_link_t *link)
 {
     client_handle_t handle;
     tuple_t tuple;
     cisparse_t parse;
     local_info_t *dev;
-    int i, j;
+    int i, j, last_fn, last_ret;
     u_char buf[64];
     win_req_t req;
     modwin_t mod;
@@ -285,97 +289,66 @@ static void skeleton_config(dev_link_t *link)
        This reads the card's CONFIG tuple to find its configuration
        registers.
     */
-    do {
-	tuple.DesiredTuple = CISTPL_CONFIG;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	tuple.TupleData = buf;
-	tuple.TupleDataMax = 64;
-	tuple.TupleOffset = 0;
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if (i != CS_SUCCESS) break;
-	link->conf.ConfigBase = parse.config.base;
-    } while (0);
-    if (i != CS_SUCCESS) {
-	cs_error(ParseTuple, i);
-	link->state &= ~DEV_CONFIG_PENDING;
-	return;
-    }
+    tuple.DesiredTuple = CISTPL_CONFIG;
+    CS_CHECK(GetFirstTuple, handle, &tuple);
+    tuple.TupleData = buf;
+    tuple.TupleDataMax = 64;
+    tuple.TupleOffset = 0;
+    CS_CHECK(GetTupleData, handle, &tuple);
+    CS_CHECK(ParseTuple, handle, &tuple, &parse);
+    link->conf.ConfigBase = parse.config.base;
     
     /* Configure card */
     link->state |= DEV_CONFIG;
 
-    do {
+    /*
+       Try allocating IO ports.  This tries a few fixed addresses.
+       If you want, you can also read the card's config table to
+       pick addresses -- see the serial driver for an example.
+    */
+    for (j = 0; j < 0x400; j += 0x20) {
+	/* The '^0x300' is so that we probe 0x300-0x3ff first, then
+	   0x200-0x2ff, and so on, because this seems safer */
+	link->io.BasePort1 = j^0x300;
+	link->io.BasePort2 = link->io.BasePort1 + 0x10
+	i = CardServices(RequestIO, link->handle, &link->io);
+	if (i == CS_SUCCESS) break;
+	}
+    if (i != CS_SUCCESS) {
+	cs_error(link->handle, RequestIO, i);
+	goto failed;
+    }
+    
+    /*
+       Now allocate an interrupt line.  Note that this does not
+       actually assign a handler to the interrupt.
+    */
+    CS_CHECK(RequestIRQ, link->handle, &link->irq);
 	
-	/*
-	   Try allocating IO ports.  This tries a few fixed addresses.
-	   If you want, you can also read the card's config table to
-	   pick addresses -- see the serial driver for an example.
-	*/
-	for (j = 0x300; j < 0x400; j += 0x20) {
-	    link->io.BasePort1 = j;
-	    link->io.BasePort2 = j+0x10;
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i == CS_SUCCESS) break;
-	}
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIO, i);
-	    break;
-	}
-	
-	/*
-	   Now allocate an interrupt line.  Note that this does not
-	   actually assign a handler to the interrupt.
-	*/
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    break;
-	}
-	
-	/*
-	   This actually configures the PCMCIA socket -- setting up
-	   the I/O windows and the interrupt mapping.
-	*/
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestConfiguration, i);
-	    break;
-	}
-
-	/*
-	   Allocate a 16K memory window.  Note that the dev_link_t
-	   structure provides space for one window handle -- if your
-	   device needs several windows, you'll need to keep track of
-	   the handles in your private data structure, link->priv.
-	*/
-	req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_AM|WIN_ENABLE;
-	req.Base = NULL;
-	req.Size = 0x4000;
-	req.AccessSpeed = 0;
-	link->win = (window_handle_t)link->handle;
-	i = CardServices(RequestWindow, &link->win, &req);
-	if (i != 0) {
-	    cs_error(RequestWindow, i);
-	    break;
-	}
-
-	mod.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
-	i = CardServices(ModifyWindow, link->win, &mod);
-	if (i != 0) {
-	    cs_error(ModifyWindow, i);
-	    break;
-	    }
-	mem.CardOffset = 0; mem.Page = 0;
-	i = CardServices(MapMemPage, link->win, &mem);
-	if (i != 0) {
-	    cs_error(MapMemPage, i);
-	    break;
-	    }
-    } while (0);
+    /*
+       This actually configures the PCMCIA socket -- setting up
+       the I/O windows and the interrupt mapping.
+    */
+    CS_CHECK(RequestConfiguration, link->handle, &link->conf);
 
+    /*
+       Allocate a 16K memory window.  Note that the dev_link_t
+       structure provides space for one window handle -- if your
+       device needs several windows, you'll need to keep track of
+       the handles in your private data structure, link->priv.
+    */
+    req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_AM|WIN_ENABLE;
+    req.Base = NULL;
+    req.Size = 0x4000;
+    req.AccessSpeed = 0;
+    link->win = (window_handle_t)link->handle;
+    CS_CHECK(RequestWindow, &link->win, &req);
+    
+    mod.Attributes = WIN_DATA_WIDTH_16|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
+    CS_CHECK(ModifyWindow, link->win, &mod);
+    mem.CardOffset = 0; mem.Page = 0;
+    CS_CHECK(MapMemPage, link->win, &mem);
+    
     /* At this point, the dev_node_t structure(s) should be
        initialized and arranged in a linked list at link->dev. */
     sprintf(dev->node.dev_name, "skel0");
@@ -384,13 +357,14 @@ static void skeleton_config(dev_link_t *link)
     link->dev = &dev->node;
     
     link->state &= ~DEV_CONFIG_PENDING;
-    /* If any step failed, release any partially configured state */
-    if (i != 0) {
-	skeleton_release((u_long)link);
-	return;
-    }
-
     printk(KERN_DEBUG "skeleton device loaded\n");
+    return;
+
+cs_failed:
+    cs_error(link->handle, last_fn, last_ret);
+failed:
+    skeleton_release((u_long)link);
+
 } /* skeleton_config */
 
 /*======================================================================
