@@ -11,7 +11,9 @@
     (erik@vt.edu).  Donald wrote the SMC 91c92 code using parts of
     Erik's SMC 91c94 driver.  Rowan wrote a similar driver, and I've
     incorporated some parts of his driver here.  I (Dave) wrote most
-    of the PCMCIA glue code, and the Ositech support code.
+    of the PCMCIA glue code, and the Ositech support code.  Kelly
+    Stephens (kstephen@holli.com) added support for the Motorola
+    Mariner, with help from Allen Brost. 
 
     This software may be used and distributed according to the terms of
     the GNU Public License, incorporated herein by reference.
@@ -21,7 +23,6 @@
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
-#include <ctype.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
@@ -71,10 +72,12 @@ static char *if_names[] = { "Auto", "10baseT", "10base2"};
 static int if_port = 1;
 
 /* Bit map of interrupts to choose from. */
-static u_long irq_mask = 0xdeb8;
+static u_int irq_mask = 0xdeb8;
+static int irq_list[4] = { -1 };
 
 MODULE_PARM(if_port, "i");
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-4i");
 
 /* Operational parameter that usually are not changed. */
 
@@ -112,11 +115,21 @@ struct smc_private {
 /* Special definitions for Megahertz multifunction cards */
 #define MEGAHERTZ_ISR		0x0380
 
+/* Special function registers for Motorola Mariner */
+#define MOT_LAN			0x0000
+#define MOT_UART		0x0020
+#define MOT_EEPROM		0x20
+
+#define MOT_NORMAL \
+(COR_LEVEL_REQ | COR_FUNC_ENA | COR_ADDR_DECODE | COR_IREQ_ENA)
+
 /* Special function registers for Ositech cards */
 #define OSITECH_AUI_CTL		0x0c
 #define OSITECH_PWRDOWN		0x0d
 #define OSITECH_RESET		0x0e
 #define OSITECH_ISR		0x0f
+#define OSITECH_AUI_PWR		0x0c
+#define OSITECH_RESET_ISR	0x0e
 
 #define OSI_AUI_PWR		0x40
 #define OSI_LAN_PWRDOWN		0x02
@@ -133,6 +146,7 @@ struct smc_private {
 #define	 TCR_CLEAR	0	/* do NOTHING */
 #define  TCR_ENABLE	0x0001	/* if this is 1, we can transmit */
 #define	 TCR_PAD_EN	0x0080	/* pads short packets to 64 bytes */
+#define  TCR_MONCSN	0x0400  /* Monitor Carrier. */
 #define  TCR_FDUPLX	0x0800  /* Full duplex mode. */
 #define	 TCR_NORMAL TCR_ENABLE | TCR_PAD_EN
 
@@ -149,8 +163,13 @@ struct smc_private {
 #define  CFG_STATIC		0x0030
 #define  CFG_IRQ_SEL_1		0x0004
 #define  CFG_IRQ_SEL_0		0x0002
+#define BASE_ADDR		2
 #define	ADDR0			4
+#define	GENERAL			10
 #define	CONTROL			12
+#define  CTL_STORE		0x0001
+#define  CTL_RELOAD		0x0002
+#define  CTL_EE_SELECT		0x0004
 #define  CTL_AUTO_RELEASE	0x0800
 #define	 CTL_POWERDOWN		0x2000
 
@@ -171,9 +190,6 @@ struct smc_private {
 #define	 PTR_RCV	0x8000
 #define	DATA_1		8
 #define	INTERRUPT	12
-#define INT_MASK	13
-enum { IntrRxInt = 0x01, IntrTxInt = 0x02, IntrTxEmpty = 0x04,
-       IntrAlloc = 0x08, IntrRxOvr = 0x10, IntrProtoErr = 0x20 };
 #define  IM_RCV_INT		0x1
 #define	 IM_TX_INT		0x2
 #define	 IM_TX_EMPTY_INT	0x4
@@ -200,6 +216,7 @@ enum RxCfg { RxAllMulti = 0x0004, RxPromisc = 0x0002,
 #define	MULTICAST2	2
 #define	MULTICAST4	4
 #define	MULTICAST6	6
+#define REVISION	0x0a
 
 /* Transmit status bits. */
 #define TS_SUCCESS 0x0001
@@ -216,6 +233,9 @@ enum RxCfg { RxAllMulti = 0x0004, RxPromisc = 0x0002,
 #define RS_MULTICAST	0x0001
 #define RS_ERRORS	(RS_ALGNERR | RS_BADCRC | RS_TOOLONG | RS_TOOSHORT)
 
+#define set_bits(v, p) outw(inw(p)|(v), (p))
+#define mask_bits(v, p) outw(inw(p)&(v), (p))
+
 /*====================================================================*/
 
 static dev_link_t *smc91c92_attach(void);
@@ -228,7 +248,7 @@ static int smc91c92_event(event_t event, int priority,
 static int smc91c92_open(struct device *dev);
 static int smc91c92_close(struct device *dev);
 static int smc_start_xmit(struct sk_buff *skb, struct device *dev);
-static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs);
+static void smc_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs);
 static void smc_rx(struct device *dev);
 static struct net_device_stats *smc91c92_get_stats(struct device *dev);
 static void set_rx_mode(struct device *dev);
@@ -237,9 +257,10 @@ static void smc_reset(struct device *dev);
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*====================================================================*/
@@ -263,7 +284,7 @@ static dev_link_t *smc91c92_attach(void)
     client_reg_t client_reg;
     dev_link_t *link;
     struct device *dev;
-    int ret;
+    int i, ret;
 
     DEBUG(0, "smc91c92_attach()\n");
     
@@ -277,7 +298,11 @@ static dev_link_t *smc91c92_attach(void)
     link->io.IOAddrLines = 4;
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
     link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    link->irq.IRQInfo2 = irq_mask;
+    if (irq_list[0] == -1)
+	link->irq.IRQInfo2 = irq_mask;
+    else
+	for (i = 0; i < 4; i++)
+	    link->irq.IRQInfo2 |= 1 << irq_list[i];
     link->irq.Handler = &smc_interrupt;
     link->conf.Attributes = CONF_ENABLE_IRQ;
     link->conf.Vcc = 50;
@@ -305,7 +330,7 @@ static dev_link_t *smc91c92_attach(void)
     dev->open = &smc91c92_open;
     dev->stop = &smc91c92_close;
     dev->tbusy = 1;
-    link->priv = dev;
+    link->priv = link->irq.Instance = dev;
     
     /* Register with Card Services */
     link->next = dev_list;
@@ -320,7 +345,7 @@ static dev_link_t *smc91c92_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	smc91c92_detach(link);
 	return NULL;
     }
@@ -454,9 +479,9 @@ static int mhz_3288_power(dev_link_t *link)
     busy_loop(HZ/5);
     
     /* Now read and write the COR... */
-    tmp = readb(lp->base+link->conf.ConfigBase+CISREG_COR);
+    tmp = readb(lp->base + link->conf.ConfigBase + CISREG_COR);
     udelay(5);
-    writeb(tmp, lp->base+link->conf.ConfigBase+CISREG_COR);
+    writeb(tmp, lp->base + link->conf.ConfigBase + CISREG_COR);
 
     return 0;
 }
@@ -516,9 +541,13 @@ static int mhz_mfc_config(dev_link_t *link)
 	return i;
     lp->base = req.Base;
     mem.CardOffset = mem.Page = 0;
+    if (lp->manfid == MANFID_MOTOROLA)
+	mem.CardOffset = link->conf.ConfigBase;
     i = CardServices(MapMemPage, link->win, &mem);
     
-    if ((i == CS_SUCCESS) && (lp->cardid == PRODID_MEGAHERTZ_EM3288))
+    if ((i == CS_SUCCESS)
+	&& (lp->manfid == MANFID_MEGAHERTZ)
+	&& (lp->cardid == PRODID_MEGAHERTZ_EM3288))
 	mhz_3288_power(link);
     
     return i;
@@ -564,6 +593,67 @@ static int mhz_setup(dev_link_t *link)
     return -1;
 }
 
+/*======================================================================
+
+    Configuration stuff for the Motorola Mariner
+
+    mot_config() writes directly to the Mariner configuration
+    registers because the CIS is just bogus.
+    
+======================================================================*/
+
+static void mot_config(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    struct smc_private *lp = dev->priv;
+    int ioaddr = dev->base_addr;
+    int iouart = link->io.BasePort2;
+    
+    /* Set UART base address and force map with COR bit 1 */
+    writeb(iouart & 0xff,        lp->base + MOT_UART + CISREG_IOBASE_0);
+    writeb((iouart >> 8) & 0xff, lp->base + MOT_UART + CISREG_IOBASE_1);
+    writeb(MOT_NORMAL,           lp->base + MOT_UART + CISREG_COR);
+    
+    /* Set SMC base address and force map with COR bit 1 */
+    writeb(ioaddr & 0xff,        lp->base + MOT_LAN + CISREG_IOBASE_0);
+    writeb((ioaddr >> 8) & 0xff, lp->base + MOT_LAN + CISREG_IOBASE_1);
+    writeb(MOT_NORMAL,           lp->base + MOT_LAN + CISREG_COR);
+
+    /* Wait for things to settle down */
+    busy_loop(HZ/10);
+}
+
+static int mot_setup(dev_link_t *link) {
+    struct device *dev = link->priv;
+    int ioaddr = dev->base_addr;
+    int i, wait, loop;
+    unsigned int addr;
+
+    /* Read Ethernet address from Serial EEPROM */
+    
+    for (i = 0; i < 3; i++) {
+	SMC_SELECT_BANK( 2 );
+	outw(MOT_EEPROM + i, ioaddr + POINTER);
+	SMC_SELECT_BANK( 1 );
+	outw((CTL_RELOAD | CTL_EE_SELECT), ioaddr + CONTROL);
+
+	for (loop = 0; loop < 100; loop++) {
+	    busy_loop(HZ/100);
+	    wait = ((CTL_RELOAD | CTL_STORE) & inw(ioaddr + CONTROL));
+	    if (wait == 0) break;
+	}
+	
+	if (wait)
+	    return -1;
+	
+	addr = inw(ioaddr + GENERAL);
+	dev->dev_addr[2*i]   = addr & 0xff;
+	dev->dev_addr[2*i+1] = (addr >> 8) & 0xff;
+    }
+    
+    return 0;
+}
+
 /*====================================================================*/
 
 static int smc_setup(dev_link_t *link)
@@ -671,13 +761,58 @@ static int osi_setup(dev_link_t *link)
 	dev->dev_addr[i] = buf[i+2];
 
     /* Make sure both functions are powered up */
-    outb(0x03, link->io.BasePort1 + OSITECH_PWRDOWN);
+    set_bits(0x300, link->io.BasePort1 + OSITECH_AUI_PWR);
     /* Now, turn on the interrupt for both card functions */
-    outb(0x03, link->io.BasePort1 + OSITECH_ISR);
+    set_bits(0x300, link->io.BasePort1 + OSITECH_RESET_ISR);
+    DEBUG(2, "AUI/PWR: %4.4x RESET/ISR: %4.4x\n",
+	  inw(link->io.BasePort1 + OSITECH_AUI_PWR),
+	  inw(link->io.BasePort1 + OSITECH_RESET_ISR));
 
     return 0;
 }
 
+/*======================================================================
+
+    This verifies that the chip is some SMC91cXX variant, and returns
+    the revision code if successful.  Otherwise, it returns -ENODEV.
+    
+======================================================================*/
+
+static int check_sig(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    ioaddr_t ioaddr = dev->base_addr;
+    u_short s;
+
+    SMC_SELECT_BANK(1);
+    if (inw(ioaddr + BANK_SELECT) >> 8 != 0x33) {
+	/* Try powering up the chip */
+	outw(0, ioaddr + CONTROL);
+	busy_loop(HZ/18);
+    }
+    /* Check Base Address Register to make sure bus width is OK */
+    s = inw(ioaddr + BASE_ADDR);
+    if ((inw(ioaddr + BANK_SELECT) >> 8 == 0x33) &&
+	((s >> 8) != (s & 0xff))) {
+	SMC_SELECT_BANK(3);
+	s = inw(ioaddr + REVISION);
+	return (s & 0xff);
+    }
+
+    if (link->io.Attributes1 == IO_DATA_PATH_WIDTH_AUTO) {
+	event_callback_args_t args;
+	printk(KERN_INFO "smc91c92_cs: using 8-bit IO window.\n");
+	args.client_data = link;
+	smc91c92_event(CS_EVENT_RESET_PHYSICAL, 0, &args);
+	CardServices(ReleaseIO, link->handle, &link->io);
+	link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
+	CardServices(RequestIO, link->handle, &link->io);
+	smc91c92_event(CS_EVENT_CARD_RESET, 0, &args);
+	return check_sig(link);
+    }
+    return -ENODEV;
+}
+
 /*======================================================================
 
     smc91c92_config() is scheduled to run after a CARD_INSERTION event
@@ -687,7 +822,7 @@ static int osi_setup(dev_link_t *link)
 ======================================================================*/
 
 #define CS_EXIT_TEST(ret, svc, label) \
-if (ret != CS_SUCCESS) { cs_error(svc, ret); goto label; }
+if (ret != CS_SUCCESS) { cs_error(link->handle, svc, ret); goto label; }
 
 static void smc91c92_config(dev_link_t *link)
 {
@@ -697,7 +832,8 @@ static void smc91c92_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     u_short buf[32];
-    int i;
+    char *name;
+    int i, rev;
 
     DEBUG(0, "smc91c92_config(0x%p)\n", link);
     
@@ -723,10 +859,11 @@ static void smc91c92_config(dev_link_t *link)
 
     if (lp->manfid == MANFID_OSITECH)
 	i = osi_config(link);
-    else if ((lp->manfid == MANFID_MEGAHERTZ) &&
-	     ((lp->cardid == PRODID_MEGAHERTZ_VARIOUS) ||
-	      (lp->cardid == PRODID_MEGAHERTZ_EM3288)))
-	i = mhz_mfc_config(link);
+    else if (lp->manfid == MANFID_MOTOROLA
+	     || ((lp->manfid == MANFID_MEGAHERTZ)
+		 && ((lp->cardid == PRODID_MEGAHERTZ_VARIOUS)
+		     || (lp->cardid == PRODID_MEGAHERTZ_EM3288))))
+      i = mhz_mfc_config(link);
     else {
 	link->io.NumPorts1 = 16;
 	link->conf.ConfigIndex = 0x01;
@@ -740,6 +877,9 @@ static void smc91c92_config(dev_link_t *link)
     i = CardServices(RequestConfiguration, link->handle, &link->conf);
     CS_EXIT_TEST(i, RequestConfiguration, config_failed);
     
+    if (lp->manfid == MANFID_MOTOROLA)
+	mot_config(link);
+
     dev->irq = link->irq.AssignedIRQ;
 
     if ((if_port == 1) || (if_port == 2))
@@ -762,6 +902,8 @@ static void smc91c92_config(dev_link_t *link)
     case 0x128: /* For broken Megahertz cards */
     case MANFID_MEGAHERTZ:
 	i = mhz_setup(link); break;
+    case MANFID_MOTOROLA:
+	i = mot_setup(link); break;
     default:
 	printk(KERN_NOTICE "smc91c92_cs: unrecognized card type\n");
 	goto config_undo;
@@ -775,12 +917,21 @@ static void smc91c92_config(dev_link_t *link)
 
     link->dev = &lp->node;
     link->state &= ~DEV_CONFIG_PENDING;
-    
-    printk(KERN_INFO "%s: SMC91c92 at port %#3lx, irq %d, %s port, ",
-	   dev->name, dev->base_addr, dev->irq, if_names[dev->if_port]);
-    for (i = 0; i < 5; i++)
-	printk("%02X:", dev->dev_addr[i]);
-    printk("%02X.\n", dev->dev_addr[i]);
+
+    rev = check_sig(link);
+    name = "???";
+    if (rev > 0)
+	switch (rev >> 4) {
+	case 3: name = "92"; break;
+	case 4: name = "94"; break;
+	case 5: name = "95"; break;
+	case 7: name = "100"; break;
+	}
+    printk(KERN_INFO "%s: smc91c%s: port %#3lx, irq %d, %s port, "
+	   "hw_addr ", dev->name, name, dev->base_addr, dev->irq,
+	   if_names[dev->if_port]); 
+    for (i = 0; i < 6; i++)
+	printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : "\n"));
     
     return;
     
@@ -822,8 +973,6 @@ static void smc91c92_release(u_long arg)
     CardServices(ReleaseIRQ, link->handle, &link->irq);
     if (link->win)
 	CardServices(ReleaseWindow, link->win);
-    if (dev->irq != 0)
-	irq2dev_map[dev->irq] = NULL;
     
     link->state &= ~(DEV_CONFIG | DEV_RELEASE_PENDING);
     if (link->state & DEV_STALE_LINK)
@@ -884,10 +1033,12 @@ static int smc91c92_event(event_t event, int priority,
 		(lp->cardid == PRODID_MEGAHERTZ_EM3288))
 		mhz_3288_power(link);
 	    CardServices(RequestConfiguration, link->handle, &link->conf);
+	    if (lp->manfid == MANFID_MOTOROLA)
+		mot_config(link);
 	    if (lp->manfid == MANFID_OSITECH) {
 		/* Power up the card and enable interrupts */
-		outb(0x03, dev->base_addr - 0x10 + OSITECH_PWRDOWN);
-		outb(0x03, dev->base_addr - 0x10 + OSITECH_ISR);
+		set_bits(0x0300, dev->base_addr-0x10+OSITECH_AUI_PWR);
+		set_bits(0x0300, dev->base_addr-0x10+OSITECH_RESET_ISR);
 	    }
 	    if (link->open) {
 		smc_reset(dev);
@@ -906,26 +1057,33 @@ static int smc91c92_event(event_t event, int priority,
     
 ======================================================================*/
 
+#ifdef PCMCIA_DEBUG
+static void smc_dump(struct device *dev)
+{
+    u_short window, save, ioaddr;
+    ioaddr = dev->base_addr;
+    save = inw(ioaddr + BANK_SELECT);
+    for (window = 0; window < 4; window++) {
+	SMC_SELECT_BANK(window);
+	printk(KERN_DEBUG "%s:  bank %d  %4.4x %4.4x %4.4x %4.4x "
+	       "%4.4x %4.4x %4.4x %4.4x\n", dev->name, window,
+	       inw(ioaddr), inw(ioaddr+2), inw(ioaddr+4),
+	       inw(ioaddr+6), inw(ioaddr+8), inw(ioaddr+10),
+	       inw(ioaddr+12), inw(ioaddr+14));
+    }
+    outw(save, ioaddr + BANK_SELECT);
+}
+#endif
+
 static int smc91c92_open(struct device *dev)
 {
     struct smc_private *lp = (struct smc_private *)dev->priv;
-    int ioaddr = dev->base_addr;
     dev_link_t *link;
 
 #ifdef PCMCIA_DEBUG
     DEBUG(0, "%s: smc91c92_open(%p), ID/Window %4.4x.\n",
-	  dev->name, dev, inw(ioaddr + BANK_SELECT));
-
-    if (pc_debug > 1) {
-	int i, window;
-	for (window = 0; window < 4; window++) {
-	    printk(KERN_DEBUG "%s:  Register bank %d ", dev->name, window);
-	    SMC_SELECT_BANK(window);
-	    for (i = 0; i < 16; i += 2)
-		printk(" %4.4x", inw(ioaddr + i));
-	    printk(".\n");
-	}
-    }
+	  dev->name, dev, inw(dev->base_addr + BANK_SELECT));
+    if (pc_debug > 1) smc_dump(dev);
 #endif
     
     /* Check that the PCMCIA card is still here. */
@@ -934,22 +1092,13 @@ static int smc91c92_open(struct device *dev)
     /* Physical device present signature. */
     if (!DEV_OK(link))
 	return -ENODEV;
-    if (inb(ioaddr + BANK_SELECT + 1) != 0x33) {
-	/* Try powering up the chip */
-	outw(0, ioaddr + CONTROL);
-	/* Wait 55 msec for startup */
-	busy_loop(HZ/18);
-	if (inb(ioaddr + BANK_SELECT + 1) != 0x33) {
-	    printk("smc91c92_cs: Yikes!  Bad chip signature!\n");
-	    return -ENODEV;
-	}
+    if (check_sig(link) < 0) {
+	printk("smc91c92_cs: Yikes!  Bad chip signature!\n");
+	return -ENODEV;
     }
-    
     link->open++;
     MOD_INC_USE_COUNT;
     
-    irq2dev_map[dev->irq] = dev;
-    
     dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
     lp->saved_skb = 0;
     lp->packets_waiting = 0;
@@ -975,10 +1124,10 @@ static int smc91c92_close(struct device *dev)
     /* Shut off all interrupts, and turn off the Tx and Rx sections.
        Don't bother to check for chip present. */
     SMC_SELECT_BANK( 2 );	/* Nominally paranoia, but do no assume... */
-    outb( 0, ioaddr + INT_MASK );
+    outw(0, ioaddr + INTERRUPT);
     SMC_SELECT_BANK( 0 );
-    outb( RCR_CLEAR, ioaddr + RCR );
-    outb( TCR_CLEAR, ioaddr + TCR );
+    mask_bits(0xff00, ioaddr + RCR);
+    mask_bits(0xff00, ioaddr + TCR);
     
     /* Put the chip into power-down mode. */
     SMC_SELECT_BANK( 1 );
@@ -1022,12 +1171,12 @@ static void smc_hardware_send_packet( struct device * dev )
     }
     
     /* There should be a packet slot waiting. */
-    packet_no = inb(ioaddr + PNR_ARR + 1);
+    packet_no = inw(ioaddr + PNR_ARR) >> 8;
     if ( packet_no & 0x80 ) {
 	/* If not, there is a hardware problem!  Likely an ejected card. */
 	printk(KERN_WARNING "%s: 91c92 hardware Tx buffer allocation"
 	       " failed, status %#2.2x.\n", dev->name, packet_no);
-	dev_kfree_skb (skb, FREE_WRITE);
+	DEV_KFREE_SKB (skb);
 	lp->saved_skb = NULL;
 	dev->tbusy = 0;
 	return;
@@ -1037,7 +1186,7 @@ static void smc_hardware_send_packet( struct device * dev )
     lp->stats.tx_bytes += skb->len;
 #endif
     /* The card should use the just-allocated buffer. */
-    outb( packet_no, ioaddr + PNR_ARR );
+    outw( packet_no, ioaddr + PNR_ARR );
     /* point to the beginning of the packet */
     outw( PTR_AUTOINC , ioaddr + POINTER );
     
@@ -1047,7 +1196,7 @@ static void smc_hardware_send_packet( struct device * dev )
 	unsigned char *buf = skb->data;
 	int length = skb->len;	/* The chip will pad to ethernet min length. */
 
-	DEBUG(2, "%s: Trying to xmit packet of length %x.\n",
+	DEBUG(2, "%s: Trying to xmit packet of length %d.\n",
 	      dev->name, length);
 	
 #ifdef USE_32_BIT
@@ -1069,14 +1218,15 @@ static void smc_hardware_send_packet( struct device * dev )
     }
     
     /* Enable the Tx interrupts, both Tx (TxErr) and TxEmpty. */
-    outb(IM_TX_INT | IM_TX_EMPTY_INT | inb(ioaddr + INT_MASK),
-	 ioaddr + INT_MASK);
+    outw(((IM_TX_INT|IM_TX_EMPTY_INT)<<8) |
+	 (inw(ioaddr + INTERRUPT) & 0xff00),
+	 ioaddr + INTERRUPT);
     
     /* The chip does the rest of the work. */
     outw( MC_ENQUEUE , ioaddr + MMU_CMD );
     
     lp->saved_skb = NULL;
-    dev_kfree_skb (skb, FREE_WRITE);
+    DEV_KFREE_SKB (skb);
     dev->trans_start = jiffies;
     dev->tbusy = 0;
     return;
@@ -1089,7 +1239,7 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
     struct smc_private *lp = (struct smc_private *)dev->priv;
     int ioaddr = dev->base_addr;
     unsigned short num_pages;
-    short time_out;
+    short time_out, ir;
     
     /* Transmitter timeout, serious problems. */
     if (dev->tbusy) {
@@ -1098,11 +1248,12 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
 	    return 1;
 	printk(KERN_NOTICE "%s: SMC91c92 transmit timed out, "
 	       "Tx_status %2.2x status %4.4x.\n",
-	       dev->name, inb(ioaddr + 0), inw(ioaddr + 2));
+	       dev->name, inw(ioaddr)&0xff, inw(ioaddr + 2));
 	lp->stats.tx_errors++;
 	smc_reset(dev);
 	dev->trans_start = jiffies;
 	dev->tbusy = 0;
+	lp->saved_skb = NULL;
     }
 
 #if (LINUX_VERSION_CODE < VERSION(2,1,25))
@@ -1135,9 +1286,9 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
     
     num_pages = skb->len >> 8;
     
-    if (num_pages > 7 ) {
+    if (num_pages > 7) {
 	printk(KERN_ERR "%s: Far too big packet error.\n", dev->name);
-	dev_kfree_skb (skb, FREE_WRITE);
+	DEV_KFREE_SKB (skb);
 	lp->saved_skb = NULL;
 	lp->stats.tx_dropped++;
 	return 0;		/* Do not re-queue this packet. */
@@ -1150,15 +1301,18 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
     /* Allocate the memory; send the packet now if we win. */
     outw( MC_ALLOC | num_pages, ioaddr + MMU_CMD );
     for (time_out = MEMORY_WAIT_TIME; time_out >= 0; time_out--) {
-	if ( inb(ioaddr + INTERRUPT) & IM_ALLOC_INT) {
+	ir = inw(ioaddr+INTERRUPT);
+	if (ir & IM_ALLOC_INT) {
 	    /* Acknowledge the interrupt, send the packet. */
-	    outb( IM_ALLOC_INT, ioaddr + INTERRUPT);
+	    outw((ir&0xff00) | IM_ALLOC_INT, ioaddr + INTERRUPT);
 	    smc_hardware_send_packet(dev);	/* Send the packet now.. */
 	    return 0;
 	}
     }
+    
     /* Otherwise defer until the Tx-space-allocated interrupt. */
-    outb(IM_ALLOC_INT | inb(ioaddr + INT_MASK),  ioaddr + INT_MASK);
+    DEBUG(2, "%s: memory allocation deferred.\n", dev->name);
+    outw((IM_ALLOC_INT << 8) | (ir & 0xff00), ioaddr + INTERRUPT);
     
     return 0;
 }
@@ -1173,12 +1327,12 @@ static void smc_tx_err( struct device * dev )
 {
     struct smc_private *lp = (struct smc_private *)dev->priv;
     int ioaddr = dev->base_addr;
-    int saved_packet = inb(ioaddr + PNR_ARR);
+    int saved_packet = inw(ioaddr + PNR_ARR) & 0xff;
     int packet_no = inw(ioaddr + FIFO_PORTS) & 0x7f;
     int tx_status;
     
     /* select this as the packet to read from */
-    outb( packet_no, ioaddr + PNR_ARR );
+    outw( packet_no, ioaddr + PNR_ARR );
     
     /* read the first word from this packet */
     outw( PTR_AUTOINC | PTR_READ | 0, ioaddr + POINTER );
@@ -1204,7 +1358,7 @@ static void smc_tx_err( struct device * dev )
     /* one less packet waiting for me */
     lp->packets_waiting--;
     
-    outb( saved_packet, ioaddr + PNR_ARR );
+    outw( saved_packet, ioaddr + PNR_ARR );
     return;
 }
 
@@ -1245,7 +1399,7 @@ static void smc_eph_irq(struct device *dev)
 		  (CFG_IRQ_SEL_1 | CFG_IRQ_SEL_0) : 0),
 		 ioaddr + CONFIG);
 	    if (lp->manfid == MANFID_OSITECH)
-		outb(OSI_AUI_PWR, ioaddr - 0x10 + OSITECH_AUI_CTL);
+		set_bits(OSI_AUI_PWR, ioaddr - 0x10 + OSITECH_AUI_PWR);
 	}
     }
 
@@ -1257,16 +1411,16 @@ static void smc_eph_irq(struct device *dev)
 
 /*====================================================================*/
     
-static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
+static void smc_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 {
-    struct device *dev = (struct device *)(irq2dev_map[irq]);
+    struct device *dev = (struct device *)DEV_ID;
     struct smc_private *lp;
     int ioaddr;
     unsigned short saved_bank, saved_pointer;
     char bogus_cnt = INTR_WORK;		/* Work we are willing to do. */
-    unsigned char mask;
-    
-    if (dev == NULL)
+    u_short mask, status;
+
+    if ((dev == NULL) || !dev->start)
 	return;
     ioaddr = dev->base_addr;
     
@@ -1298,15 +1452,15 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
     
     SMC_SELECT_BANK(2);
     saved_pointer = inw(ioaddr + POINTER);
-    mask = inb(ioaddr + INT_MASK);
+    mask = inw(ioaddr + INTERRUPT) >> 8;
     /* clear all interrupts */
-    outb( 0, ioaddr + INT_MASK );
+    outw( 0, ioaddr + INTERRUPT );
     
     do { /* read the status flag, and mask it */
-	int status = inb( ioaddr + INTERRUPT );
+	status = inw(ioaddr + INTERRUPT) & 0xff;
 	DEBUG(3, "%s: Status is %#2.2x (mask %#2.2x).\n", dev->name,
 	      status, mask);
-	if ((status & mask) == 0 )
+	if ((status & mask) == 0)
 	    break;
 	
 	if (status & IM_RCV_INT) {
@@ -1315,11 +1469,11 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 	}
 	if (status & IM_TX_INT) {
 	    smc_tx_err(dev);
-	    outb(IM_TX_INT, ioaddr + INTERRUPT );
+	    outw(IM_TX_INT, ioaddr + INTERRUPT);
 	}
 	status &= mask;
 	if (status & IM_TX_EMPTY_INT) {
-	    outb( IM_TX_EMPTY_INT, ioaddr + INTERRUPT );
+	    outw(IM_TX_EMPTY_INT, ioaddr + INTERRUPT);
 	    mask &= ~IM_TX_EMPTY_INT;
 	    lp->stats.tx_packets += lp->packets_waiting;
 	    lp->packets_waiting = 0;
@@ -1339,19 +1493,18 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 	if (status & IM_RX_OVRN_INT) {
 	    lp->stats.rx_errors++;
 	    lp->stats.rx_fifo_errors++;		
-	    outb( IM_RX_OVRN_INT, ioaddr + INTERRUPT );
+	    outw(IM_RX_OVRN_INT, ioaddr + INTERRUPT);
 	}
 	if (status & IM_EPH_INT)
 	    smc_eph_irq(dev);
     } while ( --bogus_cnt);
 
-    DEBUG(3, "  Restoring saved registers mask %2.2x bank"
-	  " %d pointer %d.\n", mask, saved_bank, saved_pointer);
+    DEBUG(3, "  Restoring saved registers mask %2.2x bank %4.4x"
+	  " pointer %4.4x.\n", mask, saved_bank, saved_pointer);
     
     /* restore state register */
-    outb( mask, ioaddr + INT_MASK );
-    outw( saved_pointer, ioaddr + POINTER );
-    
+    outw((mask<<8), ioaddr + INTERRUPT);
+    outw(saved_pointer, ioaddr + POINTER);
     SMC_SELECT_BANK( saved_bank );
 
 #ifdef PCMCIA_DEBUG
@@ -1363,8 +1516,17 @@ irq_done:
     
     if (lp->manfid == MANFID_OSITECH) {
 	/* Retrigger interrupt if needed */
-	outb(0x00, ioaddr - 0x10 + OSITECH_ISR);
-	outb(0x03, ioaddr - 0x10 + OSITECH_ISR);
+	mask_bits(0x00ff, ioaddr-0x10+OSITECH_RESET_ISR);
+	set_bits(0x0300, ioaddr-0x10+OSITECH_RESET_ISR);
+    }
+    if (lp->manfid == MANFID_MOTOROLA) {
+	u_char cor;
+	cor = readb(lp->base + MOT_UART + CISREG_COR);
+	writeb(cor & ~COR_IREQ_ENA, lp->base + MOT_UART + CISREG_COR);
+	writeb(cor, lp->base + MOT_UART + CISREG_COR);
+	cor = readb(lp->base + MOT_LAN + CISREG_COR);
+	writeb(cor & ~COR_IREQ_ENA, lp->base + MOT_LAN + CISREG_COR);
+	writeb(cor, lp->base + MOT_LAN + CISREG_COR);
     }
 #ifdef DOES_NOT_WORK
     if (lp->base != NULL) { /* Megahertz MFC's */
@@ -1397,7 +1559,7 @@ static void smc_rx(struct device *dev)
     rx_status = inw(ioaddr + DATA_1);
     packet_length = inw(ioaddr + DATA_1) & 0x07ff;
 
-    DEBUG(2, "%s: Receive status %4.4x length %4d.\n",
+    DEBUG(2, "%s: Receive status %4.4x length %d.\n",
 	  dev->name, rx_status, packet_length);
     
     if ( !(rx_status & RS_ERRORS )) {		
@@ -1600,26 +1762,29 @@ static void smc_reset(struct device *dev)
 	 (dev->if_port == 2 ? CFG_AUI_SELECT : 0),
 	 ioaddr + CONFIG);
     if (lp->manfid == MANFID_OSITECH)
-	outb(dev->if_port == 2 ? OSI_AUI_PWR : 0,
-	     ioaddr - 0x10 + OSITECH_AUI_CTL);
+	outw((dev->if_port == 2 ? OSI_AUI_PWR : 0) |
+	     (inw(ioaddr-0x10+OSITECH_AUI_PWR) & 0xff00),
+	     ioaddr - 0x10 + OSITECH_AUI_PWR);
     
     /* Fill in the physical address.  The databook is wrong about the order! */
-    for (i = 0; i < 6; i++)
-	outb(dev->dev_addr[i], ioaddr + ADDR0 + i);
+    for (i = 0; i < 6; i += 2)
+	outw((dev->dev_addr[i+1]<<8)+dev->dev_addr[i],
+	     ioaddr + ADDR0 + i);
     
     /* Reset the MMU */
     SMC_SELECT_BANK(2);
     outw(MC_RESET, ioaddr + MMU_CMD );
-    outb(0, ioaddr + INT_MASK );
+    outw(0, ioaddr + INTERRUPT );
     
     /* Re-enable the chip. */
     SMC_SELECT_BANK(0);
-    outw(TCR_ENABLE | TCR_PAD_EN, ioaddr + TCR);
+    outw(TCR_ENABLE | TCR_PAD_EN | TCR_MONCSN, ioaddr + TCR);
     set_rx_mode(dev);
     
     /* Enable interrupts. */
     SMC_SELECT_BANK( 2 );
-    outb( IM_EPH_INT | IM_RX_OVRN_INT | IM_RCV_INT, ioaddr + INT_MASK );
+    outw( (IM_EPH_INT | IM_RX_OVRN_INT | IM_RCV_INT) << 8,
+	  ioaddr + INTERRUPT );
 }
 
 /*====================================================================*/
