@@ -35,7 +35,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"sram_mtd.c 1.21 1997/06/01 05:03:16 (David Hinds)";
+"sram_mtd.c 1.26 1998/01/09 04:19:01 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -65,9 +65,10 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void cs_error(int func, int ret)
+static void cs_error(client_handle_t handle, int func, int ret)
 {
-    CardServices(ReportError, dev_info, (void *)func, (void *)ret);
+    error_info_t err = { func, ret };
+    CardServices(ReportError, handle, &err);
 }
 
 /*======================================================================
@@ -109,7 +110,7 @@ static dev_link_t *sram_attach(void)
     client_reg.event_callback_args.client_data = link;
     ret = CardServices(RegisterClient, &link->handle, &client_reg);
     if (ret != 0) {
-	cs_error(RegisterClient, ret);
+	cs_error(link->handle, RegisterClient, ret);
 	sram_detach(link);
 	return NULL;
     }
@@ -154,7 +155,7 @@ static void sram_detach(dev_link_t *link)
     if (link->handle) {
 	ret = CardServices(DeregisterClient, link->handle);
 	if (ret != CS_SUCCESS)
-	    cs_error(DeregisterClient, ret);
+	    cs_error(link->handle, DeregisterClient, ret);
     }
     
     /* Unlink device structure, free bits */
@@ -171,14 +172,14 @@ static void sram_detach(dev_link_t *link)
     
 ======================================================================*/
 
-static void printk_size(u_long sz)
+static void printk_size(u_int sz)
 {
     if (sz & 0x3ff)
-	printk("%ld bytes", sz);
+	printk("%u bytes", sz);
     else if (sz & 0xfffff)
-	printk("%ld kb", sz >> 10);
+	printk("%u kb", sz >> 10);
     else
-	printk("%ld mb", sz >> 20);
+	printk("%u mb", sz >> 20);
 }
 
 static void sram_config(dev_link_t *link)
@@ -196,9 +197,9 @@ static void sram_config(dev_link_t *link)
     req.Base = NULL; req.Size = WINDOW_SIZE;
     req.AccessSpeed = 0;
     link->win = (window_handle_t)link->handle;
-    ret = CardServices(RequestWindow, &link->win, &req);
+    ret = MTDHelperEntry(MTDRequestWindow, &link->win, &req);
     if (ret != 0) {
-	cs_error(RequestWindow, ret);
+	cs_error(link->handle, RequestWindow, ret);
 	link->state &= ~DEV_CONFIG_PENDING;
 	sram_release((u_long)link);
 	return;
@@ -219,10 +220,10 @@ static void sram_config(dev_link_t *link)
 	    reg.MediaID = (u_long)&dev->region[i];
 	    ret = CardServices(RegisterMTD, link->handle, &reg);
 	    if (ret != 0) break;		
-	    printk(KERN_INFO "sram_mtd: %s at 0x%lx, ",
+	    printk(KERN_INFO "sram_mtd: %s at 0x%x, ",
 		   attr ? "attr" : "common", region.CardOffset);
 	    printk_size(region.RegionSize);
-	    printk(", %ld ns\n", region.AccessSpeed);
+	    printk(", %d ns\n", region.AccessSpeed);
 	    dev->region[i] = region; i++;
 	    ret = CardServices(GetNextRegion, link->handle, &region);
 	}
@@ -246,9 +247,9 @@ static void sram_release(u_long arg)
     DEBUG(0, "sram_release(0x%p)\n", link);
 
     if (link->win) {
-	int ret = CardServices(ReleaseWindow, link->win);
+	int ret = MTDHelperEntry(MTDReleaseWindow, link->win);
 	if (ret != CS_SUCCESS)
-	    cs_error(ReleaseWindow, ret);
+	    cs_error(link->handle, ReleaseWindow, ret);
     }
     dev = link->priv;
     link->state &= ~DEV_CONFIG;
@@ -265,10 +266,10 @@ static int sram_read(dev_link_t *link, char *buf, mtd_request_t *req)
     sram_dev_t *dev = (sram_dev_t *)link->priv;
     region_info_t *region;
     mtd_mod_win_t mod;
-    u_long from, length, nb;
+    u_int from, length, nb;
     int ret;
     
-    DEBUG(1, "sram_read(0x%p, 0x%ld, 0x%p, 0x%lx, 0x%lx)\n", link,
+    DEBUG(1, "sram_read(0x%p, 0x%lx, 0x%p, 0x%x, 0x%x)\n", link,
 	  req->MediaID, buf, req->SrcCardOffset, req->TransferLength);
 
     region = (region_info_t *)(req->MediaID);
@@ -283,7 +284,7 @@ static int sram_read(dev_link_t *link, char *buf, mtd_request_t *req)
     for (length = req->TransferLength; length > 0; length -= nb) {
 	ret = MTDHelperEntry(MTDModifyWindow, link->win, &mod);
 	if (ret != CS_SUCCESS) {
-	    cs_error(MapMemPage, ret);
+	    cs_error(link->handle, MapMemPage, ret);
 	    return ret;
 	}
 	nb = (from+length > WINDOW_SIZE) ? WINDOW_SIZE-from : length;
@@ -307,17 +308,17 @@ static int sram_write(dev_link_t *link, char *buf, mtd_request_t *req)
     sram_dev_t *dev = (sram_dev_t *)link->priv;
     mtd_mod_win_t mod;
     region_info_t *region;
-    u_long from, length, nb;
+    u_int from, length, nb;
     status_t status;
     int ret;
 
-    DEBUG(1, "sram_write(0x%p, 0x%lx, 0x%p, 0x%lx, 0x%lx)\n", link,
+    DEBUG(1, "sram_write(0x%p, 0x%lx, 0x%p, 0x%x, 0x%x)\n", link,
 	  req->MediaID, buf, req->DestCardOffset, req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
     if (ret != 0) {
-	cs_error(GetStatus, ret);
+	cs_error(link->handle, GetStatus, ret);
 	return CS_GENERAL_FAILURE;
     }
     if (status.CardState & CS_EVENT_WRITE_PROTECT)
@@ -335,7 +336,7 @@ static int sram_write(dev_link_t *link, char *buf, mtd_request_t *req)
     for (length = req->TransferLength ; length > 0; length -= nb) {
 	ret = MTDHelperEntry(MTDModifyWindow, link->win, &mod);
 	if (ret != CS_SUCCESS) {
-	    cs_error(MapMemPage, ret);
+	    cs_error(link->handle, MapMemPage, ret);
 	    return ret;
 	}
 	nb = (from+length > WINDOW_SIZE) ? WINDOW_SIZE-from : length;
@@ -357,7 +358,7 @@ static int sram_write(dev_link_t *link, char *buf, mtd_request_t *req)
 #if 0
 static int sram_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 {
-    DEBUG(1, "sram_erase(0x%p, 0x%lx, 0x%p, 0x%lx, 0x%lx)\n", link,
+    DEBUG(1, "sram_erase(0x%p, 0x%lx, 0x%p, 0x%x, 0x%x)\n", link,
 	  req->MediaID, buf, req->DestCardOffset, req->TransferLength);
 
     if (req->Function & MTD_REQ_TIMEOUT) {
