@@ -35,7 +35,7 @@
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"tcic.c 1.82 1997/06/01 05:03:16 (David Hinds)";
+"tcic.c 1.86 1997/12/29 17:12:06 (David Hinds)";
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 #else
 #define DEBUG(n, args...)
@@ -55,7 +55,8 @@ static int ignore = -1;
 static int do_scan = 1;
 
 /* Bit map of interrupts to choose from */
-static u_long irq_mask = 0xffff;
+static u_int irq_mask = 0xffff;
+static int irq_list[16] = { -1 };
 
 /* The card status change interrupt -- 0 means autoselect */
 static int cs_irq = 0;
@@ -76,6 +77,7 @@ MODULE_PARM(tcic_base, "i");
 MODULE_PARM(ignore, "i");
 MODULE_PARM(do_scan, "i");
 MODULE_PARM(irq_mask, "i");
+MODULE_PARM(irq_list, "1-16i");
 MODULE_PARM(cs_irq, "i");
 MODULE_PARM(poll_interval, "i");
 MODULE_PARM(poll_quick, "i");
@@ -90,7 +92,7 @@ static int tcic_service(u_int sock, u_int cmd, void *arg);
 
 typedef struct socket_info_t {
     u_short	psock;
-    void	(*handler)(void *info, u_long events);
+    void	(*handler)(void *info, u_int events);
     void	*info;
     u_char	last_sstat;
     u_char	id;
@@ -141,7 +143,7 @@ static void tcic_setw(u_char reg, u_short data)
 #define tcic_setw(reg, data) outw(data, tcic_base+reg)
 #endif
 
-static void tcic_setl(u_char reg, u_long data)
+static void tcic_setl(u_char reg, u_int data)
 {
 #ifdef PCMCIA_DEBUG_X
     printk(KERN_DEBUG "tcic_setl(%#x, %#lx)\n", tcic_base+reg, data);
@@ -197,7 +199,7 @@ static int to_ns(int cycles)
 
 /*====================================================================*/
 
-static void busy_loop(u_long len)
+static void busy_loop(u_int len)
 {
     u_long flags, timeout = jiffies + len;
     save_flags(flags);
@@ -209,14 +211,14 @@ static void busy_loop(u_long len)
 
 /*====================================================================*/
 
-static volatile u_long irq_hits;
+static volatile u_int irq_hits;
 
 static void irq_count IRQ(int irq, void *dev, struct pt_regs *regs)
 {
     irq_hits++;
 }
 
-static u_long try_irq(int irq)
+static u_int try_irq(int irq)
 {
     u_short cfg;
 
@@ -247,10 +249,10 @@ static u_long try_irq(int irq)
     return (irq_hits != 1);
 }
 
-static u_long irq_scan(u_long mask0)
+static u_int irq_scan(u_int mask0)
 {
-    u_long mask1, mask2;
-    int i, ni;
+    u_int mask1;
+    int i;
 
 #ifdef __alpha__
 #define PIC 0x4d0
@@ -259,17 +261,38 @@ static u_long irq_scan(u_long mask0)
     if (level_mask)
 	mask0 &= ~level_mask;
 #endif
+
+    mask1 = 0;
+    if (do_scan) {
+	for (i = 0; i < 16; i++)
+	    if ((mask0 & (1 << i)) && (try_irq(i) == 0))
+		mask1 |= (1 << i);
+	for (i = 0; i < 16; i++)
+	    if ((mask1 & (1 << i)) && (try_irq(i) != 0)) {
+		mask1 ^= (1 << i);
+	    }
+    }
     
-    mask1 = mask2 = ni = 0;
+    if (mask1) {
+	printk("scanned");
+    } else {
+	/* Fallback: just find interrupts that aren't in use */
+	for (i = 0; i < 16; i++)
+	    if ((mask0 & (1 << i)) &&
+		(REQUEST_IRQ(i, irq_count, 0, "x", NULL) == 0)) {
+		mask1 |= (1 << i);
+		FREE_IRQ(i, NULL);
+	    }
+	printk("default");
+    }
+    
+    printk(") = ");
     for (i = 0; i < 16; i++)
-	if ((mask0 & (1 << i)) && (try_irq(i) == 0))
-	    mask1 |= (1 << i);
-    for (i = 0; i < 16; i++)
-	if ((mask0 & (1 << i)) && (try_irq(i) == 0)) {
-	    mask2 |= (1 << i);
-	    ni++;
-	}
-    return (mask1 & mask2);
+	if (mask1 & (1<<i))
+	    printk("%s%d", ((mask1 & ((1<<i)-1)) ? "," : ""), i);
+    printk(" ");
+    
+    return mask1;
 }
 
 /*======================================================================
@@ -285,7 +308,7 @@ static int is_active(int s)
 {
     u_short scf1, ioctl, base, num;
     u_char pwr, sstat;
-    u_long addr;
+    u_int addr;
     
     tcic_setl(TCIC_ADDR, (s << TCIC_ADDR_SS_SHFT)
 	      | TCIC_ADDR_INDREG | TCIC_SCF1(s));
@@ -335,7 +358,7 @@ static int get_tcic_id(void)
 int tcic_init(void)
 {
     int i, sock;
-    u_long mask, scan;
+    u_int mask, scan;
 
     DEBUG(0, "%s\n", version);
     
@@ -404,15 +427,16 @@ int tcic_init(void)
     poll_timer.prev = poll_timer.next = NULL;
 
     /* Build interrupt mask */
-    printk(", %d sockets\n" KERN_INFO "  irq mask (", sockets);
-    mask = tcic_cap.irq_mask & irq_mask;
-    if (do_scan && (scan = irq_scan(mask))) {
-	mask = scan;
-	printk("scanned");
-    }
+    printk(", %d sockets\n" KERN_INFO "  irq list (", sockets);
+    if (irq_list[0] == -1)
+	mask = irq_mask;
     else
-	printk("default");
-    printk(") = 0x%lx, ", mask);
+	for (i = mask = 0; i < 16; i++)
+	    mask |= (1<<irq_list[i]);
+    mask &= tcic_cap.irq_mask;
+
+    /* Scan interrupts */
+    mask = irq_scan(mask);
     tcic_cap.irq_mask = mask;
     
     /* Check for only two interrupts available */
@@ -427,8 +451,7 @@ int tcic_init(void)
 		    (REQUEST_IRQ(i, tcic_interrupt, 0, "tcic", NULL) == 0))
 		    break;
 	    cs_irq = i;
-	}
-	else {
+	} else {
 	    if (REQUEST_IRQ(cs_irq, tcic_interrupt, 0, "tcic", NULL) == 0)
 		cs_irq = 0;
 	}
@@ -440,7 +463,8 @@ int tcic_init(void)
     if (cs_irq != 0)
 	printk("status change on irq %d\n", cs_irq);
     else
-	printk("polled status mode\n");
+	printk("polled status, interval = %d ms\n",
+	       poll_interval * 1000 / HZ);
     
     for (i = 0; i < sockets; i++) {
 	tcic_setw(TCIC_ADDR+2, socket_table[i].psock << TCIC_SS_SHFT);
@@ -466,7 +490,7 @@ int tcic_init(void)
 
 static void tcic_finish(void)
 {
-    unsigned long flags;
+    u_long flags;
     unregister_ss_entry(&tcic_service);
     save_flags(flags);
     cli();
@@ -487,7 +511,7 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
     int i, quick = 0;
     u_char latch, sstat;
     u_short psock;
-    u_long events;
+    u_int events;
     static volatile int active = 0;
 
     if (active) {
@@ -516,8 +540,7 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
 	events |= (latch & TCIC_SSTAT_WP) ? SS_WRPROT : 0;
 	if (tcic_getw(TCIC_DATA) & TCIC_SCF1_IOSTS) {
 	    events |= (latch & TCIC_SSTAT_LBAT1) ? SS_STSCHG : 0;
-	}
-	else {
+	} else {
 	    events |= (latch & TCIC_SSTAT_RDY) ? SS_READY : 0;
 	    events |= (latch & TCIC_SSTAT_LBAT1) ? SS_BATDEAD : 0;
 	    events |= (latch & TCIC_SSTAT_LBAT2) ? SS_BATWARN : 0;
@@ -538,7 +561,7 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
     
 } /* tcic_interrupt */
 
-void tcic_timer(u_long data)
+static void tcic_timer(u_long data)
 {
     DEBUG(2, "tcic: tcic_timer()\n");
     timer_pending = 0;
@@ -552,8 +575,7 @@ static int tcic_register_callback(u_short lsock, ss_callback_t *call)
     if (call == NULL) {
 	socket_table[lsock].handler = NULL;
 	MOD_DEC_USE_COUNT;
-    }
-    else {
+    } else {
 	MOD_INC_USE_COUNT;
 	socket_table[lsock].handler = call->handler;
 	socket_table[lsock].info = call->info;
@@ -575,8 +597,7 @@ static int tcic_get_status(u_short lsock, u_int *value)
     *value |= (reg & TCIC_SSTAT_WP) ? SS_WRPROT : 0;
     if (tcic_getw(TCIC_DATA) & TCIC_SCF1_IOSTS) {
 	*value |= (reg & TCIC_SSTAT_LBAT1) ? SS_STSCHG : 0;
-    }
-    else {
+    } else {
 	*value |= (reg & TCIC_SSTAT_RDY) ? SS_READY : 0;
 	*value |= (reg & TCIC_SSTAT_LBAT1) ? SS_BATDEAD : 0;
 	*value |= (reg & TCIC_SSTAT_LBAT2) ? SS_BATWARN : 0;
@@ -636,15 +657,14 @@ static int tcic_get_socket(u_short lsock, socket_state_t *state)
     state->csc_mask = (scf2 & TCIC_SCF2_MCD) ? 0 : SS_DETECT;
     if (state->flags & SS_IOCARD) {
 	state->csc_mask |= (scf2 & TCIC_SCF2_MLBAT1) ? 0 : SS_STSCHG;
-    }
-    else {
+    } else {
 	state->csc_mask |= (scf2 & TCIC_SCF2_MLBAT1) ? 0 : SS_BATDEAD;
 	state->csc_mask |= (scf2 & TCIC_SCF2_MLBAT2) ? 0 : SS_BATWARN;
 	state->csc_mask |= (scf2 & TCIC_SCF2_MRDY) ? 0 : SS_READY;
     }
 
-    DEBUG(1, "tcic: GetSocket(%d) = flags %#3.3lx, Vcc %d, Vpp %d, "
-	  "io_irq %d, csc_mask %#2.2lx\n", lsock, state->flags,
+    DEBUG(1, "tcic: GetSocket(%d) = flags %#3.3x, Vcc %d, Vpp %d, "
+	  "io_irq %d, csc_mask %#2.2x\n", lsock, state->flags,
 	  state->Vcc, state->Vpp, state->io_irq, state->csc_mask);
     return 0;
 } /* tcic_get_socket */
@@ -657,8 +677,8 @@ static int tcic_set_socket(u_short lsock, socket_state_t *state)
     u_char reg;
     u_short scf1, scf2;
 
-    DEBUG(1, "tcic: SetSocket(%d, flags %#3.3lx, Vcc %d, Vpp %d, "
-	  "io_irq %d, csc_mask %#2.2lx)\n", lsock, state->flags,
+    DEBUG(1, "tcic: SetSocket(%d, flags %#3.3x, Vcc %d, Vpp %d, "
+	  "io_irq %d, csc_mask %#2.2x)\n", lsock, state->flags,
 	  state->Vcc, state->Vpp, state->io_irq, state->csc_mask);
     tcic_setw(TCIC_ADDR+2, (psock << TCIC_SS_SHFT) | TCIC_ADR2_INDREG);
 
@@ -717,8 +737,7 @@ static int tcic_set_socket(u_short lsock, socket_state_t *state)
     if (state->csc_mask & SS_DETECT) scf2 &= ~TCIC_SCF2_MCD;
     if (state->flags & SS_IOCARD) {
 	if (state->csc_mask & SS_STSCHG) reg &= ~TCIC_SCF2_MLBAT1;
-    }
-    else {
+    } else {
 	if (state->csc_mask & SS_BATDEAD) reg &= ~TCIC_SCF2_MLBAT1;
 	if (state->csc_mask & SS_BATWARN) reg &= ~TCIC_SCF2_MLBAT2;
 	if (state->csc_mask & SS_READY) reg &= ~TCIC_SCF2_MRDY;
@@ -736,7 +755,7 @@ static int tcic_get_io_map(u_short lsock, struct pcmcia_io_map *io)
 {
     u_short psock = socket_table[lsock].psock;
     u_short base, ioctl;
-    u_long addr;
+    u_int addr;
     
     if (io->map > 1) return -EINVAL;
     tcic_setw(TCIC_ADDR+2, TCIC_ADR2_INDREG | (psock << TCIC_SS_SHFT));
@@ -773,7 +792,7 @@ static int tcic_get_io_map(u_short lsock, struct pcmcia_io_map *io)
 static int tcic_set_io_map(u_short lsock, struct pcmcia_io_map *io)
 {
     u_short psock = socket_table[lsock].psock;
-    u_long addr;
+    u_int addr;
     u_short base, len, ioctl;
     
     DEBUG(1, "tcic: SetIOMap(%d, %d, %#2.2x, %d ns, "
@@ -820,8 +839,7 @@ static int tcic_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     if (base & TCIC_MBASE_4K_BIT) {
 	mem->sys_start = base & TCIC_MBASE_HA_MASK;
 	mem->sys_stop = mem->sys_start;
-    }
-    else {
+    } else {
 	base &= TCIC_MBASE_HA_MASK;
 	mem->sys_start = (base & (base-1));
 	mem->sys_stop = mem->sys_start + (base ^ (base-1));
@@ -844,7 +862,7 @@ static int tcic_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     mem->speed = to_ns(ctl & TCIC_MCTL_WSCNT_MASK);
     
     DEBUG(1, "tcic: GetMemMap(%d, %d) = %#2.2x, %d ns, "
-	  "%#5.5lx-%#5.5lx, %#5.5lx\n", lsock, mem->map, mem->flags,
+	  "%#5.5lx-%#5.5lx, %#5.5x\n", lsock, mem->map, mem->flags,
 	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     return 0;
 } /* tcic_get_mem_map */
@@ -858,7 +876,7 @@ static int tcic_set_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     u_long base, len, mmap;
 
     DEBUG(1, "tcic: SetMemMap(%d, %d, %#2.2x, %d ns, "
-	  "%#5.5lx-%#5.5lx, %#5.5lx)\n", lsock, mem->map, mem->flags,
+	  "%#5.5lx-%#5.5lx, %#5.5x)\n", lsock, mem->map, mem->flags,
 	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     if ((mem->map > 3) || (mem->card_start > 0x3ffffff) ||
 	(mem->sys_start > 0xffffff) || (mem->sys_stop > 0xffffff) ||
@@ -899,8 +917,8 @@ typedef int (*subfn_t)(u_short, void *);
     
 static subfn_t service_table[] = {
     (subfn_t)&tcic_register_callback,
-    (subfn_t)&tcic_get_status,
     (subfn_t)&tcic_inquire_socket,
+    (subfn_t)&tcic_get_status,
     (subfn_t)&tcic_get_socket,
     (subfn_t)&tcic_set_socket,
     (subfn_t)&tcic_get_io_map,
