@@ -74,10 +74,11 @@
  * like DEC RoamAbout, or Digital Ocean, Epson, ...), you must modify this
  * part to accomodate your hardware...
  */
-const char	MAC_ADDRESSES[][3] =
+const unsigned char	MAC_ADDRESSES[][3] =
 {
   { 0x08, 0x00, 0x0E },		/* AT&T Wavelan (standard) & DEC RoamAbout */
   { 0x08, 0x00, 0x6A },		/* AT&T Wavelan (alternate) */
+  { 0x00, 0x00, 0xE1 }		/* Hitachi Wavelan */
   /* Add your card here and send me the patch ! */
 };
 
