@@ -467,8 +467,9 @@ static inline unsigned long	/* flags */
 	wv_splhi(void);		/* Disable interrupts */
 static inline void
 	wv_splx(unsigned long);	/* ReEnable interrupts : flags */
-static inline void
-	cs_error(int,		/* Report error to cardmgr */
+static void
+	cs_error(client_handle_t, /* Report error to cardmgr */
+		 int,
 		 int);
 /* ----------------- MODEM MANAGEMENT SUBROUTINES ----------------- */
 static inline u_char		/* data */
@@ -605,15 +606,15 @@ static volatile int	wv_wait_completed = 0;
 
 /* Bit map of interrupts to choose from */
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4 and 3 */
-static const u_long	irq_mask = 0xdeb8;
+static u_long	irq_mask = 0xdeb8;
+static int 	irq_list[4] = { -1 };
 
 /* Shared memory speed, in ns */
-static const int	mem_speed = 0;
+static int	mem_speed = 0;
 
 /* New module interface */
-#if (LINUX_VERSION_CODE > VERSION(2,1,17))
 MODULE_PARM(irq_mask, "1l");
+MODULE_PARM(irq_list, "1-4i");
 MODULE_PARM(mem_speed, "1i");
-#endif	/* > 2.1.17 */
 
 #endif	/* WAVELAN_CS_H */
